@@ -17,13 +17,13 @@
 
 // written by Sascha Steinbiss <sascha@steinbiss.name>
 
-use crate::mqtt::mqtt::{MQTTTransaction, MQTTState};
+use crate::mqtt::mqtt::{MQTTState, MQTTTransaction};
 use crate::mqtt::mqtt_message::{MQTTOperation, MQTTTypeCode};
 use std::ffi::CStr;
 use std::ptr;
 use std::str::FromStr;
 
-#[derive(FromPrimitive, Debug, Copy, Clone, PartialOrd, PartialEq)]
+#[derive(FromPrimitive, Debug, Copy, Clone, PartialOrd, PartialEq, Eq)]
 #[allow(non_camel_case_types)]
 #[repr(u8)]
 pub enum MQTTFlagState {
@@ -33,31 +33,24 @@ pub enum MQTTFlagState {
 }
 
 #[inline]
-fn check_flag_state(
-    flag_state: MQTTFlagState,
-    flag_value: bool,
-    ok: &mut bool,
-) {
+fn check_flag_state(flag_state: MQTTFlagState, flag_value: bool, ok: &mut bool) {
     match flag_state {
         MQTTFlagState::MQTT_MUST_BE_SET => {
             if !flag_value {
                 *ok = false;
             }
-        },
+        }
         MQTTFlagState::MQTT_CANT_BE_SET => {
             if flag_value {
                 *ok = false;
             }
-        },
+        }
         _ => {}
     }
 }
 
 #[no_mangle]
-pub extern "C" fn rs_mqtt_tx_has_type(
-    tx: &MQTTTransaction,
-    mtype: u8,
-) -> u8 {
+pub extern "C" fn rs_mqtt_tx_has_type(tx: &MQTTTransaction, mtype: u8) -> u8 {
     for msg in tx.msg.iter() {
         if mtype == msg.header.message_type as u8 {
             return 1;
@@ -67,15 +60,13 @@ pub extern "C" fn rs_mqtt_tx_has_type(
 }
 
 #[no_mangle]
-pub extern "C" fn rs_mqtt_cstr_message_code(
+pub unsafe extern "C" fn rs_mqtt_cstr_message_code(
     str: *const std::os::raw::c_char,
 ) -> std::os::raw::c_int {
-    unsafe {
-        let msgtype: &CStr = CStr::from_ptr(str);
-        if let Ok(s) = msgtype.to_str() {
-            if let Ok(x) = MQTTTypeCode::from_str(s) {
-                return x as i32;
-            }
+    let msgtype: &CStr = CStr::from_ptr(str);
+    if let Ok(s) = msgtype.to_str() {
+        if let Ok(x) = MQTTTypeCode::from_str(s) {
+            return x as i32;
         }
     }
     return -1;
@@ -83,9 +74,7 @@ pub extern "C" fn rs_mqtt_cstr_message_code(
 
 #[no_mangle]
 pub extern "C" fn rs_mqtt_tx_has_flags(
-    tx: &MQTTTransaction,
-    qretain: MQTTFlagState,
-    qdup: MQTTFlagState,
+    tx: &MQTTTransaction, qretain: MQTTFlagState, qdup: MQTTFlagState,
 ) -> u8 {
     for msg in tx.msg.iter() {
         let mut ok = true;
@@ -100,10 +89,7 @@ pub extern "C" fn rs_mqtt_tx_has_flags(
 }
 
 #[no_mangle]
-pub extern "C" fn rs_mqtt_tx_has_qos(
-    tx: &MQTTTransaction,
-    qos: u8,
-) -> u8 {
+pub extern "C" fn rs_mqtt_tx_has_qos(tx: &MQTTTransaction, qos: u8) -> u8 {
     for msg in tx.msg.iter() {
         if qos == msg.header.qos_level {
             return 1;
@@ -113,20 +99,14 @@ pub extern "C" fn rs_mqtt_tx_has_qos(
 }
 
 #[no_mangle]
-pub extern "C" fn rs_mqtt_tx_get_protocol_version(
-    state: &MQTTState,
-) -> u8 {
+pub extern "C" fn rs_mqtt_tx_get_protocol_version(state: &MQTTState) -> u8 {
     return state.protocol_version;
 }
 
 #[no_mangle]
 pub extern "C" fn rs_mqtt_tx_has_connect_flags(
-    tx: &MQTTTransaction,
-    username: MQTTFlagState,
-    password: MQTTFlagState,
-    will: MQTTFlagState,
-    will_retain: MQTTFlagState,
-    clean_session: MQTTFlagState,
+    tx: &MQTTTransaction, username: MQTTFlagState, password: MQTTFlagState, will: MQTTFlagState,
+    will_retain: MQTTFlagState, clean_session: MQTTFlagState,
 ) -> u8 {
     for msg in tx.msg.iter() {
         if let MQTTOperation::CONNECT(ref cv) = msg.op {
@@ -145,152 +125,119 @@ pub extern "C" fn rs_mqtt_tx_has_connect_flags(
 }
 
 #[no_mangle]
-pub extern "C" fn rs_mqtt_tx_get_connect_clientid(
-    tx: &MQTTTransaction,
-    buffer: *mut *const u8,
-    buffer_len: *mut u32,
+pub unsafe extern "C" fn rs_mqtt_tx_get_connect_clientid(
+    tx: &MQTTTransaction, buffer: *mut *const u8, buffer_len: *mut u32,
 ) -> u8 {
     for msg in tx.msg.iter() {
         if let MQTTOperation::CONNECT(ref cv) = msg.op {
             let p = &cv.client_id;
-            if p.len() > 0 {
-                unsafe {
-                    *buffer = p.as_ptr();
-                    *buffer_len = p.len() as u32;
-                }
+            if !p.is_empty() {
+                *buffer = p.as_ptr();
+                *buffer_len = p.len() as u32;
                 return 1;
             }
         }
     }
 
-    unsafe {
-        *buffer = ptr::null();
-        *buffer_len = 0;
-    }
+    *buffer = ptr::null();
+    *buffer_len = 0;
     return 0;
 }
 
 #[no_mangle]
-pub extern "C" fn rs_mqtt_tx_get_connect_username(
-    tx: &MQTTTransaction,
-    buffer: *mut *const u8,
-    buffer_len: *mut u32,
+pub unsafe extern "C" fn rs_mqtt_tx_get_connect_username(
+    tx: &MQTTTransaction, buffer: *mut *const u8, buffer_len: *mut u32,
 ) -> u8 {
     for msg in tx.msg.iter() {
         if let MQTTOperation::CONNECT(ref cv) = msg.op {
             if let Some(p) = &cv.username {
-                if p.len() > 0 {
-                    unsafe {
-                        *buffer = p.as_ptr();
-                        *buffer_len = p.len() as u32;
-                    }
+                if !p.is_empty() {
+                    *buffer = p.as_ptr();
+                    *buffer_len = p.len() as u32;
                     return 1;
                 }
             }
         }
     }
 
-    unsafe {
-        *buffer = ptr::null();
-        *buffer_len = 0;
-    }
+    *buffer = ptr::null();
+    *buffer_len = 0;
 
     return 0;
 }
 
 #[no_mangle]
-pub extern "C" fn rs_mqtt_tx_get_connect_password(
-    tx: &MQTTTransaction,
-    buffer: *mut *const u8,
-    buffer_len: *mut u32,
+pub unsafe extern "C" fn rs_mqtt_tx_get_connect_password(
+    tx: &MQTTTransaction, buffer: *mut *const u8, buffer_len: *mut u32,
 ) -> u8 {
     for msg in tx.msg.iter() {
         if let MQTTOperation::CONNECT(ref cv) = msg.op {
             if let Some(p) = &cv.password {
-                if p.len() > 0 {
-                    unsafe {
-                        *buffer = p.as_ptr();
-                        *buffer_len = p.len() as u32;
-                    }
+                if !p.is_empty() {
+                    *buffer = p.as_ptr();
+                    *buffer_len = p.len() as u32;
                     return 1;
                 }
             }
         }
     }
 
-    unsafe {
-        *buffer = ptr::null();
-        *buffer_len = 0;
-    }
+    *buffer = ptr::null();
+    *buffer_len = 0;
     return 0;
 }
 
 #[no_mangle]
-pub extern "C" fn rs_mqtt_tx_get_connect_willtopic(
-    tx: &MQTTTransaction,
-    buffer: *mut *const u8,
-    buffer_len: *mut u32,
+pub unsafe extern "C" fn rs_mqtt_tx_get_connect_willtopic(
+    tx: &MQTTTransaction, buffer: *mut *const u8, buffer_len: *mut u32,
 ) -> u8 {
     for msg in tx.msg.iter() {
         if let MQTTOperation::CONNECT(ref cv) = msg.op {
             if let Some(p) = &cv.will_topic {
-                if p.len() > 0 {
-                    unsafe {
-                        *buffer = p.as_ptr();
-                        *buffer_len = p.len() as u32;
-                    }
+                if !p.is_empty() {
+                    *buffer = p.as_ptr();
+                    *buffer_len = p.len() as u32;
                     return 1;
                 }
             }
         }
     }
 
-    unsafe {
-        *buffer = ptr::null();
-        *buffer_len = 0;
-    }
+    *buffer = ptr::null();
+    *buffer_len = 0;
 
     return 0;
 }
 
 #[no_mangle]
-pub extern "C" fn rs_mqtt_tx_get_connect_willmessage(
-    tx: &MQTTTransaction,
-    buffer: *mut *const u8,
-    buffer_len: *mut u32,
+pub unsafe extern "C" fn rs_mqtt_tx_get_connect_willmessage(
+    tx: &MQTTTransaction, buffer: *mut *const u8, buffer_len: *mut u32,
 ) -> u8 {
     for msg in tx.msg.iter() {
         if let MQTTOperation::CONNECT(ref cv) = msg.op {
             if let Some(p) = &cv.will_message {
-                if p.len() > 0 {
-                    unsafe {
-                        *buffer = p.as_ptr();
-                        *buffer_len = p.len() as u32;
-                    }
+                if !p.is_empty() {
+                    *buffer = p.as_ptr();
+                    *buffer_len = p.len() as u32;
                     return 1;
                 }
             }
         }
     }
 
-    unsafe {
-        *buffer = ptr::null();
-        *buffer_len = 0;
-    }
+    *buffer = ptr::null();
+    *buffer_len = 0;
 
     return 0;
 }
 
 #[no_mangle]
-pub extern "C" fn rs_mqtt_tx_get_connack_sessionpresent(
-    tx: &MQTTTransaction,
-    session_present: *mut bool,
+pub unsafe extern "C" fn rs_mqtt_tx_get_connack_sessionpresent(
+    tx: &MQTTTransaction, session_present: *mut bool,
 ) -> u8 {
     for msg in tx.msg.iter() {
         if let MQTTOperation::CONNACK(ref ca) = msg.op {
-            unsafe {
-                *session_present = ca.session_present;
-            }
+            *session_present = ca.session_present;
             return 1;
         }
     }
@@ -298,76 +245,59 @@ pub extern "C" fn rs_mqtt_tx_get_connack_sessionpresent(
 }
 
 #[no_mangle]
-pub extern "C" fn rs_mqtt_tx_get_publish_topic(
-    tx: &MQTTTransaction,
-    buffer: *mut *const u8,
-    buffer_len: *mut u32,
+pub unsafe extern "C" fn rs_mqtt_tx_get_publish_topic(
+    tx: &MQTTTransaction, buffer: *mut *const u8, buffer_len: *mut u32,
 ) -> u8 {
     for msg in tx.msg.iter() {
         if let MQTTOperation::PUBLISH(ref pubv) = msg.op {
             let p = &pubv.topic;
-            if p.len() > 0 {
-                unsafe {
-                    *buffer = p.as_ptr();
-                    *buffer_len = p.len() as u32;
-                }
+            if !p.is_empty() {
+                *buffer = p.as_ptr();
+                *buffer_len = p.len() as u32;
                 return 1;
             }
         }
     }
 
-    unsafe {
-        *buffer = ptr::null();
-        *buffer_len = 0;
-    }
+    *buffer = ptr::null();
+    *buffer_len = 0;
 
     return 0;
 }
 
 #[no_mangle]
-pub extern "C" fn rs_mqtt_tx_get_publish_message(
-    tx: &MQTTTransaction,
-    buffer: *mut *const u8,
-    buffer_len: *mut u32,
+pub unsafe extern "C" fn rs_mqtt_tx_get_publish_message(
+    tx: &MQTTTransaction, buffer: *mut *const u8, buffer_len: *mut u32,
 ) -> u8 {
     for msg in tx.msg.iter() {
         if let MQTTOperation::PUBLISH(ref pubv) = msg.op {
             let p = &pubv.message;
-            if p.len() > 0 {
-                unsafe {
-                    *buffer = p.as_ptr();
-                    *buffer_len = p.len() as u32;
-                }
+            if !p.is_empty() {
+                *buffer = p.as_ptr();
+                *buffer_len = p.len() as u32;
                 return 1;
             }
         }
     }
 
-    unsafe {
-        *buffer = ptr::null();
-        *buffer_len = 0;
-    }
+    *buffer = ptr::null();
+    *buffer_len = 0;
 
     return 0;
 }
 
 #[no_mangle]
-pub extern "C" fn rs_mqtt_tx_get_subscribe_topic(tx: &MQTTTransaction,
-    i: u32,
-    buf: *mut *const u8,
-    len: *mut u32)
-    -> u8
-{
+pub unsafe extern "C" fn rs_mqtt_tx_get_subscribe_topic(
+    tx: &MQTTTransaction, i: u32, buf: *mut *const u8, len: *mut u32,
+) -> u8 {
     let mut offset = 0;
     for msg in tx.msg.iter() {
         if let MQTTOperation::SUBSCRIBE(ref subv) = msg.op {
             if (i as usize) < subv.topics.len() + offset {
                 let topic = &subv.topics[(i as usize) - offset];
-                if topic.topic_name.len() > 0 {
-                    unsafe {
-                        *len = topic.topic_name.len() as u32;
-                        *buf = topic.topic_name.as_ptr();
-                    }
+                if !topic.topic_name.is_empty() {
+                    *len = topic.topic_name.len() as u32;
+                    *buf = topic.topic_name.as_ptr();
                     return 1;
                 }
             } else {
@@ -376,31 +306,24 @@ pub extern "C" fn rs_mqtt_tx_get_subscribe_topic(tx: &MQTTTransaction,
         }
     }
 
-    unsafe {
-        *buf = ptr::null();
-        *len = 0;
-    }
+    *buf = ptr::null();
+    *len = 0;
 
     return 0;
 }
 
 #[no_mangle]
-pub extern "C" fn rs_mqtt_tx_get_unsubscribe_topic(tx: &MQTTTransaction,
-    i: u32,
-    buf: *mut *const u8,
-    len: *mut u32)
-    -> u8
-{
+pub unsafe extern "C" fn rs_mqtt_tx_get_unsubscribe_topic(
+    tx: &MQTTTransaction, i: u32, buf: *mut *const u8, len: *mut u32,
+) -> u8 {
     let mut offset = 0;
     for msg in tx.msg.iter() {
         if let MQTTOperation::UNSUBSCRIBE(ref unsubv) = msg.op {
             if (i as usize) < unsubv.topics.len() + offset {
                 let topic = &unsubv.topics[(i as usize) - offset];
-                if topic.len() > 0 {
-                    unsafe {
-                        *len = topic.len() as u32;
-                        *buf = topic.as_ptr();
-                    }
+                if !topic.is_empty() {
+                    *len = topic.len() as u32;
+                    *buf = topic.as_ptr();
                     return 1;
                 }
             } else {
@@ -409,19 +332,14 @@ pub extern "C" fn rs_mqtt_tx_get_unsubscribe_topic(tx: &MQTTTransaction,
         }
     }
 
-    unsafe {
-        *buf = ptr::null();
-        *len = 0;
-    }
+    *buf = ptr::null();
+    *len = 0;
 
     return 0;
 }
 
 #[no_mangle]
-pub extern "C" fn rs_mqtt_tx_get_reason_code(
-    tx: &MQTTTransaction,
-    result: *mut u8,
-) -> u8 {
+pub unsafe extern "C" fn rs_mqtt_tx_get_reason_code(tx: &MQTTTransaction, result: *mut u8) -> u8 {
     for msg in tx.msg.iter() {
         match msg.op {
             MQTTOperation::PUBACK(ref v)
@@ -429,43 +347,32 @@ pub extern "C" fn rs_mqtt_tx_get_reason_code(
             | MQTTOperation::PUBREC(ref v)
             | MQTTOperation::PUBCOMP(ref v) => {
                 if let Some(rcode) = v.reason_code {
-                    unsafe {
-                        *result = rcode;
-                    }
+                    *result = rcode;
                     return 1;
                 }
             }
             MQTTOperation::AUTH(ref v) => {
-                unsafe {
-                    *result = v.reason_code;
-                }
+                *result = v.reason_code;
                 return 1;
             }
             MQTTOperation::CONNACK(ref v) => {
-                unsafe {
-                    *result = v.return_code;
-                }
+                *result = v.return_code;
                 return 1;
             }
             MQTTOperation::DISCONNECT(ref v) => {
                 if let Some(rcode) = v.reason_code {
-                    unsafe {
-                        *result = rcode;
-                    }
+                    *result = rcode;
                     return 1;
                 }
             }
-            _ => return 0
+            _ => return 0,
         }
     }
     return 0;
 }
 
 #[no_mangle]
-pub extern "C" fn rs_mqtt_tx_unsuback_has_reason_code(
-    tx: &MQTTTransaction,
-    code: u8,
-) -> u8 {
+pub extern "C" fn rs_mqtt_tx_unsuback_has_reason_code(tx: &MQTTTransaction, code: u8) -> u8 {
     for msg in tx.msg.iter() {
         if let MQTTOperation::UNSUBACK(ref unsuback) = msg.op {
             if let Some(ref reason_codes) = unsuback.reason_codes {
@@ -483,10 +390,11 @@ pub extern "C" fn rs_mqtt_tx_unsuback_has_reason_code(
 #[cfg(test)]
 mod test {
     use super::*;
-    use std;
     use crate::mqtt::mqtt::MQTTTransaction;
     use crate::mqtt::mqtt_message::*;
     use crate::mqtt::parser::FixedHeader;
+    use crate::core::Direction;
+    use std;
 
     #[test]
     fn test_multi_unsubscribe() {
@@ -503,7 +411,7 @@ mod test {
                 topics: vec!["foo".to_string(), "baar".to_string()],
                 properties: None,
             }),
-        });
+        }, Direction::ToServer);
         t.msg.push(MQTTMessage {
             header: FixedHeader {
                 message_type: MQTTTypeCode::UNSUBSCRIBE,
@@ -520,23 +428,23 @@ mod test {
         });
         let mut s: *const u8 = std::ptr::null_mut();
         let mut slen: u32 = 0;
-        let mut r = rs_mqtt_tx_get_unsubscribe_topic(&t, 0, &mut s, &mut slen);
+        let mut r = unsafe { rs_mqtt_tx_get_unsubscribe_topic(&t, 0, &mut s, &mut slen) };
         assert_eq!(r, 1);
-        let mut topic = String::from_utf8_lossy(build_slice!(s, slen as usize));
+        let mut topic = String::from_utf8_lossy(unsafe { build_slice!(s, slen as usize) });
         assert_eq!(topic, "foo");
-        r = rs_mqtt_tx_get_unsubscribe_topic(&t, 1, &mut s, &mut slen);
+        r = unsafe { rs_mqtt_tx_get_unsubscribe_topic(&t, 1, &mut s, &mut slen) };
         assert_eq!(r, 1);
-        topic = String::from_utf8_lossy(build_slice!(s, slen as usize));
+        topic = String::from_utf8_lossy(unsafe { build_slice!(s, slen as usize) });
         assert_eq!(topic, "baar");
-        r = rs_mqtt_tx_get_unsubscribe_topic(&t, 2, &mut s, &mut slen);
+        r = unsafe { rs_mqtt_tx_get_unsubscribe_topic(&t, 2, &mut s, &mut slen) };
         assert_eq!(r, 1);
-        topic = String::from_utf8_lossy(build_slice!(s, slen as usize));
+        topic = String::from_utf8_lossy(unsafe { build_slice!(s, slen as usize) });
         assert_eq!(topic, "fieee");
-        r = rs_mqtt_tx_get_unsubscribe_topic(&t, 3, &mut s, &mut slen);
+        r = unsafe { rs_mqtt_tx_get_unsubscribe_topic(&t, 3, &mut s, &mut slen) };
         assert_eq!(r, 1);
-        topic = String::from_utf8_lossy(build_slice!(s, slen as usize));
+        topic = String::from_utf8_lossy(unsafe { build_slice!(s, slen as usize) });
         assert_eq!(topic, "baaaaz");
-        r = rs_mqtt_tx_get_unsubscribe_topic(&t, 4, &mut s, &mut slen);
+        r = unsafe { rs_mqtt_tx_get_unsubscribe_topic(&t, 4, &mut s, &mut slen) };
         assert_eq!(r, 0);
     }
 
@@ -560,10 +468,11 @@ mod test {
                     MQTTSubscribeTopicData {
                         topic_name: "baar".to_string(),
                         qos: 1,
-                    }],
+                    },
+                ],
                 properties: None,
             }),
-        });
+        }, Direction::ToServer);
         t.msg.push(MQTTMessage {
             header: FixedHeader {
                 message_type: MQTTTypeCode::SUBSCRIBE,
@@ -582,29 +491,30 @@ mod test {
                     MQTTSubscribeTopicData {
                         topic_name: "baaaaz".to_string(),
                         qos: 1,
-                    }],
+                    },
+                ],
                 properties: None,
             }),
         });
         let mut s: *const u8 = std::ptr::null_mut();
         let mut slen: u32 = 0;
-        let mut r = rs_mqtt_tx_get_subscribe_topic(&t, 0, &mut s, &mut slen);
+        let mut r = unsafe { rs_mqtt_tx_get_subscribe_topic(&t, 0, &mut s, &mut slen) };
         assert_eq!(r, 1);
-        let mut topic = String::from_utf8_lossy(build_slice!(s, slen as usize));
+        let mut topic = String::from_utf8_lossy(unsafe { build_slice!(s, slen as usize) });
         assert_eq!(topic, "foo");
-        r = rs_mqtt_tx_get_subscribe_topic(&t, 1, &mut s, &mut slen);
+        r = unsafe { rs_mqtt_tx_get_subscribe_topic(&t, 1, &mut s, &mut slen) };
         assert_eq!(r, 1);
-        topic = String::from_utf8_lossy(build_slice!(s, slen as usize));
+        topic = String::from_utf8_lossy(unsafe { build_slice!(s, slen as usize) });
         assert_eq!(topic, "baar");
-        r = rs_mqtt_tx_get_subscribe_topic(&t, 2, &mut s, &mut slen);
+        r = unsafe { rs_mqtt_tx_get_subscribe_topic(&t, 2, &mut s, &mut slen) };
         assert_eq!(r, 1);
-        topic = String::from_utf8_lossy(build_slice!(s, slen as usize));
+        topic = String::from_utf8_lossy(unsafe { build_slice!(s, slen as usize) });
         assert_eq!(topic, "fieee");
-        r = rs_mqtt_tx_get_subscribe_topic(&t, 3, &mut s, &mut slen);
+        r = unsafe { rs_mqtt_tx_get_subscribe_topic(&t, 3, &mut s, &mut slen) };
         assert_eq!(r, 1);
-        topic = String::from_utf8_lossy(build_slice!(s, slen as usize));
+        topic = String::from_utf8_lossy(unsafe { build_slice!(s, slen as usize) });
         assert_eq!(topic, "baaaaz");
-        r = rs_mqtt_tx_get_subscribe_topic(&t, 4, &mut s, &mut slen);
+        r = unsafe { rs_mqtt_tx_get_subscribe_topic(&t, 4, &mut s, &mut slen) };
         assert_eq!(r, 0);
     }
 }
