@@ -24,6 +24,7 @@ use crate::smb::smb1::*;
 use crate::smb::smb2::*;
 use crate::dcerpc::dcerpc::*;
 use crate::smb::funcs::*;
+use crate::smb::smb_status::*;
 
 #[cfg(not(feature = "debug"))]
 fn debug_add_progress(_js: &mut JsonBuilder, _tx: &SMBTransaction) -> Result<(), JsonError> { Ok(()) }
@@ -66,17 +67,17 @@ fn guid_to_string(guid: &Vec<u8>) -> String {
 
 fn smb_common_header(jsb: &mut JsonBuilder, state: &SMBState, tx: &SMBTransaction) -> Result<(), JsonError>
 {
-    jsb.set_uint("id", tx.id as u64)?;
+    jsb.set_uint("id", tx.id)?;
 
     if state.dialect != 0 {
         let dialect = &smb2_dialect_string(state.dialect);
-        jsb.set_string("dialect", &dialect)?;
+        jsb.set_string("dialect", dialect)?;
     } else {
-        let dialect = match &state.dialect_vec {
-            &Some(ref d) => str::from_utf8(&d).unwrap_or("invalid"),
-            &None        => "unknown",
+        let dialect = match state.dialect_vec {
+            Some(ref d) => str::from_utf8(d).unwrap_or("invalid"),
+            None        => "unknown",
         };
-        jsb.set_string("dialect", &dialect)?;
+        jsb.set_string("dialect", dialect)?;
     }
 
     match tx.vercmd.get_version() {
@@ -98,11 +99,18 @@ fn smb_common_header(jsb: &mut JsonBuilder, state: &SMBState, tx: &SMBTransactio
     match tx.vercmd.get_ntstatus() {
         (true, ntstatus) => {
             let status = smb_ntstatus_string(ntstatus);
-            jsb.set_string("status", &status)?;
+            match status {
+                Some(x) => jsb.set_string("status", x)?,
+                None => {
+                    let status_str = format!("{}", ntstatus);
+                    jsb.set_string("status", &status_str)?
+                },
+            };
             let status_hex = format!("0x{:x}", ntstatus);
             jsb.set_string("status_code", &status_hex)?;
         },
         (false, _) => {
+            #[allow(clippy::single_match)]
             match tx.vercmd.get_dos_error() {
                 (true, errclass, errcode) => {
                     match errclass {
@@ -129,7 +137,7 @@ fn smb_common_header(jsb: &mut JsonBuilder, state: &SMBState, tx: &SMBTransactio
     }
 
 
-    jsb.set_uint("session_id", tx.hdr.ssn_id as u64)?;
+    jsb.set_uint("session_id", tx.hdr.ssn_id)?;
     jsb.set_uint("tree_id", tx.hdr.tree_id as u64)?;
 
     debug_add_progress(jsb, tx)?;
@@ -159,39 +167,33 @@ fn smb_common_header(jsb: &mut JsonBuilder, state: &SMBState, tx: &SMBTransactio
                 jsb.set_string("realm", &ticket.realm.0)?;
                 jsb.open_array("snames")?;
                 for sname in ticket.sname.name_string.iter() {
-                    jsb.append_string(&sname)?;
+                    jsb.append_string(sname)?;
                 }
                 jsb.close()?;
                 jsb.close()?;
             }
 
-            match x.request_host {
-                Some(ref r) => {
-                    jsb.open_object("request")?;
-                    let os = String::from_utf8_lossy(&r.native_os);
-                    jsb.set_string("native_os", &os)?;
-                    let lm = String::from_utf8_lossy(&r.native_lm);
-                    jsb.set_string("native_lm", &lm)?;
-                    jsb.close()?;
-                },
-                None => { },
+            if let Some(ref r) = x.request_host {
+                jsb.open_object("request")?;
+                let os = String::from_utf8_lossy(&r.native_os);
+                jsb.set_string("native_os", &os)?;
+                let lm = String::from_utf8_lossy(&r.native_lm);
+                jsb.set_string("native_lm", &lm)?;
+                jsb.close()?;
             }
-            match x.response_host {
-                Some(ref r) => {
-                    jsb.open_object("response")?;
-                    let os = String::from_utf8_lossy(&r.native_os);
-                    jsb.set_string("native_os", &os)?;
-                    let lm = String::from_utf8_lossy(&r.native_lm);
-                    jsb.set_string("native_lm", &lm)?;
-                    jsb.close()?;
-                },
-                None => { },
+            if let Some(ref r) = x.response_host {
+                jsb.open_object("response")?;
+                let os = String::from_utf8_lossy(&r.native_os);
+                jsb.set_string("native_os", &os)?;
+                let lm = String::from_utf8_lossy(&r.native_lm);
+                jsb.set_string("native_lm", &lm)?;
+                jsb.close()?;
             }
         },
         Some(SMBTransactionTypeData::CREATE(ref x)) => {
             let mut name_raw = x.filename.to_vec();
             name_raw.retain(|&i|i != 0x00);
-            if name_raw.len() > 0 {
+            if !name_raw.is_empty() {
                 let name = String::from_utf8_lossy(&name_raw);
                 if x.directory {
                     jsb.set_string("directory", &name)?;
@@ -231,14 +233,14 @@ fn smb_common_header(jsb: &mut JsonBuilder, state: &SMBState, tx: &SMBTransactio
             if x.smb_ver == 1 {
                 jsb.open_array("client_dialects")?;
                 for d in &x.dialects {
-                    let dialect = String::from_utf8_lossy(&d);
+                    let dialect = String::from_utf8_lossy(d);
                     jsb.append_string(&dialect)?;
                 }
                 jsb.close()?;
             } else if x.smb_ver == 2 {
                 jsb.open_array("client_dialects")?;
                 for d in &x.dialects2 {
-                    let dialect = String::from_utf8_lossy(&d);
+                    let dialect = String::from_utf8_lossy(d);
                     jsb.append_string(&dialect)?;
                 }
                 jsb.close()?;
@@ -258,8 +260,6 @@ fn smb_common_header(jsb: &mut JsonBuilder, state: &SMBState, tx: &SMBTransactio
             }
         },
         Some(SMBTransactionTypeData::TREECONNECT(ref x)) => {
-            jsb.set_uint("tree_id", x.tree_id as u64)?;
-
             let share_name = String::from_utf8_lossy(&x.share_name);
             if x.is_pipe {
                 jsb.set_string("named_pipe", &share_name)?;
@@ -272,11 +272,11 @@ fn smb_common_header(jsb: &mut JsonBuilder, state: &SMBState, tx: &SMBTransactio
                 jsb.open_object("service")?;
 
                 if let Some(ref s) = x.req_service {
-                    let serv = String::from_utf8_lossy(&s);
+                    let serv = String::from_utf8_lossy(s);
                     jsb.set_string("request", &serv)?;
                 }
                 if let Some(ref s) = x.res_service {
-                    let serv = String::from_utf8_lossy(&s);
+                    let serv = String::from_utf8_lossy(s);
                     jsb.set_string("response", &serv)?;
                 }
                 jsb.close()?;
@@ -336,10 +336,16 @@ fn smb_common_header(jsb: &mut JsonBuilder, state: &SMBState, tx: &SMBTransactio
                         jsb.set_uint("frag_cnt", x.frag_cnt_ts as u64)?;
                         jsb.set_uint("stub_data_size", x.stub_data_ts.len() as u64)?;
                         jsb.close()?;
-                    },
-                    DCERPC_TYPE_BIND => {
-                        match state.dcerpc_ifaces {
-                            Some(ref ifaces) => {
+                        if let Some(ref ifaces) = state.dcerpc_ifaces {
+                            // First filter the interfaces to those
+                            // with the context_id we want to log to
+                            // avoid creating an empty "interfaces"
+                            // array.
+                            let mut ifaces = ifaces
+                                .iter()
+                                .filter(|i| i.context_id == x.context_id)
+                                .peekable();
+                            if ifaces.peek().is_some() {
                                 jsb.open_array("interfaces")?;
                                 for i in ifaces {
                                     jsb.start_object()?;
@@ -348,22 +354,37 @@ fn smb_common_header(jsb: &mut JsonBuilder, state: &SMBState, tx: &SMBTransactio
                                     jsb.set_string("uuid", &ifstr)?;
                                     let vstr = format!("{}.{}", i.ver, i.ver_min);
                                     jsb.set_string("version", &vstr)?;
-
-                                    if i.acked {
-                                        jsb.set_uint("ack_result", i.ack_result as u64)?;
-                                        jsb.set_uint("ack_reason", i.ack_reason as u64)?;
-                                    }
                                     jsb.close()?;
                                 }
                                 jsb.close()?;
-                            },
-                            _ => {},
+                            }
+                        }
+                    },
+                    DCERPC_TYPE_BIND => {
+                        if let Some(ref ifaces) = state.dcerpc_ifaces {
+                            jsb.open_array("interfaces")?;
+                            for i in ifaces {
+                                jsb.start_object()?;
+                                let ifstr = uuid::Uuid::from_slice(&i.uuid);
+                                let ifstr = ifstr.map(|ifstr| ifstr.to_hyphenated().to_string()).unwrap();
+                                jsb.set_string("uuid", &ifstr)?;
+                                let vstr = format!("{}.{}", i.ver, i.ver_min);
+                                jsb.set_string("version", &vstr)?;
+                                
+                                if i.acked {
+                                    jsb.set_uint("ack_result", i.ack_result as u64)?;
+                                    jsb.set_uint("ack_reason", i.ack_reason as u64)?;
+                                }
+                                jsb.close()?;
+                            }
+                            jsb.close()?;
                         }
                     },
                     _ => {},
                 }
             }
             if x.res_set {
+                #[allow(clippy::single_match)]
                 match x.res_cmd {
                     DCERPC_TYPE_RESPONSE => {
                         jsb.open_object("res")?;
@@ -384,7 +405,7 @@ fn smb_common_header(jsb: &mut JsonBuilder, state: &SMBState, tx: &SMBTransactio
         Some(SMBTransactionTypeData::SETFILEPATHINFO(ref x)) => {
             let mut name_raw = x.filename.to_vec();
             name_raw.retain(|&i|i != 0x00);
-            if name_raw.len() > 0 {
+            if !name_raw.is_empty() {
                 let name = String::from_utf8_lossy(&name_raw);
                 jsb.set_string("filename", &name)?;
             } else {
@@ -407,6 +428,7 @@ fn smb_common_header(jsb: &mut JsonBuilder, state: &SMBState, tx: &SMBTransactio
                 _ => { },
             }
 
+            #[allow(clippy::single_match)]
             match x.loi {
                 1013 => { // Set Disposition Information
                     jsb.set_string("level_of_interest", "Set Disposition Information")?;
@@ -423,14 +445,14 @@ fn smb_common_header(jsb: &mut JsonBuilder, state: &SMBState, tx: &SMBTransactio
 }
 
 #[no_mangle]
-pub extern "C" fn rs_smb_log_json_request(mut jsb: &mut JsonBuilder, state: &mut SMBState, tx: &mut SMBTransaction) -> bool
+pub extern "C" fn rs_smb_log_json_request(jsb: &mut JsonBuilder, state: &mut SMBState, tx: &mut SMBTransaction) -> bool
 {
-    smb_common_header(&mut jsb, state, tx).is_ok()
+    smb_common_header(jsb, state, tx).is_ok()
 }
 
 #[no_mangle]
-pub extern "C" fn rs_smb_log_json_response(mut jsb: &mut JsonBuilder, state: &mut SMBState, tx: &mut SMBTransaction) -> bool
+pub extern "C" fn rs_smb_log_json_response(jsb: &mut JsonBuilder, state: &mut SMBState, tx: &mut SMBTransaction) -> bool
 {
-    smb_common_header(&mut jsb, state, tx).is_ok()
+    smb_common_header(jsb, state, tx).is_ok()
 }
 
