@@ -16,139 +16,127 @@
  */
 
 use super::ssh::SSHTransaction;
-use crate::core::STREAM_TOCLIENT;
+use crate::core::Direction;
 use std::ptr;
 
 #[no_mangle]
-pub extern "C" fn rs_ssh_tx_get_protocol(
+pub unsafe extern "C" fn rs_ssh_tx_get_protocol(
     tx: *mut std::os::raw::c_void, buffer: *mut *const u8, buffer_len: *mut u32, direction: u8,
 ) -> u8 {
     let tx = cast_pointer!(tx, SSHTransaction);
-    if direction & STREAM_TOCLIENT != 0 {
-        let m = &tx.srv_hdr.protover;
-        if m.len() > 0 {
-            unsafe {
+    match direction.into() {
+        Direction::ToServer => {
+            let m = &tx.cli_hdr.protover;
+            if !m.is_empty() {
                 *buffer = m.as_ptr();
                 *buffer_len = m.len() as u32;
+                return 1;
             }
-            return 1;
         }
-    } else {
-        let m = &tx.cli_hdr.protover;
-        if m.len() > 0 {
-            unsafe {
+        Direction::ToClient => {
+            let m = &tx.srv_hdr.protover;
+            if !m.is_empty() {
                 *buffer = m.as_ptr();
                 *buffer_len = m.len() as u32;
+                return 1;
             }
-            return 1;
         }
     }
-    unsafe {
-        *buffer = ptr::null();
-        *buffer_len = 0;
-    }
+    *buffer = ptr::null();
+    *buffer_len = 0;
 
     return 0;
 }
 
 #[no_mangle]
-pub extern "C" fn rs_ssh_tx_get_software(
+pub unsafe extern "C" fn rs_ssh_tx_get_software(
     tx: *mut std::os::raw::c_void, buffer: *mut *const u8, buffer_len: *mut u32, direction: u8,
 ) -> u8 {
     let tx = cast_pointer!(tx, SSHTransaction);
-    if direction & STREAM_TOCLIENT != 0 {
-        let m = &tx.srv_hdr.swver;
-        if m.len() > 0 {
-            unsafe {
+    match direction.into() {
+        Direction::ToServer => {
+            let m = &tx.cli_hdr.swver;
+            if !m.is_empty() {
                 *buffer = m.as_ptr();
                 *buffer_len = m.len() as u32;
+                return 1;
             }
-            return 1;
         }
-    } else {
-        let m = &tx.cli_hdr.swver;
-        if m.len() > 0 {
-            unsafe {
+        Direction::ToClient => {
+            let m = &tx.srv_hdr.swver;
+            if !m.is_empty() {
                 *buffer = m.as_ptr();
                 *buffer_len = m.len() as u32;
+                return 1;
             }
-            return 1;
         }
     }
-    unsafe {
-        *buffer = ptr::null();
-        *buffer_len = 0;
-    }
+    *buffer = ptr::null();
+    *buffer_len = 0;
 
     return 0;
 }
 
 #[no_mangle]
-pub extern "C" fn rs_ssh_tx_get_hassh(
+pub unsafe extern "C" fn rs_ssh_tx_get_hassh(
     tx: *mut std::os::raw::c_void,
     buffer: *mut *const u8,
     buffer_len: *mut u32,
     direction: u8,
 ) -> u8 {
     let tx = cast_pointer!(tx, SSHTransaction);
-    if direction & STREAM_TOCLIENT != 0 {
-        let m = &tx.srv_hdr.hassh;
-        if m.len() > 0 {
-            unsafe {
+    match direction.into() {
+        Direction::ToServer => {
+            let m = &tx.cli_hdr.hassh;
+            if !m.is_empty() {
                 *buffer = m.as_ptr();
                 *buffer_len = m.len() as u32;
+                return 1;
             }
-            return 1;
         }
-    } else {
-        let m = &tx.cli_hdr.hassh;
-        if m.len() > 0 {
-            unsafe {
+        Direction::ToClient => {
+            let m = &tx.srv_hdr.hassh;
+            if !m.is_empty() {
                 *buffer = m.as_ptr();
                 *buffer_len = m.len() as u32;
+                return 1;
             }
-            return 1;
         }
     }
-    unsafe {
-        *buffer = ptr::null();
-        *buffer_len = 0;
-    }
+    *buffer = ptr::null();
+    *buffer_len = 0;
 
     return 0;
 }
 
 #[no_mangle]
-pub extern "C" fn rs_ssh_tx_get_hassh_string(
+pub unsafe extern "C" fn rs_ssh_tx_get_hassh_string(
     tx: *mut std::os::raw::c_void,
     buffer: *mut *const u8,
     buffer_len: *mut u32,
     direction: u8,
 ) -> u8 {
     let tx = cast_pointer!(tx, SSHTransaction);
-    if direction & STREAM_TOCLIENT != 0 {
-        let m = &tx.srv_hdr.hassh_string;
-        if m.len() > 0 {
-            unsafe {
+    match direction.into() {
+        Direction::ToServer => {
+            let m = &tx.cli_hdr.hassh_string;
+            if !m.is_empty() {
                 *buffer = m.as_ptr();
                 *buffer_len = m.len() as u32;
+                return 1;
             }
-            return 1;
         }
-    } else {
-        let m = &tx.cli_hdr.hassh_string;
-        if m.len() > 0 {
-            unsafe {
+        Direction::ToClient => {
+            let m = &tx.srv_hdr.hassh_string;
+            if !m.is_empty() {
                 *buffer = m.as_ptr();
                 *buffer_len = m.len() as u32;
+                return 1;
             }
-            return 1;
         }
     }
-    unsafe {
-        *buffer = ptr::null();
-        *buffer_len = 0;
-    }
+    *buffer = ptr::null();
+    *buffer_len = 0;
 
     return 0;
 }
