@@ -15,40 +15,49 @@
  * 02110-1301, USA.
  */
 
-use nom::combinator::rest;
-use nom::number::streaming::{be_u32, be_u8};
+use digest::Digest;
+use digest::Update;
+use md5::Md5;
+use nom7::branch::alt;
+use nom7::bytes::streaming::{is_not, tag, take, take_while};
+use nom7::character::streaming::char;
+use nom7::combinator::{complete, eof, not, rest, verify};
+use nom7::multi::length_data;
+use nom7::number::streaming::{be_u32, be_u8};
+use nom7::sequence::terminated;
+use nom7::IResult;
 use std::fmt;
 
 #[derive(PartialEq, Eq, Copy, Clone, Debug)]
 pub enum MessageCode {
-	SshMsgDisconnect,
-	SshMsgIgnore,
-	SshMsgUnimplemented,
-	SshMsgDebug,
-	SshMsgServiceRequest,
-	SshMsgServiceAccept,
-	SshMsgKexinit,
-	SshMsgNewKeys,
-	SshMsgKexdhInit,
-	SshMsgKexdhReply,
+	Disconnect,
+	Ignore,
+	Unimplemented,
+	Debug,
+	ServiceRequest,
+	ServiceAccept,
+	Kexinit,
+	NewKeys,
+	KexdhInit,
+	KexdhReply,
 	
-	SshMsgUndefined(u8),
+	Undefined(u8),
 }
 
 impl MessageCode {
     fn from_u8(value: u8) -> MessageCode {
         match value {
-            1 => MessageCode::SshMsgDisconnect,
-            2 => MessageCode::SshMsgIgnore,
-            3 => MessageCode::SshMsgUnimplemented,
-            4 => MessageCode::SshMsgDebug,
-            5 => MessageCode::SshMsgServiceRequest,
-            6 => MessageCode::SshMsgServiceAccept,
-            20 => MessageCode::SshMsgKexinit,
-            21 => MessageCode::SshMsgNewKeys,
-            30 => MessageCode::SshMsgKexdhInit,
-            31 => MessageCode::SshMsgKexdhReply,
-            _ => MessageCode::SshMsgUndefined(value),
+            1 => MessageCode::Disconnect,
+            2 => MessageCode::Ignore,
+            3 => MessageCode::Unimplemented,
+            4 => MessageCode::Debug,
+            5 => MessageCode::ServiceRequest,
+            6 => MessageCode::ServiceAccept,
+            20 => MessageCode::Kexinit,
+            21 => MessageCode::NewKeys,
+            30 => MessageCode::KexdhInit,
+            31 => MessageCode::KexdhReply,
+            _ => MessageCode::Undefined(value),
         }
     }
 }
@@ -62,19 +71,20 @@ fn is_not_lineend(b: u8) -> bool {
 }
 
 //may leave \r at the end to be removed
-named!(pub ssh_parse_line<&[u8], &[u8]>,
-    terminated!(
-        take_while!(is_not_lineend),
-        alt!( tag!("\n") | tag!("\r\n") |
-              do_parse!(
-                    bytes: tag!("\r") >>
-                    not!(eof!()) >> (bytes)
-                )
-            )
-    )
-);
+pub fn ssh_parse_line(i: &[u8]) -> IResult<&[u8], &[u8]> {
+    fn parser(i: &[u8]) -> IResult<&[u8], &[u8]> {
+        let (i, bytes) = tag("\r")(i)?;
+        let (i, _) = not(eof)(i)?;
+        Ok((i, bytes))
+    }
+    terminated(
+        take_while(is_not_lineend),
+        alt(( tag("\n"), tag("\r\n"), parser
+            ))
+        )(i)
+}
 
-#[derive(PartialEq)]
+#[derive(PartialEq, Eq)]
 pub struct SshBanner<'a> {
     pub protover: &'a [u8],
     pub swver: &'a [u8],
@@ -82,18 +92,16 @@ pub struct SshBanner<'a> {
 
 // Could be simplified adding dummy \n at the end
 // or use nom5 nom::bytes::complete::is_not
-named!(pub ssh_parse_banner<SshBanner>,
-    do_parse!(
-        tag!("SSH-") >>
-        protover: is_not!("-") >>
-        char!('-') >>
-        swver: alt!( complete!( is_not!(" \r\n") ) | rest ) >>
-        //remaining after space is comments
-        (SshBanner{protover, swver})
-    )
-);
+pub fn ssh_parse_banner(i: &[u8]) -> IResult<&[u8], SshBanner> {
+    let (i, _) = tag("SSH-")(i)?;
+    let (i, protover) = is_not("-")(i)?;
+    let (i, _) = char('-')(i)?;
+    let (i, swver) = alt((complete(is_not(" \r\n")), rest))(i)?;
+    //remaining after space is comments
+    Ok((i, SshBanner { protover, swver }))
+}
 
-#[derive(PartialEq)]
+#[derive(PartialEq, Eq)]
 pub struct SshRecordHeader {
     pub pkt_len: u32,
     padding_len: u8,
@@ -110,33 +118,39 @@ impl fmt::Display for SshRecordHeader {
     }
 }
 
-named!(pub ssh_parse_record_header<SshRecordHeader>,
-    do_parse!(
-        pkt_len: verify!(be_u32, |&val| val > 1) >>
-        padding_len: be_u8 >>
-        msg_code: be_u8 >>
-        (SshRecordHeader{pkt_len: pkt_len,
-            padding_len: padding_len,
-            msg_code: MessageCode::from_u8(msg_code)})
-    )
-);
+pub fn ssh_parse_record_header(i: &[u8]) -> IResult<&[u8], SshRecordHeader> {
+    let (i, pkt_len) = verify(be_u32, |&val| val > 1)(i)?;
+    let (i, padding_len) = be_u8(i)?;
+    let (i, msg_code) = be_u8(i)?;
+    Ok((
+        i,
+        SshRecordHeader {
+            pkt_len,
+            padding_len,
+            msg_code: MessageCode::from_u8(msg_code),
+        },
+    ))
+}
 
 //test for evasion against pkt_len=0or1...
-named!(pub ssh_parse_record<SshRecordHeader>,
-    do_parse!(
-        pkt_len: verify!(be_u32, |&val| val > 1) >>
-        padding_len: be_u8 >>
-        msg_code: be_u8 >>
-        take!((pkt_len-2) as usize) >>
-        (SshRecordHeader{pkt_len: pkt_len,
-            padding_len: padding_len,
-            msg_code: MessageCode::from_u8(msg_code)})
-    )
-);
+pub fn ssh_parse_record(i: &[u8]) -> IResult<&[u8], SshRecordHeader> {
+    let (i, pkt_len) = verify(be_u32, |&val| val > 1)(i)?;
+    let (i, padding_len) = be_u8(i)?;
+    let (i, msg_code) = be_u8(i)?;
+    let (i, _) = take((pkt_len - 2) as usize)(i)?;
+    Ok((
+        i,
+        SshRecordHeader {
+            pkt_len,
+            padding_len,
+            msg_code: MessageCode::from_u8(msg_code),
+        },
+    ))
+}
 
-#[derive(Debug,PartialEq)]
+#[derive(Debug, PartialEq, Eq)]
 pub struct SshPacketKeyExchange<'a> {
-    pub cookie: &'a[u8],
+    pub cookie: &'a [u8],
     pub kex_algs: &'a [u8],
     pub server_host_key_algs: &'a [u8],
     pub encr_algs_client_to_server: &'a [u8],
@@ -151,72 +165,87 @@ pub struct SshPacketKeyExchange<'a> {
     pub reserved: u32,
 }
 
-use md5::compute;
-
 const SSH_HASSH_STRING_DELIMITER_SLICE: [u8; 1] = [b';'];
 
 impl<'a> SshPacketKeyExchange<'a> {
-    pub fn generate_hassh(&self, hassh_string: &mut Vec<u8>, hassh: &mut Vec<u8>, to_server: &bool) {
-        let slices = if *to_server { 
-            [self.kex_algs, &SSH_HASSH_STRING_DELIMITER_SLICE,
-             self.encr_algs_server_to_client, &SSH_HASSH_STRING_DELIMITER_SLICE,
-             self.mac_algs_server_to_client, &SSH_HASSH_STRING_DELIMITER_SLICE,
-             self.comp_algs_server_to_client]}
-        else {
-            [self.kex_algs, &SSH_HASSH_STRING_DELIMITER_SLICE,
-             self.encr_algs_client_to_server, &SSH_HASSH_STRING_DELIMITER_SLICE,
-             self.mac_algs_client_to_server, &SSH_HASSH_STRING_DELIMITER_SLICE,
-             self.comp_algs_client_to_server]
+    pub fn generate_hassh(
+        &self, hassh_string: &mut Vec<u8>, hassh: &mut Vec<u8>, to_server: &bool,
+    ) {
+        let slices = if *to_server {
+            [
+                self.kex_algs,
+                &SSH_HASSH_STRING_DELIMITER_SLICE,
+                self.encr_algs_server_to_client,
+                &SSH_HASSH_STRING_DELIMITER_SLICE,
+                self.mac_algs_server_to_client,
+                &SSH_HASSH_STRING_DELIMITER_SLICE,
+                self.comp_algs_server_to_client,
+            ]
+        } else {
+            [
+                self.kex_algs,
+                &SSH_HASSH_STRING_DELIMITER_SLICE,
+                self.encr_algs_client_to_server,
+                &SSH_HASSH_STRING_DELIMITER_SLICE,
+                self.mac_algs_client_to_server,
+                &SSH_HASSH_STRING_DELIMITER_SLICE,
+                self.comp_algs_client_to_server,
+            ]
         };
         // reserving memory
         hassh_string.reserve_exact(slices.iter().fold(0, |acc, x| acc + x.len()));
         // copying slices to hassh string
-        slices.iter().for_each(|&x| hassh_string.extend_from_slice(x)); 
-        hassh.extend(format!("{:x?}", compute(&hassh_string)).as_bytes());
+        slices
+            .iter()
+            .for_each(|&x| hassh_string.extend_from_slice(x));
+        hassh.extend(format!("{:x}", Md5::new().chain(&hassh_string).finalize()).as_bytes());
     }
 }
 
-named!(parse_string<&[u8]>, do_parse!(
-    len: be_u32 >>
-    string: take!(len) >>
-    ( string )
-));
+#[inline]
+fn parse_string(i: &[u8]) -> IResult<&[u8], &[u8]> {
+    length_data(be_u32)(i)
+}
 
-named!(pub ssh_parse_key_exchange<SshPacketKeyExchange>, do_parse!(
-    cookie: take!(16) >>
-    kex_algs: parse_string >>
-    server_host_key_algs: parse_string >>
-    encr_algs_client_to_server: parse_string >>
-    encr_algs_server_to_client: parse_string >>
-    mac_algs_client_to_server: parse_string >>
-    mac_algs_server_to_client: parse_string >>
-    comp_algs_client_to_server: parse_string >>
-    comp_algs_server_to_client: parse_string >>
-    langs_client_to_server: parse_string >>
-    langs_server_to_client: parse_string >>
-    first_kex_packet_follows: be_u8 >>
-    reserved: be_u32 >>
-    ( SshPacketKeyExchange {
-        cookie: cookie,
-        kex_algs: kex_algs,
-        server_host_key_algs: server_host_key_algs,
-        encr_algs_client_to_server: encr_algs_client_to_server,
-        encr_algs_server_to_client: encr_algs_server_to_client,
-        mac_algs_client_to_server: mac_algs_client_to_server,
-        mac_algs_server_to_client: mac_algs_server_to_client,
-        comp_algs_client_to_server: comp_algs_client_to_server,
-        comp_algs_server_to_client: comp_algs_server_to_client,
-        langs_client_to_server: langs_client_to_server,
-        langs_server_to_client: langs_server_to_client,
-        first_kex_packet_follows: first_kex_packet_follows,
-        reserved: reserved,
-    } )
-));
+pub fn ssh_parse_key_exchange(i: &[u8]) -> IResult<&[u8], SshPacketKeyExchange> {
+    let (i, cookie) = take(16_usize)(i)?;
+    let (i, kex_algs) = parse_string(i)?;
+    let (i, server_host_key_algs) = parse_string(i)?;
+    let (i, encr_algs_client_to_server) = parse_string(i)?;
+    let (i, encr_algs_server_to_client) = parse_string(i)?;
+    let (i, mac_algs_client_to_server) = parse_string(i)?;
+    let (i, mac_algs_server_to_client) = parse_string(i)?;
+    let (i, comp_algs_client_to_server) = parse_string(i)?;
+    let (i, comp_algs_server_to_client) = parse_string(i)?;
+    let (i, langs_client_to_server) = parse_string(i)?;
+    let (i, langs_server_to_client) = parse_string(i)?;
+    let (i, first_kex_packet_follows) = be_u8(i)?;
+    let (i, reserved) = be_u32(i)?;
+    Ok((
+        i,
+        SshPacketKeyExchange {
+            cookie,
+            kex_algs,
+            server_host_key_algs,
+            encr_algs_client_to_server,
+            encr_algs_server_to_client,
+            mac_algs_client_to_server,
+            mac_algs_server_to_client,
+            comp_algs_client_to_server,
+            comp_algs_server_to_client,
+            langs_client_to_server,
+            langs_server_to_client,
+            first_kex_packet_follows,
+            reserved,
+        },
+    ))
+}
 
 #[cfg(test)]
 mod tests {
 
     use super::*;
+    use nom7::{Err, Needed};
 
     /// Simple test of some valid data.
     #[test]
@@ -289,7 +318,7 @@ mod tests {
             Ok((_, _)) => {
                 panic!("Expected incomplete result");
             }
-            Err(nom::Err::Incomplete(_)) => {
+            Err(Err::Incomplete(_)) => {
                 //OK
                 assert_eq!(1, 1);
             }
@@ -695,7 +724,7 @@ mod tests {
         0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00];
 
         if let Err(e) = ssh_parse_key_exchange(&server_key_exchange) {
-            assert_eq!(e, nom::Err::Incomplete(nom::Needed::Size(16755)));
+            assert_eq!(e, Err::Incomplete(Needed::new(15964)));
         }
         else {
             panic!("ssh_parse_key_exchange() parsed malicious key_exchange");
