@@ -25,6 +25,9 @@
 #ifndef __APP_LAYER_DETECT_PROTO__H__
 #define __APP_LAYER_DETECT_PROTO__H__
 
+#include "flow.h"
+#include "app-layer-protos.h"
+
 typedef struct AppLayerProtoDetectThreadCtx_ AppLayerProtoDetectThreadCtx;
 
 typedef AppProto (*ProbingParserFPtr)(
@@ -105,7 +108,7 @@ int AppLayerProtoDetectPMRegisterPatternCI(uint8_t ipproto, AppProto alproto,
  * \brief The first function to be called.  This initializes a global
  *        protocol detection context.
  *
- * \retval 0 On succcess;
+ * \retval 0 On success;
  * \retval -1 On failure.
  */
 int AppLayerProtoDetectSetup(void);
@@ -115,8 +118,8 @@ int AppLayerProtoDetectSetup(void);
  */
 void AppLayerProtoDetectReset(Flow *);
 
-void AppLayerRequestProtocolChange(Flow *f, uint16_t dp, AppProto expect_proto);
-void AppLayerRequestProtocolTLSUpgrade(Flow *f);
+bool AppLayerRequestProtocolChange(Flow *f, uint16_t dp, AppProto expect_proto);
+bool AppLayerRequestProtocolTLSUpgrade(Flow *f);
 
 /**
  * \brief Cleans up the app layer protocol detection phase.
@@ -132,7 +135,7 @@ int AppLayerProtoDetectDeSetup(void);
  *        With this function you are associating/registering a string
  *        that can be used by users to write rules, i.e.
  *        you register the http protocol for protocol detection using
- *        AppLayerProtoDetectRegisterProtocol(ctx, ALPROTO_HTTP, "http"),
+ *        AppLayerProtoDetectRegisterProtocol(ctx, ALPROTO_HTTP1, "http"),
  *        following which you can write rules like -
  *        alert http any any -> any any (sid:1;)
  *        which basically matches on the HTTP protocol.
@@ -148,6 +151,8 @@ int AppLayerProtoDetectDeSetup(void);
  */
 void AppLayerProtoDetectRegisterProtocol(AppProto alproto, const char *alproto_name);
 
+void AppLayerProtoDetectRegisterAlias(const char *proto_name, const char *proto_alias);
+
 /**
  * \brief Given a protocol name, checks if proto detection is enabled in
  *        the conf file.
@@ -160,6 +165,19 @@ void AppLayerProtoDetectRegisterProtocol(AppProto alproto, const char *alproto_n
 int AppLayerProtoDetectConfProtoDetectionEnabled(const char *ipproto,
                                                  const char *alproto);
 
+/**
+ * \brief Given a protocol name, checks if proto detection is enabled in
+ *        the conf file.
+ *
+ * \param alproto Name of the app layer protocol.
+ * \param default_enabled enable by default if not in the configuration file
+ *
+ * \retval 1 If enabled.
+ * \retval 0 If disabled.
+ */
+int AppLayerProtoDetectConfProtoDetectionEnabledDefault(
+        const char *ipproto, const char *alproto, bool default_enabled);
+
 /**
  * \brief Inits and returns an app layer protocol detection thread context.
 
