@@ -24,9 +24,6 @@
 #ifndef __APP_LAYER_ENIP_H__
 #define __APP_LAYER_ENIP_H__
 
-#include "decode.h"
-#include "detect-engine-state.h"
-#include "queue.h"
 
 void RegisterENIPUDPParsers(void);
 void RegisterENIPTCPParsers(void);
