@@ -1,4 +1,4 @@
-/* Copyright (C) 2007-2020 Open Information Security Foundation
+/* Copyright (C) 2007-2021 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -90,39 +90,22 @@ typedef enum {
 } FtpRequestCommand;
 
 typedef struct FtpCommand_ {
-    FtpRequestCommand command;
     const char *command_name;
+    FtpRequestCommand command;
     const uint8_t command_length;
 } FtpCommand;
 extern const FtpCommand FtpCommands[FTP_COMMAND_MAX + 1];
 
 typedef uint32_t FtpRequestCommandArgOfs;
 
-typedef uint16_t FtpResponseCode;
-
-enum {
-    FTP_FIELD_NONE = 0,
-
-    FTP_FIELD_REQUEST_LINE,
-    FTP_FIELD_REQUEST_COMMAND,
-    FTP_FIELD_REQUEST_ARGS,
-
-    FTP_FIELD_RESPONSE_LINE,
-    FTP_FIELD_REPONSE_CODE,
-
-    /* must be last */
-    FTP_FIELD_MAX,
-};
-
 /** used to hold the line state when we have fragmentation. */
 typedef struct FtpLineState_ {
     /** used to indicate if the current_line buffer is a malloced buffer.  We
      * use a malloced buffer, if a line is fragmented */
-    uint8_t *db;
-    uint32_t db_len;
-    uint8_t current_line_db;
-    /** we have seen LF for the currently parsed line */
-    uint8_t current_line_lf_seen;
+    const uint8_t *buf;
+    uint32_t len;
+    uint8_t delim_len;
+    bool lf_found;
 } FtpLineState;
 
 typedef struct FTPString_ {
@@ -155,34 +138,20 @@ typedef struct FTPTransaction_  {
     /* Handle multiple responses */
     TAILQ_HEAD(, FTPString_) response_list;
 
-    DetectEngineState *de_state;
-
     TAILQ_ENTRY(FTPTransaction_) next;
 } FTPTransaction;
 
 /** FTP State for app layer parser */
 typedef struct FtpState_ {
-    const uint8_t *input;
-    int32_t input_len;
-    uint8_t direction;
     bool active;
 
     FTPTransaction *curr_tx;
     TAILQ_HEAD(, FTPTransaction_) tx_list;  /**< transaction list */
     uint64_t tx_cnt;
 
-    /* --parser details-- */
-    /** current line extracted by the parser from the call to FTPGetline() */
-    const uint8_t *current_line;
-    /** length of the line in current_line.  Doesn't include the delimiter */
-    uint32_t current_line_len;
-    uint8_t current_line_delimiter_len;
     bool current_line_truncated_ts;
     bool current_line_truncated_tc;
 
-    /* 0 for toserver, 1 for toclient */
-    FtpLineState line_state[2];
-
     FtpRequestCommand command;
     FtpRequestCommandArgOfs arg_offset;
     uint32_t port_line_len;
@@ -190,9 +159,8 @@ typedef struct FtpState_ {
     uint8_t *port_line;
 
     uint16_t dyn_port;
-    /* specifies which loggers are done logging */
-    uint32_t logged;
 
+    AppLayerStateData state_data;
 } FtpState;
 
 enum {
@@ -205,13 +173,13 @@ typedef struct FtpDataState_ {
     uint8_t *input;
     uint8_t *file_name;
     FileContainer *files;
-    DetectEngineState *de_state;
     int32_t input_len;
     int16_t file_len;
     FtpRequestCommand command;
     uint8_t state;
     uint8_t direction;
     AppLayerTxData tx_data;
+    AppLayerStateData state_data;
 } FtpDataState;
 
 void RegisterFTPParsers(void);
