@@ -30,7 +30,7 @@
 #include "app-layer-detect-proto.h"
 #include "app-layer-parser.h"
 #include "app-layer-rdp.h"
-#include "rust-rdp-rdp-gen.h"
+#include "rust.h"
 
 void RegisterRdpParsers(void) {
     SCLogDebug("Registering rdp parser");
