@@ -1,4 +1,4 @@
-/* Copyright (C) 2007-2013 Open Information Security Foundation
+/* Copyright (C) 2007-2021 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -40,6 +40,7 @@
 #include "decode-events.h"
 #include "flow.h"
 
+#include "util-validate.h"
 #include "util-unittest.h"
 #include "util-debug.h"
 
@@ -49,6 +50,8 @@
 int DecodePPPOEDiscovery(ThreadVars *tv, DecodeThreadVars *dtv, Packet *p,
         const uint8_t *pkt, uint32_t len)
 {
+    DEBUG_VALIDATE_BUG_ON(pkt == NULL);
+
     StatsIncr(tv, dtv->counter_pppoe);
 
     if (len < PPPOE_DISCOVERY_HEADER_MIN_LEN) {
@@ -57,8 +60,6 @@ int DecodePPPOEDiscovery(ThreadVars *tv, DecodeThreadVars *dtv, Packet *p,
     }
 
     p->pppoedh = (PPPOEDiscoveryHdr *)pkt;
-    if (p->pppoedh == NULL)
-        return TM_ECODE_FAILED;
 
     /* parse the PPPOE code */
     switch (p->pppoedh->pppoe_code)
@@ -129,6 +130,8 @@ int DecodePPPOEDiscovery(ThreadVars *tv, DecodeThreadVars *dtv, Packet *p,
 int DecodePPPOESession(ThreadVars *tv, DecodeThreadVars *dtv, Packet *p,
         const uint8_t *pkt, uint32_t len)
 {
+    DEBUG_VALIDATE_BUG_ON(pkt == NULL);
+
     StatsIncr(tv, dtv->counter_pppoe);
 
     if (len < PPPOE_SESSION_HEADER_MIN_LEN) {
@@ -137,8 +140,6 @@ int DecodePPPOESession(ThreadVars *tv, DecodeThreadVars *dtv, Packet *p,
     }
 
     p->pppoesh = (PPPOESessionHdr *)pkt;
-    if (p->pppoesh == NULL)
-        return TM_ECODE_FAILED;
 
     SCLogDebug("PPPOE VERSION %" PRIu32 " TYPE %" PRIu32 " CODE %" PRIu32 " SESSIONID %" PRIu32 " LENGTH %" PRIu32 "",
            PPPOE_SESSION_GET_VERSION(p->pppoesh),  PPPOE_SESSION_GET_TYPE(p->pppoesh),  p->pppoesh->pppoe_code,  SCNtohs(p->pppoesh->session_id),  SCNtohs(p->pppoesh->pppoe_length));
@@ -211,7 +212,7 @@ int DecodePPPOESession(ThreadVars *tv, DecodeThreadVars *dtv, Packet *p,
                 }
 
                 if (IPV4_GET_RAW_VER((IPV4Hdr *)(pkt + pppoesh_len)) == 4) {
-                    DecodeIPV4(tv, dtv, p, pkt + pppoesh_len, len - pppoesh_len);
+                    DecodeIPV4(tv, dtv, p, pkt + pppoesh_len, (uint16_t)(len - pppoesh_len));
                 }
                 break;
 
@@ -223,7 +224,7 @@ int DecodePPPOESession(ThreadVars *tv, DecodeThreadVars *dtv, Packet *p,
                 if (unlikely(len - pppoesh_len > USHRT_MAX)) {
                     return TM_ECODE_FAILED;
                 }
-                DecodeIPV4(tv, dtv, p, pkt + pppoesh_len, len - pppoesh_len);
+                DecodeIPV4(tv, dtv, p, pkt + pppoesh_len, (uint16_t)(len - pppoesh_len));
                 break;
 
             /* PPP IPv6 was not tested */
@@ -236,7 +237,7 @@ int DecodePPPOESession(ThreadVars *tv, DecodeThreadVars *dtv, Packet *p,
                     return TM_ECODE_FAILED;
                 }
 
-                DecodeIPV6(tv, dtv, p, pkt + pppoesh_len, len - pppoesh_len);
+                DecodeIPV6(tv, dtv, p, pkt + pppoesh_len, (uint16_t)(len - pppoesh_len));
                 break;
 
             default:
@@ -258,8 +259,7 @@ static int DecodePPPOEtest01 (void)
 
     uint8_t raw_pppoe[] = { 0x11, 0x00, 0x00, 0x00, 0x00 };
     Packet *p = PacketGetFromAlloc();
-    if (unlikely(p == NULL))
-        return 0;
+    FAIL_IF_NULL(p);
     ThreadVars tv;
     DecodeThreadVars dtv;
 
@@ -268,13 +268,10 @@ static int DecodePPPOEtest01 (void)
 
     DecodePPPOESession(&tv, &dtv, p, raw_pppoe, sizeof(raw_pppoe));
 
-    if (ENGINE_ISSET_EVENT(p,PPPOE_PKT_TOO_SMALL))  {
-        SCFree(p);
-        return 1;
-    }
+    FAIL_IF(!ENGINE_ISSET_EVENT(p, PPPOE_PKT_TOO_SMALL));
 
     SCFree(p);
-    return 0;
+    PASS;
 }
 
 /** DecodePPPOEtest02
@@ -296,11 +293,9 @@ static int DecodePPPOEtest02 (void)
         0x46, 0x47, 0x48, 0x49 };
 
     Packet *p = PacketGetFromAlloc();
-    if (unlikely(p == NULL))
-        return 0;
+    FAIL_IF_NULL(p);
     ThreadVars tv;
     DecodeThreadVars dtv;
-    int ret = 0;
 
     memset(&tv, 0, sizeof(ThreadVars));
     memset(&dtv, 0, sizeof(DecodeThreadVars));
@@ -309,21 +304,14 @@ static int DecodePPPOEtest02 (void)
 
     DecodePPPOESession(&tv, &dtv, p, raw_pppoe, sizeof(raw_pppoe));
 
-    if(ENGINE_ISSET_EVENT(p,PPPOE_PKT_TOO_SMALL))  {
-        goto end;
-    }
+    FAIL_IF(ENGINE_ISSET_EVENT(p, PPPOE_PKT_TOO_SMALL));
 
     // and we insist that the invalid ICMP encapsulated (type 0xab, code 0xcd) is flagged
+    FAIL_IF(!ENGINE_ISSET_EVENT(p, ICMPV4_UNKNOWN_TYPE));
 
-    if(! ENGINE_ISSET_EVENT(p,ICMPV4_UNKNOWN_TYPE))  {
-        goto end;
-    }
-
-    ret = 1;
-end:
     FlowShutdown();
     SCFree(p);
-    return ret;
+    PASS;
 }
 
 
@@ -344,8 +332,7 @@ static int DecodePPPOEtest03 (void)
     };
 
     Packet *p = PacketGetFromAlloc();
-    if (unlikely(p == NULL))
-        return 0;
+    FAIL_IF_NULL(p);
     ThreadVars tv;
     DecodeThreadVars dtv;
 
@@ -353,13 +340,10 @@ static int DecodePPPOEtest03 (void)
     memset(&dtv, 0, sizeof(DecodeThreadVars));
 
     DecodePPPOEDiscovery(&tv, &dtv, p, raw_pppoe, sizeof(raw_pppoe));
-    if (p->pppoedh == NULL) {
-    SCFree(p);
-    return 0;
-    }
+    FAIL_IF_NULL(p->pppoedh);
 
     SCFree(p);
-    return 1;
+    PASS;
 }
 
 /** DecodePPPOEtest04
@@ -376,8 +360,7 @@ static int DecodePPPOEtest04 (void)
     };
 
     Packet *p = PacketGetFromAlloc();
-    if (unlikely(p == NULL))
-        return 0;
+    FAIL_IF_NULL(p);
     ThreadVars tv;
     DecodeThreadVars dtv;
 
@@ -386,17 +369,14 @@ static int DecodePPPOEtest04 (void)
 
     DecodePPPOEDiscovery(&tv, &dtv, p, raw_pppoe, sizeof(raw_pppoe));
 
-    if(ENGINE_ISSET_EVENT(p,PPPOE_WRONG_CODE))  {
-        SCFree(p);
-        return 1;
-    }
+    FAIL_IF(!ENGINE_ISSET_EVENT(p, PPPOE_WRONG_CODE));
 
     SCFree(p);
-    return 0;
+    PASS;
 }
 
 /** DecodePPPOEtest05
- *  \brief Valid exaple PADO PPPOE packet taken from RFC2516, but too short for given length
+ *  \brief Valid example PADO PPPOE packet taken from RFC2516, but too short for given length
  *  \retval 0 Expected test value
  */
 static int DecodePPPOEtest05 (void)
@@ -411,8 +391,7 @@ static int DecodePPPOEtest05 (void)
     };
 
     Packet *p = PacketGetFromAlloc();
-    if (unlikely(p == NULL))
-        return 0;
+    FAIL_IF_NULL(p);
     ThreadVars tv;
     DecodeThreadVars dtv;
 
@@ -421,13 +400,10 @@ static int DecodePPPOEtest05 (void)
 
     DecodePPPOEDiscovery(&tv, &dtv, p, raw_pppoe, sizeof(raw_pppoe));
 
-    if(ENGINE_ISSET_EVENT(p,PPPOE_MALFORMED_TAGS))  {
-        SCFree(p);
-        return 1;
-    }
+    FAIL_IF(!ENGINE_ISSET_EVENT(p, PPPOE_MALFORMED_TAGS));
 
     SCFree(p);
-    return 0;
+    PASS;
 }
 
 /** DecodePPPOEtest06
@@ -444,24 +420,11 @@ static int DecodePPPOEtest06 (void)
     pppoesh.pppoe_version_type = 0xAB;
     pppoedh.pppoe_version_type = 0xCD;
 
-    if (PPPOE_SESSION_GET_VERSION(&pppoesh) != 0x0A) {
-        printf("Error, PPPOE macro pppoe_session_get_version failed: ");
-        return 0;
-    }
-    if (PPPOE_SESSION_GET_TYPE(&pppoesh) != 0x0B) {
-        printf("Error, PPPOE macro pppoe_session_get_type failed: ");
-        return 0;
-    }
-    if (PPPOE_DISCOVERY_GET_VERSION(&pppoedh) != 0x0C) {
-        printf("Error, PPPOE macro pppoe_discovery_get_version failed: ");
-        return 0;
-    }
-    if (PPPOE_DISCOVERY_GET_TYPE(&pppoedh) != 0x0D) {
-        printf("Error, PPPOE macro pppoe_discovery_get_type failed: ");
-        return 0;
-    }
-
-    return 1;
+    FAIL_IF(PPPOE_SESSION_GET_VERSION(&pppoesh) != 0x0A);
+    FAIL_IF(PPPOE_SESSION_GET_TYPE(&pppoesh) != 0x0B);
+    FAIL_IF(PPPOE_DISCOVERY_GET_VERSION(&pppoedh) != 0x0C);
+    FAIL_IF(PPPOE_DISCOVERY_GET_TYPE(&pppoedh) != 0x0D);
+    PASS;
 }
 
 /** DecodePPPOEtest07
@@ -477,8 +440,7 @@ static int DecodePPPOEtest07(void)
         0x00, 0x00, 0x00, 0x00 };
 
     Packet *p = PacketGetFromAlloc();
-    if (unlikely(p == NULL))
-        FAIL;
+    FAIL_IF_NULL(p);
     ThreadVars tv;
     DecodeThreadVars dtv;
 
@@ -506,8 +468,7 @@ static int DecodePPPOEtest08(void)
         0x08, 0x0a, 0xcb, 0xae, 0x92, 0x63, 0x00, 0x00, 0x00, 0x00 };
 
     Packet *p = PacketGetFromAlloc();
-    if (unlikely(p == NULL))
-        FAIL;
+    FAIL_IF_NULL(p);
     ThreadVars tv;
     DecodeThreadVars dtv;
 
@@ -534,8 +495,7 @@ static int DecodePPPOEtest09(void)
         0x00, 0x00, 0x00, 0x00, 0x00 };
 
     Packet *p = PacketGetFromAlloc();
-    if (unlikely(p == NULL))
-        FAIL;
+    FAIL_IF_NULL(p);
     ThreadVars tv;
     DecodeThreadVars dtv;
 
@@ -563,8 +523,7 @@ static int DecodePPPOEtest10(void)
         0x02, 0x08, 0x0a, 0xcb, 0xae, 0x92, 0x63, 0x00, 0x00, 0x00, 0x00 };
 
     Packet *p = PacketGetFromAlloc();
-    if (unlikely(p == NULL))
-        FAIL;
+    FAIL_IF_NULL(p);
     ThreadVars tv;
     DecodeThreadVars dtv;
 
@@ -579,8 +538,6 @@ static int DecodePPPOEtest10(void)
 }
 #endif /* UNITTESTS */
 
-
-
 /**
  * \brief Registers PPPOE unit tests
  * \todo More PPPOE tests
