@@ -31,16 +31,13 @@
  */
 
 #include "suricata-common.h"
-#include "decode.h"
 #include "decode-tcp.h"
+#include "decode.h"
 #include "decode-events.h"
 #include "util-unittest.h"
 #include "util-debug.h"
 #include "util-optimize.h"
 #include "flow.h"
-#include "util-profiling.h"
-#include "pkt-var.h"
-#include "host.h"
 
 #define SET_OPTS(dst, src) \
     (dst).type = (src).type; \
@@ -132,7 +129,7 @@ static void DecodeTCPOptions(Packet *p, const uint8_t *pkt, uint16_t pktlen)
                             memcpy(&values, tcp_opts[tcp_opt_cnt].data, sizeof(values));
                             p->tcpvars.ts_val = SCNtohl(values[0]);
                             p->tcpvars.ts_ecr = SCNtohl(values[1]);
-                            p->tcpvars.ts_set = TRUE;
+                            p->tcpvars.ts_set = true;
                         }
                     }
                     break;
@@ -282,6 +279,9 @@ int DecodeTCP(ThreadVars *tv, DecodeThreadVars *dtv, Packet *p,
 }
 
 #ifdef UNITTESTS
+#include "util-unittest-helper.h"
+#include "packet.h"
+
 static int TCPCalculateValidChecksumtest01(void)
 {
     uint16_t csum = 0;
@@ -413,7 +413,7 @@ static int TCPGetWscaleTest01(void)
 
     retval = 1;
 end:
-    PACKET_RECYCLE(p);
+    PacketRecycle(p);
     FlowShutdown();
     SCFree(p);
     return retval;
@@ -459,7 +459,7 @@ static int TCPGetWscaleTest02(void)
 
     retval = 1;
 end:
-    PACKET_RECYCLE(p);
+    PacketRecycle(p);
     FlowShutdown();
     SCFree(p);
     return retval;
@@ -504,7 +504,7 @@ static int TCPGetWscaleTest03(void)
 
     retval = 1;
 end:
-    PACKET_RECYCLE(p);
+    PacketRecycle(p);
     FlowShutdown();
     SCFree(p);
     return retval;
@@ -569,7 +569,7 @@ static int TCPGetSackTest01(void)
 
     retval = 1;
 end:
-    PACKET_RECYCLE(p);
+    PacketRecycle(p);
     FlowShutdown();
     SCFree(p);
     return retval;
