@@ -56,10 +56,6 @@ static int DecodeUDPPacket(ThreadVars *t, Packet *p, const uint8_t *pkt, uint16_
         return -1;
     }
 
-    if (unlikely(len != UDP_GET_LEN(p))) {
-        // packet can still be valid, keeping for consistency with decoder.udp.hlen_invalid event
-        ENGINE_SET_INVALID_EVENT(p, UDP_HLEN_INVALID);
-    }
     if (unlikely(UDP_GET_LEN(p) < UDP_HEADER_LEN)) {
         ENGINE_SET_INVALID_EVENT(p, UDP_LEN_INVALID);
         return -1;
