@@ -1,4 +1,4 @@
-/* Copyright (C) 2007-2013 Open Information Security Foundation
+/* Copyright (C) 2007-2022 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -23,6 +23,7 @@
 
 #include "suricata-common.h"
 #include "detect-engine.h"
+#include "detect-engine-build.h"
 #include "detect-engine-prefilter.h"
 #include "detect-engine-prefilter-common.h"
 #include "detect-parse.h"
@@ -37,19 +38,23 @@
 static void DetectAppLayerProtocolRegisterTests(void);
 #endif
 
+typedef struct DetectAppLayerProtocolData_ {
+    AppProto alproto;
+    uint8_t negated;
+} DetectAppLayerProtocolData;
+
 static int DetectAppLayerProtocolPacketMatch(
         DetectEngineThreadCtx *det_ctx,
         Packet *p, const Signature *s, const SigMatchCtx *ctx)
 {
     SCEnter();
 
-    int r = 0;
+    bool r = false;
     const DetectAppLayerProtocolData *data = (const DetectAppLayerProtocolData *)ctx;
 
     /* if the sig is PD-only we only match when PD packet flags are set */
-    if ((s->flags & SIG_FLAG_PDONLY) &&
-        (p->flags & (PKT_PROTO_DETECT_TS_DONE|PKT_PROTO_DETECT_TC_DONE)) == 0)
-    {
+    if (s->type == SIG_TYPE_PDONLY &&
+            (p->flags & (PKT_PROTO_DETECT_TS_DONE | PKT_PROTO_DETECT_TC_DONE)) == 0) {
         SCLogDebug("packet %"PRIu64": flags not set", p->pcap_cnt);
         SCReturnInt(0);
     }
@@ -67,16 +72,14 @@ static int DetectAppLayerProtocolPacketMatch(
         SCLogDebug("toserver packet %"PRIu64": looking for %u/neg %u, got %u",
                 p->pcap_cnt, data->alproto, data->negated, f->alproto_ts);
 
-        r = (data->negated) ? (f->alproto_ts != data->alproto) :
-            (f->alproto_ts == data->alproto);
+        r = AppProtoEquals(data->alproto, f->alproto_ts);
 
     } else if ((f->alproto_tc != ALPROTO_UNKNOWN) && (p->flowflags & FLOW_PKT_TOCLIENT))
     {
         SCLogDebug("toclient packet %"PRIu64": looking for %u/neg %u, got %u",
                 p->pcap_cnt, data->alproto, data->negated, f->alproto_tc);
 
-        r = (data->negated) ? (f->alproto_tc != data->alproto) :
-            (f->alproto_tc == data->alproto);
+        r = AppProtoEquals(data->alproto, f->alproto_tc);
     }
     else {
         SCLogDebug("packet %"PRIu64": default case: direction %02x, approtos %u/%u/%u",
@@ -84,8 +87,11 @@ static int DetectAppLayerProtocolPacketMatch(
             p->flowflags & (FLOW_PKT_TOCLIENT|FLOW_PKT_TOSERVER),
             f->alproto, f->alproto_ts, f->alproto_tc);
     }
-
-    SCReturnInt(r);
+    r = r ^ data->negated;
+    if (r) {
+        SCReturnInt(1);
+    }
+    SCReturnInt(0);
 }
 
 static DetectAppLayerProtocolData *DetectAppLayerProtocolParse(const char *arg, bool negate)
@@ -98,8 +104,9 @@ static DetectAppLayerProtocolData *DetectAppLayerProtocolParse(const char *arg,
     } else {
         alproto = AppLayerGetProtoByName((char *)arg);
         if (alproto == ALPROTO_UNKNOWN) {
-            SCLogError(SC_ERR_INVALID_SIGNATURE, "app-layer-protocol "
-                    "keyword supplied with unknown protocol \"%s\"", arg);
+            SCLogError("app-layer-protocol "
+                       "keyword supplied with unknown protocol \"%s\"",
+                    arg);
             return NULL;
         }
     }
@@ -118,16 +125,16 @@ static bool HasConflicts(const DetectAppLayerProtocolData *us,
 {
     /* mixing negated and non negated is illegal */
     if (them->negated ^ us->negated)
-        return TRUE;
+        return true;
     /* multiple non-negated is illegal */
     if (!us->negated)
-        return TRUE;
+        return true;
     /* duplicate option */
     if (us->alproto == them->alproto)
-        return TRUE;
+        return true;
 
     /* all good */
-    return FALSE;
+    return false;
 }
 
 static int DetectAppLayerProtocolSetup(DetectEngineCtx *de_ctx,
@@ -137,7 +144,7 @@ static int DetectAppLayerProtocolSetup(DetectEngineCtx *de_ctx,
     SigMatch *sm = NULL;
 
     if (s->alproto != ALPROTO_UNKNOWN) {
-        SCLogError(SC_ERR_CONFLICTING_RULE_KEYWORDS, "Either we already "
+        SCLogError("Either we already "
                    "have the rule match on an app layer protocol set through "
                    "other keywords that match on this protocol, or have "
                    "already seen a non-negated app-layer-protocol.");
@@ -154,9 +161,9 @@ static int DetectAppLayerProtocolSetup(DetectEngineCtx *de_ctx,
             const DetectAppLayerProtocolData *them = (const DetectAppLayerProtocolData *)tsm->ctx;
 
             if (HasConflicts(data, them)) {
-                SCLogError(SC_ERR_CONFLICTING_RULE_KEYWORDS, "can't mix "
-                        "positive app-layer-protocol match with negated "
-                        "match or match for 'failed'.");
+                SCLogError("can't mix "
+                           "positive app-layer-protocol match with negated "
+                           "match or match for 'failed'.");
                 goto error;
             }
         }
@@ -192,7 +199,7 @@ PrefilterPacketAppProtoMatch(DetectEngineThreadCtx *det_ctx, Packet *p, const vo
 {
     const PrefilterPacketHeaderCtx *ctx = pectx;
 
-    if (PrefilterPacketHeaderExtraMatch(ctx, p) == FALSE) {
+    if (!PrefilterPacketHeaderExtraMatch(ctx, p)) {
         SCLogDebug("packet %"PRIu64": extra match failed", p->pcap_cnt);
         SCReturn;
     }
@@ -236,8 +243,8 @@ PrefilterPacketAppProtoCompare(PrefilterPacketHeaderValue v, void *smctx)
     const DetectAppLayerProtocolData *a = smctx;
     if (v.u16[0] == a->alproto &&
         v.u8[2] == (uint8_t)a->negated)
-        return TRUE;
-    return FALSE;
+        return true;
+    return false;
 }
 
 static int PrefilterSetupAppProto(DetectEngineCtx *de_ctx, SigGroupHead *sgh)
@@ -250,11 +257,11 @@ static int PrefilterSetupAppProto(DetectEngineCtx *de_ctx, SigGroupHead *sgh)
 
 static bool PrefilterAppProtoIsPrefilterable(const Signature *s)
 {
-    if (s->flags & SIG_FLAG_PDONLY) {
+    if (s->type == SIG_TYPE_PDONLY) {
         SCLogDebug("prefilter on PD %u", s->id);
-        return TRUE;
+        return true;
     }
-    return FALSE;
+    return false;
 }
 
 void DetectAppLayerProtocolRegister(void)
@@ -320,10 +327,10 @@ static int DetectAppLayerProtocolTest03(void)
 
     FAIL_IF(s->alproto != ALPROTO_UNKNOWN);
 
-    FAIL_IF_NULL(s->sm_lists[DETECT_SM_LIST_MATCH]);
-    FAIL_IF_NULL(s->sm_lists[DETECT_SM_LIST_MATCH]->ctx);
+    FAIL_IF_NULL(s->init_data->smlists[DETECT_SM_LIST_MATCH]);
+    FAIL_IF_NULL(s->init_data->smlists[DETECT_SM_LIST_MATCH]->ctx);
 
-    data = (DetectAppLayerProtocolData *)s->sm_lists[DETECT_SM_LIST_MATCH]->ctx;
+    data = (DetectAppLayerProtocolData *)s->init_data->smlists[DETECT_SM_LIST_MATCH]->ctx;
     FAIL_IF(data->alproto != ALPROTO_HTTP);
     FAIL_IF(data->negated);
     DetectEngineCtxFree(de_ctx);
@@ -344,10 +351,10 @@ static int DetectAppLayerProtocolTest04(void)
     FAIL_IF(s->alproto != ALPROTO_UNKNOWN);
     FAIL_IF(s->flags & SIG_FLAG_APPLAYER);
 
-    FAIL_IF_NULL(s->sm_lists[DETECT_SM_LIST_MATCH]);
-    FAIL_IF_NULL(s->sm_lists[DETECT_SM_LIST_MATCH]->ctx);
+    FAIL_IF_NULL(s->init_data->smlists[DETECT_SM_LIST_MATCH]);
+    FAIL_IF_NULL(s->init_data->smlists[DETECT_SM_LIST_MATCH]->ctx);
 
-    data = (DetectAppLayerProtocolData*)s->sm_lists[DETECT_SM_LIST_MATCH]->ctx;
+    data = (DetectAppLayerProtocolData *)s->init_data->smlists[DETECT_SM_LIST_MATCH]->ctx;
     FAIL_IF_NULL(data);
     FAIL_IF(data->alproto != ALPROTO_HTTP);
     FAIL_IF(data->negated == 0);
@@ -370,15 +377,15 @@ static int DetectAppLayerProtocolTest05(void)
     FAIL_IF(s->alproto != ALPROTO_UNKNOWN);
     FAIL_IF(s->flags & SIG_FLAG_APPLAYER);
 
-    FAIL_IF_NULL(s->sm_lists[DETECT_SM_LIST_MATCH]);
-    FAIL_IF_NULL(s->sm_lists[DETECT_SM_LIST_MATCH]->ctx);
+    FAIL_IF_NULL(s->init_data->smlists[DETECT_SM_LIST_MATCH]);
+    FAIL_IF_NULL(s->init_data->smlists[DETECT_SM_LIST_MATCH]->ctx);
 
-    data = (DetectAppLayerProtocolData*)s->sm_lists[DETECT_SM_LIST_MATCH]->ctx;
+    data = (DetectAppLayerProtocolData *)s->init_data->smlists[DETECT_SM_LIST_MATCH]->ctx;
     FAIL_IF_NULL(data);
     FAIL_IF(data->alproto != ALPROTO_HTTP);
     FAIL_IF(data->negated == 0);
 
-    data = (DetectAppLayerProtocolData*)s->sm_lists[DETECT_SM_LIST_MATCH]->next->ctx;
+    data = (DetectAppLayerProtocolData *)s->init_data->smlists[DETECT_SM_LIST_MATCH]->next->ctx;
     FAIL_IF_NULL(data);
     FAIL_IF(data->alproto != ALPROTO_SMTP);
     FAIL_IF(data->negated == 0);
@@ -491,10 +498,10 @@ static int DetectAppLayerProtocolTest13(void)
 
     FAIL_IF(s->alproto != ALPROTO_UNKNOWN);
 
-    FAIL_IF_NULL(s->sm_lists[DETECT_SM_LIST_MATCH]);
-    FAIL_IF_NULL(s->sm_lists[DETECT_SM_LIST_MATCH]->ctx);
+    FAIL_IF_NULL(s->init_data->smlists[DETECT_SM_LIST_MATCH]);
+    FAIL_IF_NULL(s->init_data->smlists[DETECT_SM_LIST_MATCH]->ctx);
 
-    data = (DetectAppLayerProtocolData *)s->sm_lists[DETECT_SM_LIST_MATCH]->ctx;
+    data = (DetectAppLayerProtocolData *)s->init_data->smlists[DETECT_SM_LIST_MATCH]->ctx;
     FAIL_IF(data->alproto != ALPROTO_FAILED);
     FAIL_IF(data->negated);
     DetectEngineCtxFree(de_ctx);
@@ -512,9 +519,9 @@ static int DetectAppLayerProtocolTest14(void)
             "(app-layer-protocol:http; flowbits:set,blah; sid:1;)");
     FAIL_IF_NULL(s1);
     FAIL_IF(s1->alproto != ALPROTO_UNKNOWN);
-    FAIL_IF_NULL(s1->sm_lists[DETECT_SM_LIST_MATCH]);
-    FAIL_IF_NULL(s1->sm_lists[DETECT_SM_LIST_MATCH]->ctx);
-    data = (DetectAppLayerProtocolData *)s1->sm_lists[DETECT_SM_LIST_MATCH]->ctx;
+    FAIL_IF_NULL(s1->init_data->smlists[DETECT_SM_LIST_MATCH]);
+    FAIL_IF_NULL(s1->init_data->smlists[DETECT_SM_LIST_MATCH]->ctx);
+    data = (DetectAppLayerProtocolData *)s1->init_data->smlists[DETECT_SM_LIST_MATCH]->ctx;
     FAIL_IF(data->alproto != ALPROTO_HTTP);
     FAIL_IF(data->negated);
 
@@ -522,9 +529,9 @@ static int DetectAppLayerProtocolTest14(void)
             "(app-layer-protocol:http; flow:to_client; sid:2;)");
     FAIL_IF_NULL(s2);
     FAIL_IF(s2->alproto != ALPROTO_UNKNOWN);
-    FAIL_IF_NULL(s2->sm_lists[DETECT_SM_LIST_MATCH]);
-    FAIL_IF_NULL(s2->sm_lists[DETECT_SM_LIST_MATCH]->ctx);
-    data = (DetectAppLayerProtocolData *)s2->sm_lists[DETECT_SM_LIST_MATCH]->ctx;
+    FAIL_IF_NULL(s2->init_data->smlists[DETECT_SM_LIST_MATCH]);
+    FAIL_IF_NULL(s2->init_data->smlists[DETECT_SM_LIST_MATCH]->ctx);
+    data = (DetectAppLayerProtocolData *)s2->init_data->smlists[DETECT_SM_LIST_MATCH]->ctx;
     FAIL_IF(data->alproto != ALPROTO_HTTP);
     FAIL_IF(data->negated);
 
@@ -533,16 +540,16 @@ static int DetectAppLayerProtocolTest14(void)
             "(app-layer-protocol:http; flow:to_client,established; sid:3;)");
     FAIL_IF_NULL(s3);
     FAIL_IF(s3->alproto != ALPROTO_UNKNOWN);
-    FAIL_IF_NULL(s3->sm_lists[DETECT_SM_LIST_MATCH]);
-    FAIL_IF_NULL(s3->sm_lists[DETECT_SM_LIST_MATCH]->ctx);
-    data = (DetectAppLayerProtocolData *)s3->sm_lists[DETECT_SM_LIST_MATCH]->ctx;
+    FAIL_IF_NULL(s3->init_data->smlists[DETECT_SM_LIST_MATCH]);
+    FAIL_IF_NULL(s3->init_data->smlists[DETECT_SM_LIST_MATCH]->ctx);
+    data = (DetectAppLayerProtocolData *)s3->init_data->smlists[DETECT_SM_LIST_MATCH]->ctx;
     FAIL_IF(data->alproto != ALPROTO_HTTP);
     FAIL_IF(data->negated);
 
     SigGroupBuild(de_ctx);
-    FAIL_IF_NOT(s1->flags & SIG_FLAG_PDONLY);
-    FAIL_IF_NOT(s2->flags & SIG_FLAG_PDONLY);
-    FAIL_IF(s3->flags & SIG_FLAG_PDONLY); // failure now
+    FAIL_IF_NOT(s1->type == SIG_TYPE_PDONLY);
+    FAIL_IF_NOT(s2->type == SIG_TYPE_PDONLY);
+    FAIL_IF(s3->type == SIG_TYPE_PDONLY); // failure now
 
     DetectEngineCtxFree(de_ctx);
     PASS;
