@@ -37,8 +37,7 @@ static int DetectDnsOpcodeSetup(DetectEngineCtx *de_ctx, Signature *s,
 
     void *detect = rs_detect_dns_opcode_parse(str);
     if (detect == NULL) {
-        SCLogError(SC_ERR_INVALID_RULE_ARGUMENT,
-                "failed to parse dns.opcode: %s", str);
+        SCLogError("failed to parse dns.opcode: %s", str);
         return -1;
     }
 
@@ -74,16 +73,6 @@ static int DetectDnsOpcodeMatch(DetectEngineThreadCtx *det_ctx,
     return rs_dns_opcode_match(txv, (void *)ctx, flags);
 }
 
-static int DetectEngineInspectRequestGenericDnsOpcode(ThreadVars *tv,
-        DetectEngineCtx *de_ctx, DetectEngineThreadCtx *det_ctx,
-        const Signature *s, const SigMatchData *smd,
-        Flow *f, uint8_t flags, void *alstate,
-        void *txv, uint64_t tx_id)
-{
-    return DetectEngineInspectGenericList(tv, de_ctx, det_ctx, s, smd,
-       f, flags, alstate, txv, tx_id);
-}
-
 void DetectDnsOpcodeRegister(void)
 {
     sigmatch_table[DETECT_AL_DNS_OPCODE].name  = "dns.opcode";
@@ -94,13 +83,11 @@ void DetectDnsOpcodeRegister(void)
     sigmatch_table[DETECT_AL_DNS_OPCODE].AppLayerTxMatch =
         DetectDnsOpcodeMatch;
 
-    DetectAppLayerInspectEngineRegister("dns.opcode",
-            ALPROTO_DNS, SIG_FLAG_TOSERVER, 0,
-            DetectEngineInspectRequestGenericDnsOpcode);
+    DetectAppLayerInspectEngineRegister2(
+            "dns.opcode", ALPROTO_DNS, SIG_FLAG_TOSERVER, 0, DetectEngineInspectGenericList, NULL);
 
-    DetectAppLayerInspectEngineRegister("dns.opcode",
-            ALPROTO_DNS, SIG_FLAG_TOCLIENT, 0,
-            DetectEngineInspectRequestGenericDnsOpcode);
+    DetectAppLayerInspectEngineRegister2(
+            "dns.opcode", ALPROTO_DNS, SIG_FLAG_TOCLIENT, 0, DetectEngineInspectGenericList, NULL);
 
     dns_opcode_list_id = DetectBufferTypeGetByName("dns.opcode");
 }
