@@ -36,7 +36,6 @@
  */
 
 #include "suricata-common.h"
-#include "debug.h"
 #include "detect.h"
 #include "flow.h"
 
@@ -69,10 +68,12 @@
 #include "util-var-name.h"
 #include "tm-threads.h"
 
-static int host_threshold_id = -1; /**< host storage id for thresholds */
+#include "action-globals.h"
+
+static HostStorageId host_threshold_id = { .id = -1 };     /**< host storage id for thresholds */
 static IPPairStorageId ippair_threshold_id = { .id = -1 }; /**< ip pair storage id for thresholds */
 
-int ThresholdHostStorageId(void)
+HostStorageId ThresholdHostStorageId(void)
 {
     return host_threshold_id;
 }
@@ -80,14 +81,12 @@ int ThresholdHostStorageId(void)
 void ThresholdInit(void)
 {
     host_threshold_id = HostStorageRegister("threshold", sizeof(void *), NULL, ThresholdListFree);
-    if (host_threshold_id == -1) {
-        FatalError(SC_ERR_FATAL,
-                   "Can't initiate host storage for thresholding");
+    if (host_threshold_id.id == -1) {
+        FatalError("Can't initiate host storage for thresholding");
     }
     ippair_threshold_id = IPPairStorageRegister("threshold", sizeof(void *), NULL, ThresholdListFree);
     if (ippair_threshold_id.id == -1) {
-        FatalError(SC_ERR_FATAL,
-                   "Can't initiate IP pair storage for thresholding");
+        FatalError("Can't initiate IP pair storage for thresholding");
     }
 }
 
@@ -157,7 +156,7 @@ const DetectThresholdData *SigGetThresholdTypeIter(
  *
  */
 
-static DetectThresholdEntry* ThresholdTimeoutCheck(DetectThresholdEntry *head, struct timeval *tv)
+static DetectThresholdEntry *ThresholdTimeoutCheck(DetectThresholdEntry *head, SCTime_t ts)
 {
     DetectThresholdEntry *tmp = head;
     DetectThresholdEntry *prev = NULL;
@@ -167,8 +166,8 @@ static DetectThresholdEntry* ThresholdTimeoutCheck(DetectThresholdEntry *head, s
         /* check if the 'check' timestamp is not before the creation ts.
          * This can happen due to the async nature of the host timeout
          * code that also calls this code from a management thread. */
-        struct timeval entry = TimevalWithSeconds(&tmp->tv1, (time_t)tmp->seconds);
-        if (TimevalEarlier(tv, &entry)) {
+        SCTime_t entry = SCTIME_ADD_SECS(tmp->tv1, (time_t)tmp->seconds);
+        if (SCTIME_CMP_LTE(ts, entry)) {
             prev = tmp;
             tmp = tmp->next;
             continue;
@@ -190,21 +189,20 @@ static DetectThresholdEntry* ThresholdTimeoutCheck(DetectThresholdEntry *head, s
     return new_head;
 }
 
-int ThresholdHostTimeoutCheck(Host *host, struct timeval *tv)
+int ThresholdHostTimeoutCheck(Host *host, SCTime_t ts)
 {
     DetectThresholdEntry* head = HostGetStorageById(host, host_threshold_id);
-    DetectThresholdEntry* new_head = ThresholdTimeoutCheck(head, tv);
+    DetectThresholdEntry *new_head = ThresholdTimeoutCheck(head, ts);
     if (new_head != head) {
         HostSetStorageById(host, host_threshold_id, new_head);
     }
     return new_head == NULL;
 }
 
-
-int ThresholdIPPairTimeoutCheck(IPPair *pair, struct timeval *tv)
+int ThresholdIPPairTimeoutCheck(IPPair *pair, SCTime_t ts)
 {
     DetectThresholdEntry* head = IPPairGetStorageById(pair, ippair_threshold_id);
-    DetectThresholdEntry* new_head = ThresholdTimeoutCheck(head, tv);
+    DetectThresholdEntry *new_head = ThresholdTimeoutCheck(head, ts);
     if (new_head != head) {
         IPPairSetStorageById(pair, ippair_threshold_id, new_head);
     }
@@ -280,8 +278,7 @@ static int ThresholdHandlePacketSuppress(Packet *p,
             break;
         case TRACK_RULE:
         default:
-            SCLogError(SC_ERR_INVALID_VALUE,
-                    "track mode %d is not supported", td->track);
+            SCLogError("track mode %d is not supported", td->track);
             break;
     }
     if (m == NULL)
@@ -327,18 +324,18 @@ static inline void RateFilterSetAction(Packet *p, PacketAlert *pa, uint8_t new_a
 * \retval int 1 if threshold reached for this entry
 *
 */
-static int IsThresholdReached(DetectThresholdEntry* lookup_tsh, const DetectThresholdData *td, struct timeval packet_time)
+static int IsThresholdReached(
+        DetectThresholdEntry *lookup_tsh, const DetectThresholdData *td, SCTime_t packet_time)
 {
     int ret = 0;
 
     /* Check if we have a timeout enabled, if so,
     * we still matching (and enabling the new_action) */
     if (lookup_tsh->tv_timeout != 0) {
-        if ((packet_time.tv_sec - lookup_tsh->tv_timeout) > td->timeout) {
+        if ((SCTIME_SECS(packet_time) - lookup_tsh->tv_timeout) > td->timeout) {
             /* Ok, we are done, timeout reached */
             lookup_tsh->tv_timeout = 0;
-        }
-        else {
+        } else {
             /* Already matching */
             ret = 1;
         } /* else - if ((packet_time - lookup_tsh->tv_timeout) > td->timeout) */
@@ -346,13 +343,13 @@ static int IsThresholdReached(DetectThresholdEntry* lookup_tsh, const DetectThre
     }
     else {
         /* Update the matching state with the timeout interval */
-        struct timeval entry = TimevalWithSeconds(&lookup_tsh->tv1, (time_t)td->seconds);
-        if (TimevalEarlier(&packet_time, &entry)) {
+        SCTime_t entry = SCTIME_ADD_SECS(lookup_tsh->tv1, td->seconds);
+        if (SCTIME_CMP_LTE(packet_time, entry)) {
             lookup_tsh->current_count++;
             if (lookup_tsh->current_count > td->count) {
                 /* Then we must enable the new action by setting a
                 * timeout */
-                lookup_tsh->tv_timeout = packet_time.tv_sec;
+                lookup_tsh->tv_timeout = SCTIME_SECS(packet_time);
                 ret = 1;
             }
         } else {
@@ -364,7 +361,7 @@ static int IsThresholdReached(DetectThresholdEntry* lookup_tsh, const DetectThre
     return ret;
 }
 
-static void AddEntryToHostStorage(Host *h, DetectThresholdEntry *e, struct timeval packet_time)
+static void AddEntryToHostStorage(Host *h, DetectThresholdEntry *e, SCTime_t packet_time)
 {
     if (h && e) {
         e->current_count = 1;
@@ -375,7 +372,7 @@ static void AddEntryToHostStorage(Host *h, DetectThresholdEntry *e, struct timev
     }
 }
 
-static void AddEntryToIPPairStorage(IPPair *pair, DetectThresholdEntry *e, struct timeval packet_time)
+static void AddEntryToIPPairStorage(IPPair *pair, DetectThresholdEntry *e, SCTime_t packet_time)
 {
     if (pair && e) {
         e->current_count = 1;
@@ -406,8 +403,8 @@ static int ThresholdHandlePacket(Packet *p, DetectThresholdEntry *lookup_tsh,
             SCLogDebug("limit");
 
             if (lookup_tsh != NULL)  {
-                struct timeval entry = TimevalWithSeconds(&lookup_tsh->tv1, (time_t)td->seconds);
-                if (TimevalEarlier(&p->ts, &entry)) {
+                SCTime_t entry = SCTIME_ADD_SECS(lookup_tsh->tv1, td->seconds);
+                if (SCTIME_CMP_LTE(p->ts, entry)) {
                     lookup_tsh->current_count++;
 
                     if (lookup_tsh->current_count <= td->count) {
@@ -433,8 +430,8 @@ static int ThresholdHandlePacket(Packet *p, DetectThresholdEntry *lookup_tsh,
             SCLogDebug("threshold");
 
             if (lookup_tsh != NULL)  {
-                struct timeval entry = TimevalWithSeconds(&lookup_tsh->tv1, (time_t)td->seconds);
-                if (TimevalEarlier(&p->ts, &entry)) {
+                SCTime_t entry = SCTIME_ADD_SECS(lookup_tsh->tv1, td->seconds);
+                if (SCTIME_CMP_LTE(p->ts, entry)) {
                     lookup_tsh->current_count++;
 
                     if (lookup_tsh->current_count >= td->count) {
@@ -459,8 +456,8 @@ static int ThresholdHandlePacket(Packet *p, DetectThresholdEntry *lookup_tsh,
             SCLogDebug("both");
 
             if (lookup_tsh != NULL) {
-                struct timeval entry = TimevalWithSeconds(&lookup_tsh->tv1, (time_t)td->seconds);
-                if (TimevalEarlier(&p->ts, &entry)) {
+                SCTime_t entry = SCTIME_ADD_SECS(lookup_tsh->tv1, td->seconds);
+                if (SCTIME_CMP_LTE(p->ts, entry)) {
                     /* within time limit */
 
                     lookup_tsh->current_count++;
@@ -497,8 +494,8 @@ static int ThresholdHandlePacket(Packet *p, DetectThresholdEntry *lookup_tsh,
             SCLogDebug("detection_filter");
 
             if (lookup_tsh != NULL) {
-                struct timeval entry = TimevalWithSeconds(&lookup_tsh->tv1, (time_t)td->seconds);
-                if (TimevalEarlier(&p->ts, &entry)) {
+                SCTime_t entry = SCTIME_ADD_SECS(lookup_tsh->tv1, td->seconds);
+                if (SCTIME_CMP_LTE(p->ts, entry)) {
                     /* within timeout */
                     lookup_tsh->current_count++;
                     if (lookup_tsh->current_count > td->count) {
@@ -528,7 +525,7 @@ static int ThresholdHandlePacket(Packet *p, DetectThresholdEntry *lookup_tsh,
         }
         /* case TYPE_SUPPRESS: is not handled here */
         default:
-            SCLogError(SC_ERR_INVALID_VALUE, "type %d is not supported", td->type);
+            SCLogError("type %d is not supported", td->type);
     }
     return ret;
 }
@@ -593,7 +590,7 @@ static int ThresholdHandlePacketRule(DetectEngineCtx *de_ctx, Packet *p,
 /**
  * \brief Make the threshold logic for signatures
  *
- * \param de_ctx Dectection Context
+ * \param de_ctx Detection Context
  * \param tsh_ptr Threshold element
  * \param p Packet structure
  * \param s Signature structure
@@ -644,14 +641,13 @@ int PacketAlertThreshold(DetectEngineCtx *de_ctx, DetectEngineThreadCtx *det_ctx
 /**
  * \brief Init threshold context hash tables
  *
- * \param de_ctx Dectection Context
+ * \param de_ctx Detection Context
  *
  */
 void ThresholdHashInit(DetectEngineCtx *de_ctx)
 {
     if (SCMutexInit(&de_ctx->ths_ctx.threshold_table_lock, NULL) != 0) {
-                FatalError(SC_ERR_FATAL,
-                           "Threshold: Failed to initialize hash table mutex.");
+        FatalError("Threshold: Failed to initialize hash table mutex.");
     }
 }
 
@@ -716,10 +712,9 @@ void ThresholdHashAllocate(DetectEngineCtx *de_ctx)
     de_ctx->ths_ctx.th_size = highest_signum + 1;
     de_ctx->ths_ctx.th_entry = SCCalloc(de_ctx->ths_ctx.th_size, sizeof(DetectThresholdEntry *));
     if (de_ctx->ths_ctx.th_entry == NULL) {
-        FatalError(SC_ERR_MEM_ALLOC,
-                "Error allocating memory for rule "
-                "thresholds (tried to allocate %" PRIu32 " th_entrys for "
-                "rule tracking)",
+        FatalError(
+                "failed to allocate memory for \"by_rule\" thresholding (tried to allocate %" PRIu32
+                " entries)",
                 de_ctx->ths_ctx.th_size);
     }
 }
@@ -727,7 +722,7 @@ void ThresholdHashAllocate(DetectEngineCtx *de_ctx)
 /**
  * \brief Destroy threshold context hash tables
  *
- * \param de_ctx Dectection Context
+ * \param de_ctx Detection Context
  *
  */
 void ThresholdContextDestroy(DetectEngineCtx *de_ctx)
