@@ -32,8 +32,6 @@
 
 #include "app-layer-htp.h"
 
-#ifdef HAVE_NSS
-
 /**
  * \brief Read the bytes of a hash from an hexadecimal string
  *
@@ -50,8 +48,7 @@ int ReadHashString(uint8_t *hash, const char *string, const char *filename, int
         uint16_t expected_len)
 {
     if (strlen(string) != expected_len) {
-        SCLogError(SC_ERR_INVALID_HASH, "%s:%d hash string not %d characters",
-                filename, line_no, expected_len);
+        SCLogError("%s:%d hash string not %d characters", filename, line_no, expected_len);
         return -1;
     }
 
@@ -65,8 +62,7 @@ int ReadHashString(uint8_t *hash, const char *string, const char *filename, int
         if (value >= 0 && value <= 255)
             hash[x] = (uint8_t)value;
         else {
-            SCLogError(SC_ERR_INVALID_HASH, "%s:%d hash byte out of range %ld",
-                    filename, line_no, value);
+            SCLogError("%s:%d hash byte out of range %ld", filename, line_no, value);
             return -1;
         }
     }
@@ -252,15 +248,14 @@ static DetectFileHashData *DetectFileHashParse (const DetectEngineCtx *de_ctx,
                 snprintf(path, sizeof(path), "%s/%s", dir, str);
                 fp = fopen(path, "r");
                 if (fp == NULL) {
-                    SCLogError(SC_ERR_OPENING_RULE_FILE,
-                            "opening hash file %s: %s", path, strerror(errno));
+                    SCLogError("opening hash file %s: %s", path, strerror(errno));
                     goto error;
                 }
             }
         }
         if (fp == NULL) {
 #endif
-            SCLogError(SC_ERR_OPENING_RULE_FILE, "opening hash file %s: %s", filename, strerror(errno));
+            SCLogError("opening hash file %s: %s", filename, strerror(errno));
             goto error;
 #ifdef HAVE_LIBGEN_H
         }
@@ -322,8 +317,8 @@ error:
  * \retval 0 on Success
  * \retval -1 on Failure
  */
-int DetectFileHashSetup (DetectEngineCtx *de_ctx, Signature *s, const char *str,
-        uint32_t type, int list)
+int DetectFileHashSetup(
+        DetectEngineCtx *de_ctx, Signature *s, const char *str, uint16_t type, int list)
 {
     DetectFileHashData *filehash = NULL;
     SigMatch *sm = NULL;
@@ -379,5 +374,3 @@ void DetectFileHashFree(DetectEngineCtx *de_ctx, void *ptr)
         SCFree(filehash);
     }
 }
-
-#endif /* HAVE_NSS */
