@@ -29,9 +29,6 @@
 #define ISDATAAT_NEGATED    0x04
 #define ISDATAAT_OFFSET_VAR 0x08
 
-#define ISDATAAT_MIN 0
-#define ISDATAAT_MAX 65535
-
 typedef struct DetectIsdataatData_ {
     uint16_t dataat;     /* data offset to match */
     uint8_t flags; /* isdataat options*/
