@@ -25,7 +25,6 @@
 
 #include "suricata-common.h"
 #include "threads.h"
-#include "debug.h"
 #include "decode.h"
 #include "detect.h"
 
@@ -62,7 +61,7 @@ static int g_buffer_id = 0;
 
 static int DetectSipMethodSetup(DetectEngineCtx *de_ctx, Signature *s, const char *str)
 {
-    if (DetectBufferSetActiveList(s, g_buffer_id) < 0)
+    if (DetectBufferSetActiveList(de_ctx, s, g_buffer_id) < 0)
         return -1;
 
     if (DetectSignatureSetAppProto(s, ALPROTO_SIP) < 0)
@@ -73,28 +72,32 @@ static int DetectSipMethodSetup(DetectEngineCtx *de_ctx, Signature *s, const cha
 
 static bool DetectSipMethodValidateCallback(const Signature *s, const char **sigerror)
 {
-    const SigMatch *sm = s->init_data->smlists[g_buffer_id];
-    for ( ; sm != NULL; sm = sm->next) {
-        if (sm->type != DETECT_CONTENT)
+    for (uint32_t x = 0; x < s->init_data->buffer_index; x++) {
+        if (s->init_data->buffers[x].id != (uint32_t)g_buffer_id)
             continue;
-        const DetectContentData *cd = (const DetectContentData *)sm->ctx;
-        if (cd->content && cd->content_len) {
-            if (cd->content[cd->content_len-1] == 0x20) {
-                *sigerror = "sip.method pattern with trailing space";
-                SCLogError(SC_ERR_INVALID_SIGNATURE, "%s", *sigerror);
-                return true;
-            } else if (cd->content[0] == 0x20) {
-                *sigerror = "sip.method pattern with leading space";
-                SCLogError(SC_ERR_INVALID_SIGNATURE, "%s", *sigerror);
-                return true;
-            } else if (cd->content[cd->content_len-1] == 0x09) {
-                *sigerror = "sip.method pattern with trailing tab";
-                SCLogError(SC_ERR_INVALID_SIGNATURE, "%s", *sigerror);
-                return true;
-            } else if (cd->content[0] == 0x09) {
-                *sigerror = "sip.method pattern with leading tab";
-                SCLogError(SC_ERR_INVALID_SIGNATURE, "%s", *sigerror);
-                return true;
+        const SigMatch *sm = s->init_data->buffers[x].head;
+        for (; sm != NULL; sm = sm->next) {
+            if (sm->type != DETECT_CONTENT)
+                continue;
+            const DetectContentData *cd = (const DetectContentData *)sm->ctx;
+            if (cd->content && cd->content_len) {
+                if (cd->content[cd->content_len - 1] == 0x20) {
+                    *sigerror = "sip.method pattern with trailing space";
+                    SCLogError("%s", *sigerror);
+                    return true;
+                } else if (cd->content[0] == 0x20) {
+                    *sigerror = "sip.method pattern with leading space";
+                    SCLogError("%s", *sigerror);
+                    return true;
+                } else if (cd->content[cd->content_len - 1] == 0x09) {
+                    *sigerror = "sip.method pattern with trailing tab";
+                    SCLogError("%s", *sigerror);
+                    return true;
+                } else if (cd->content[0] == 0x09) {
+                    *sigerror = "sip.method pattern with leading tab";
+                    SCLogError("%s", *sigerror);
+                    return true;
+                }
             }
         }
     }
