@@ -23,7 +23,6 @@
 
 #include "suricata-common.h"
 #include "threads.h"
-#include "debug.h"
 #include "decode.h"
 
 #include "detect.h"
@@ -97,7 +96,7 @@ static InspectionBuffer *GetSshData(DetectEngineThreadCtx *det_ctx,
  */
 static int DetectSshHasshServerStringSetup(DetectEngineCtx *de_ctx, Signature *s, const char *arg)
 {
-    if (DetectBufferSetActiveList(s, g_ssh_hassh_server_string_buffer_id) < 0)
+    if (DetectBufferSetActiveList(de_ctx, s, g_ssh_hassh_server_string_buffer_id) < 0)
         return -1;
 
     if (DetectSignatureSetAppProto(s, ALPROTO_SSH) < 0)
@@ -109,7 +108,7 @@ static int DetectSshHasshServerStringSetup(DetectEngineCtx *de_ctx, Signature *s
     /* Check if Hassh is disabled */
     if (!RunmodeIsUnittests() && !rs_ssh_hassh_is_enabled()) {
         if (!SigMatchSilentErrorEnabled(de_ctx, DETECT_AL_SSH_HASSH_SERVER_STRING)) {
-            SCLogError(SC_WARN_HASSH_DISABLED, "hassh support is not enabled");
+            SCLogError("hassh support is not enabled");
         }
         return -2;
     }
@@ -141,4 +140,4 @@ void DetectSshHasshServerStringRegister(void)
     DetectBufferTypeSetDescriptionByName(BUFFER_NAME, BUFFER_DESC);
 
     g_ssh_hassh_server_string_buffer_id = DetectBufferTypeGetByName(BUFFER_NAME);
-}
\ No newline at end of file
+}
