@@ -1,4 +1,4 @@
-/* Copyright (C) 2007-2022 Open Information Security Foundation
+/* Copyright (C) 2007-2020 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -173,48 +173,52 @@ static DetectFlagsData *DetectFlagsParse (const char *rawstr)
 {
     SCEnter();
 
-    int ret = 0, found = 0, ignore = 0, res = 0;
-    int ov[MAX_SUBSTRINGS];
+    int found = 0, ignore = 0;
     char *ptr;
+    DetectFlagsData *de = NULL;
 
     char arg1[16] = "";
     char arg2[16] = "";
     char arg3[16] = "";
 
-    ret = DetectParsePcreExec(&parse_regex, rawstr, 0, 0, ov, MAX_SUBSTRINGS);
+    pcre2_match_data *match = NULL;
+    int ret = DetectParsePcreExec(&parse_regex, &match, rawstr, 0, 0);
     SCLogDebug("input '%s', pcre said %d", rawstr, ret);
     if (ret < 3) {
-        SCLogError(SC_ERR_PCRE_MATCH, "pcre match failed");
-        SCReturnPtr(NULL, "DetectFlagsData");
+        SCLogError("pcre match failed");
+        goto error;
     }
 
-    res = pcre_copy_substring((char *)rawstr, ov, MAX_SUBSTRINGS, 1, arg1, sizeof(arg1));
+    size_t pcre2len = sizeof(arg1);
+    int res = SC_Pcre2SubstringCopy(match, 1, (PCRE2_UCHAR8 *)arg1, &pcre2len);
     if (res < 0) {
-        SCLogError(SC_ERR_PCRE_GET_SUBSTRING, "pcre_get_substring failed");
-        SCReturnPtr(NULL, "DetectFlagsData");
+        SCLogError("pcre2_substring_copy_bynumber failed");
+        goto error;
     }
     if (ret >= 2) {
-        res = pcre_copy_substring((char *)rawstr, ov, MAX_SUBSTRINGS, 2, arg2, sizeof(arg2));
+        pcre2len = sizeof(arg2);
+        res = pcre2_substring_copy_bynumber(match, 2, (PCRE2_UCHAR8 *)arg2, &pcre2len);
         if (res < 0) {
-            SCLogError(SC_ERR_PCRE_GET_SUBSTRING, "pcre_get_substring failed");
-            SCReturnPtr(NULL, "DetectFlagsData");
+            SCLogError("pcre2_substring_copy_bynumber failed");
+            goto error;
         }
     }
     if (ret >= 3) {
-        res = pcre_copy_substring((char *)rawstr, ov, MAX_SUBSTRINGS, 3, arg3, sizeof(arg3));
+        pcre2len = sizeof(arg3);
+        res = SC_Pcre2SubstringCopy(match, 3, (PCRE2_UCHAR8 *)arg3, &pcre2len);
         if (res < 0) {
-            SCLogError(SC_ERR_PCRE_GET_SUBSTRING, "pcre_get_substring failed");
-            SCReturnPtr(NULL, "DetectFlagsData");
+            SCLogError("pcre2_substring_copy_bynumber failed");
+            goto error;
         }
     }
     SCLogDebug("args '%s', '%s', '%s'", arg1, arg2, arg3);
 
     if (strlen(arg2) == 0) {
         SCLogDebug("empty argument");
-        SCReturnPtr(NULL, "DetectFlagsData");
+        goto error;
     }
 
-    DetectFlagsData *de = SCMalloc(sizeof(DetectFlagsData));
+    de = SCMalloc(sizeof(DetectFlagsData));
     if (unlikely(de == NULL))
         goto error;
     memset(de, 0, sizeof(DetectFlagsData));
@@ -344,8 +348,8 @@ static DetectFlagsData *DetectFlagsParse (const char *rawstr)
 
                 case '!':
                     if (de->modifier != 0) {
-                        SCLogError(SC_ERR_FLAGS_MODIFIER, "\"flags\" supports only"
-                                " one modifier at a time");
+                        SCLogError("\"flags\" supports only"
+                                   " one modifier at a time");
                         goto error;
                     }
                     de->modifier = MODIFIER_NOT;
@@ -353,8 +357,8 @@ static DetectFlagsData *DetectFlagsParse (const char *rawstr)
                     break;
                 case '+':
                     if (de->modifier != 0) {
-                        SCLogError(SC_ERR_FLAGS_MODIFIER, "\"flags\" supports only"
-                                " one modifier at a time");
+                        SCLogError("\"flags\" supports only"
+                                   " one modifier at a time");
                         goto error;
                     }
                     de->modifier = MODIFIER_PLUS;
@@ -362,8 +366,8 @@ static DetectFlagsData *DetectFlagsParse (const char *rawstr)
                     break;
                 case '*':
                     if (de->modifier != 0) {
-                        SCLogError(SC_ERR_FLAGS_MODIFIER, "\"flags\" supports only"
-                                " one modifier at a time");
+                        SCLogError("\"flags\" supports only"
+                                   " one modifier at a time");
                         goto error;
                     }
                     de->modifier = MODIFIER_ANY;
@@ -447,6 +451,7 @@ static DetectFlagsData *DetectFlagsParse (const char *rawstr)
         }
     }
 
+    pcre2_match_data_free(match);
     SCLogDebug("found %"PRId32" ignore %"PRId32"", found, ignore);
     SCReturnPtr(de, "DetectFlagsData");
 
@@ -454,6 +459,9 @@ error:
     if (de) {
         SCFree(de);
     }
+    if (match) {
+        pcre2_match_data_free(match);
+    }
     SCReturnPtr(NULL, "DetectFlagsData");
 }
 
@@ -554,7 +562,7 @@ PrefilterPacketFlagsMatch(DetectEngineThreadCtx *det_ctx, Packet *p, const void
     }
 
     const PrefilterPacketHeaderCtx *ctx = pectx;
-    if (PrefilterPacketHeaderExtraMatch(ctx, p) == FALSE)
+    if (!PrefilterPacketHeaderExtraMatch(ctx, p))
         return;
 
     const uint8_t flags = p->tcph->th_flags;
@@ -582,8 +590,8 @@ PrefilterPacketFlagsCompare(PrefilterPacketHeaderValue v, void *smctx)
     if (v.u8[0] == a->modifier &&
         v.u8[1] == a->flags &&
         v.u8[2] == a->ignored_flags)
-        return TRUE;
-    return FALSE;
+        return true;
+    return false;
 }
 
 static int PrefilterSetupTcpFlags(DetectEngineCtx *de_ctx, SigGroupHead *sgh)
@@ -601,10 +609,10 @@ static bool PrefilterTcpFlagsIsPrefilterable(const Signature *s)
     for (sm = s->init_data->smlists[DETECT_SM_LIST_MATCH] ; sm != NULL; sm = sm->next) {
         switch (sm->type) {
             case DETECT_FLAGS:
-                return TRUE;
+                return true;
         }
     }
-    return FALSE;
+    return false;
 }
 
 /*
@@ -615,7 +623,7 @@ static bool PrefilterTcpFlagsIsPrefilterable(const Signature *s)
 /**
  * \test FlagsTestParse01 is a test for a  valid flags value
  *
- *  \retval 1 on succces
+ *  \retval 1 on success
  *  \retval 0 on failure
  */
 static int FlagsTestParse01 (void)
@@ -630,7 +638,7 @@ static int FlagsTestParse01 (void)
 /**
  * \test FlagsTestParse02 is a test for an invalid flags value
  *
- *  \retval 1 on succces
+ *  \retval 1 on success
  *  \retval 0 on failure
  */
 static int FlagsTestParse02 (void)
@@ -702,7 +710,7 @@ error:
 /**
  * \test FlagsTestParse04 check if ACK bit is set. Must fails.
  *
- *  \retval 1 on succces
+ *  \retval 1 on success
  *  \retval 0 on failure
  */
 static int FlagsTestParse04 (void)
@@ -1138,7 +1146,7 @@ error:
 /**
  * \test FlagsTestParse12 check if no flags are set. Must fails.
  *
- *  \retval 1 on succces
+ *  \retval 1 on success
  *  \retval 0 on failure
  */
 static int FlagsTestParse12 (void)
@@ -1195,7 +1203,7 @@ error:
 /**
  * \test test for a  valid flags value
  *
- *  \retval 1 on succces
+ *  \retval 1 on success
  *  \retval 0 on failure
  */
 static int FlagsTestParse13 (void)
