@@ -1,4 +1,4 @@
-/* Copyright (C) 2015-2017 Open Information Security Foundation
+/* Copyright (C) 2015-2022 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -38,15 +38,13 @@
 #include "detect-engine-content-inspection.h"
 #include "detect-template-rust-buffer.h"
 #include "app-layer-parser.h"
+#include "detect-engine-build.h"
 #include "rust.h"
 
-
-static int DetectTemplateRustBufferSetup(DetectEngineCtx *, Signature *,
-    const char *);
-static int DetectEngineInspectTemplateRustBuffer(ThreadVars *tv,
-    DetectEngineCtx *de_ctx, DetectEngineThreadCtx *det_ctx,
-    const Signature *s, const SigMatchData *smd,
-    Flow *f, uint8_t flags, void *alstate, void *txv, uint64_t tx_id);
+static int DetectTemplateRustBufferSetup(DetectEngineCtx *, Signature *, const char *);
+static uint8_t DetectEngineInspectTemplateRustBuffer(DetectEngineCtx *de_ctx,
+        DetectEngineThreadCtx *det_ctx, const struct DetectEngineAppInspectionEngine_ *engine,
+        const Signature *s, Flow *f, uint8_t flags, void *alstate, void *txv, uint64_t tx_id);
 #ifdef UNITTESTS
 static void DetectTemplateRustBufferRegisterTests(void);
 #endif
@@ -59,48 +57,41 @@ void DetectTemplateRustBufferRegister(void)
         return;
     }
     /* TEMPLATE_END_REMOVE */
-    sigmatch_table[DETECT_AL_TEMPLATE_RUST_BUFFER].name =
-        "template_rust_buffer";
-    sigmatch_table[DETECT_AL_TEMPLATE_RUST_BUFFER].desc =
-        "Template content modififier to match on the template buffers";
-    sigmatch_table[DETECT_AL_TEMPLATE_RUST_BUFFER].Setup =
-        DetectTemplateRustBufferSetup;
+    sigmatch_table[DETECT_AL_TEMPLATE_BUFFER].name = "template_rust_buffer";
+    sigmatch_table[DETECT_AL_TEMPLATE_BUFFER].desc =
+            "Template content modifier to match on the template buffers";
+    sigmatch_table[DETECT_AL_TEMPLATE_BUFFER].Setup = DetectTemplateRustBufferSetup;
 #ifdef UNITTESTS
-    sigmatch_table[DETECT_AL_TEMPLATE_RUST_BUFFER].RegisterTests =
-        DetectTemplateRustBufferRegisterTests;
+    sigmatch_table[DETECT_AL_TEMPLATE_BUFFER].RegisterTests = DetectTemplateRustBufferRegisterTests;
 #endif
-    sigmatch_table[DETECT_AL_TEMPLATE_RUST_BUFFER].flags |= SIGMATCH_NOOPT;
+    sigmatch_table[DETECT_AL_TEMPLATE_BUFFER].flags |= SIGMATCH_NOOPT;
 
     /* register inspect engines */
-    DetectAppLayerInspectEngineRegister("template_rust_buffer",
-            ALPROTO_TEMPLATE, SIG_FLAG_TOSERVER, 0,
-            DetectEngineInspectTemplateRustBuffer);
-    DetectAppLayerInspectEngineRegister("template_rust_buffer",
-            ALPROTO_TEMPLATE, SIG_FLAG_TOCLIENT, 0,
-            DetectEngineInspectTemplateRustBuffer);
+    DetectAppLayerInspectEngineRegister2("template_buffer", ALPROTO_TEMPLATE, SIG_FLAG_TOSERVER, 0,
+            DetectEngineInspectTemplateRustBuffer, NULL);
+    DetectAppLayerInspectEngineRegister2("template_buffer", ALPROTO_TEMPLATE, SIG_FLAG_TOCLIENT, 0,
+            DetectEngineInspectTemplateRustBuffer, NULL);
 
-    g_template_rust_id = DetectBufferTypeGetByName("template_rust_buffer");
+    g_template_rust_id = DetectBufferTypeGetByName("template_buffer");
 
     SCLogNotice("Template application layer detect registered.");
 }
 
-static int DetectTemplateRustBufferSetup(DetectEngineCtx *de_ctx, Signature *s,
-    const char *str)
+static int DetectTemplateRustBufferSetup(DetectEngineCtx *de_ctx, Signature *s, const char *str)
 {
     s->init_data->list = g_template_rust_id;
 
-    if (DetectSignatureSetAppProto(s, ALPROTO_TEMPLATE_RUST) != 0)
+    if (DetectSignatureSetAppProto(s, ALPROTO_TEMPLATE) != 0)
         return -1;
 
     return 0;
 }
 
-static int DetectEngineInspectTemplateRustBuffer(ThreadVars *tv,
-    DetectEngineCtx *de_ctx, DetectEngineThreadCtx *det_ctx,
-    const Signature *s, const SigMatchData *smd,
-    Flow *f, uint8_t flags, void *alstate, void *txv, uint64_t tx_id)
+static uint8_t DetectEngineInspectTemplateRustBuffer(DetectEngineCtx *de_ctx,
+        DetectEngineThreadCtx *det_ctx, const struct DetectEngineAppInspectionEngine_ *engine,
+        const Signature *s, Flow *f, uint8_t flags, void *alstate, void *txv, uint64_t tx_id)
 {
-    int ret = 0;
+    uint8_t ret = 0;
     const uint8_t *data = NULL;
     uint32_t data_len = 0;
 
@@ -111,9 +102,9 @@ static int DetectEngineInspectTemplateRustBuffer(ThreadVars *tv,
     }
 
     if (data != NULL) {
-        ret = DetectEngineContentInspection(de_ctx, det_ctx, s, smd,
-            NULL, f, (uint8_t *)data, data_len, 0, DETECT_CI_FLAGS_SINGLE,
-            DETECT_ENGINE_CONTENT_INSPECTION_MODE_STATE);
+        ret = DetectEngineContentInspection(de_ctx, det_ctx, s, engine->smd, NULL, f,
+                (uint8_t *)data, data_len, 0, DETECT_CI_FLAGS_SINGLE,
+                DETECT_ENGINE_CONTENT_INSPECTION_MODE_STATE);
     }
 
     SCLogNotice("Returning %d.", ret);
@@ -124,11 +115,9 @@ static int DetectEngineInspectTemplateRustBuffer(ThreadVars *tv,
 
 #include "util-unittest.h"
 #include "util-unittest-helper.h"
-#include "app-layer-parser.h"
-#include "detect-engine.h"
-#include "detect-parse.h"
 #include "flow-util.h"
 #include "stream-tcp.h"
+#include "detect-engine-alert.h"
 
 static int DetectTemplateRustBufferTest(void)
 {
@@ -149,41 +138,37 @@ static int DetectTemplateRustBufferTest(void)
     memset(&tv, 0, sizeof(ThreadVars));
     p = UTHBuildPacket(request, sizeof(request), IPPROTO_TCP);
     FLOW_INITIALIZE(&f);
-    f.alproto = ALPROTO_TEMPLATE_RUST;
+    f.alproto = ALPROTO_TEMPLATE;
     f.protoctx = (void *)&tcp;
     f.proto = IPPROTO_TCP;
     f.flags |= FLOW_IPV4;
     p->flow = &f;
     p->flags |= PKT_HAS_FLOW | PKT_STREAM_EST;
     p->flowflags |= FLOW_PKT_TOSERVER | FLOW_PKT_ESTABLISHED;
-    StreamTcpInitConfig(TRUE);
+    StreamTcpInitConfig(true);
 
     de_ctx = DetectEngineCtxInit();
     FAIL_IF_NULL(de_ctx);
 
     /* This rule should match. */
-    s = DetectEngineAppendSig(de_ctx,
-        "alert tcp any any -> any any ("
-        "msg:\"TEMPLATE Test Rule\"; "
-        "template_rust_buffer; content:\"World!\"; "
-        "sid:1; rev:1;)");
+    s = DetectEngineAppendSig(de_ctx, "alert tcp any any -> any any ("
+                                      "msg:\"TEMPLATE Test Rule\"; "
+                                      "template_rust_buffer; content:\"World!\"; "
+                                      "sid:1; rev:1;)");
     FAIL_IF_NULL(s);
 
     /* This rule should not match. */
-    s = DetectEngineAppendSig(de_ctx,
-        "alert tcp any any -> any any ("
-        "msg:\"TEMPLATE Test Rule\"; "
-        "template_rust_buffer; content:\"W0rld!\"; "
-        "sid:2; rev:1;)");
+    s = DetectEngineAppendSig(de_ctx, "alert tcp any any -> any any ("
+                                      "msg:\"TEMPLATE Test Rule\"; "
+                                      "template_rust_buffer; content:\"W0rld!\"; "
+                                      "sid:2; rev:1;)");
     FAIL_IF_NULL(s);
 
     SigGroupBuild(de_ctx);
     DetectEngineThreadCtxInit(&tv, (void *)de_ctx, (void *)&det_ctx);
 
-    FLOWLOCK_WRLOCK(&f);
-    AppLayerParserParse(NULL, alp_tctx, &f, ALPROTO_TEMPLATE_RUST,
-                        STREAM_TOSERVER, request, sizeof(request));
-    FLOWLOCK_UNLOCK(&f);
+    AppLayerParserParse(
+            NULL, alp_tctx, &f, ALPROTO_TEMPLATE, STREAM_TOSERVER, request, sizeof(request));
 
     /* Check that we have app-layer state. */
     FAIL_IF_NULL(f.alstate);
@@ -201,7 +186,7 @@ static int DetectTemplateRustBufferTest(void)
         SigGroupCleanup(de_ctx);
     if (de_ctx != NULL)
         DetectEngineCtxFree(de_ctx);
-    StreamTcpFreeConfig(TRUE);
+    StreamTcpFreeConfig(true);
     FLOW_DESTROY(&f);
     UTHFreePacket(p);
 
