@@ -25,7 +25,6 @@
 
 #include "suricata-common.h"
 #include "threads.h"
-#include "debug.h"
 #include "decode.h"
 #include "detect.h"
 
@@ -45,7 +44,6 @@
 #include "conf-yaml-loader.h"
 
 #include "util-debug.h"
-#include "util-unittest.h"
 #include "util-spm.h"
 #include "util-print.h"
 #include "util-ja3.h"
@@ -76,7 +74,7 @@ void DetectTlsJa3HashRegister(void)
 {
     sigmatch_table[DETECT_AL_TLS_JA3_HASH].name = "ja3.hash";
     sigmatch_table[DETECT_AL_TLS_JA3_HASH].alias = "ja3_hash";
-    sigmatch_table[DETECT_AL_TLS_JA3_HASH].desc = "content modifier to match the JA3 hash buffer";
+    sigmatch_table[DETECT_AL_TLS_JA3_HASH].desc = "sticky buffer to match the JA3 hash buffer";
     sigmatch_table[DETECT_AL_TLS_JA3_HASH].url = "/rules/ja3-keywords.html#ja3-hash";
     sigmatch_table[DETECT_AL_TLS_JA3_HASH].Setup = DetectTlsJa3HashSetup;
     sigmatch_table[DETECT_AL_TLS_JA3_HASH].flags |= SIGMATCH_NOOPT;
@@ -88,6 +86,12 @@ void DetectTlsJa3HashRegister(void)
     DetectAppLayerMpmRegister2("ja3.hash", SIG_FLAG_TOSERVER, 2,
             PrefilterGenericMpmRegister, GetData, ALPROTO_TLS, 0);
 
+    DetectAppLayerMpmRegister2("ja3.hash", SIG_FLAG_TOSERVER, 2, PrefilterGenericMpmRegister,
+            Ja3DetectGetHash, ALPROTO_QUIC, 1);
+
+    DetectAppLayerInspectEngineRegister2("ja3.hash", ALPROTO_QUIC, SIG_FLAG_TOSERVER, 1,
+            DetectEngineInspectBufferGeneric, Ja3DetectGetHash);
+
     DetectBufferTypeSetDescriptionByName("ja3.hash", "TLS JA3 hash");
 
     DetectBufferTypeRegisterSetupCallback("ja3.hash",
@@ -112,11 +116,13 @@ void DetectTlsJa3HashRegister(void)
  */
 static int DetectTlsJa3HashSetup(DetectEngineCtx *de_ctx, Signature *s, const char *str)
 {
-    if (DetectBufferSetActiveList(s, g_tls_ja3_hash_buffer_id) < 0)
+    if (DetectBufferSetActiveList(de_ctx, s, g_tls_ja3_hash_buffer_id) < 0)
         return -1;
 
-    if (DetectSignatureSetAppProto(s, ALPROTO_TLS) < 0)
+    if (s->alproto != ALPROTO_UNKNOWN && s->alproto != ALPROTO_TLS && s->alproto != ALPROTO_QUIC) {
+        SCLogError("rule contains conflicting protocols.");
         return -1;
+    }
 
     /* try to enable JA3 */
     SSLEnableJA3();
@@ -124,10 +130,11 @@ static int DetectTlsJa3HashSetup(DetectEngineCtx *de_ctx, Signature *s, const ch
     /* Check if JA3 is disabled */
     if (!RunmodeIsUnittests() && Ja3IsDisabled("rule")) {
         if (!SigMatchSilentErrorEnabled(de_ctx, DETECT_AL_TLS_JA3_HASH)) {
-            SCLogError(SC_WARN_JA3_DISABLED, "ja3 support is not enabled");
+            SCLogError("ja3 support is not enabled");
         }
         return -2;
     }
+    s->init_data->init_flags |= SIG_FLAG_INIT_JA3;
 
     return 0;
 }
@@ -157,60 +164,64 @@ static InspectionBuffer *GetData(DetectEngineThreadCtx *det_ctx,
 static bool DetectTlsJa3HashValidateCallback(const Signature *s,
                                               const char **sigerror)
 {
-    const SigMatch *sm = s->init_data->smlists[g_tls_ja3_hash_buffer_id];
-    for ( ; sm != NULL; sm = sm->next)
-    {
-        if (sm->type != DETECT_CONTENT)
+    for (uint32_t x = 0; x < s->init_data->buffer_index; x++) {
+        if (s->init_data->buffers[x].id != (uint32_t)g_tls_ja3_hash_buffer_id)
             continue;
+        const SigMatch *sm = s->init_data->buffers[x].head;
+        for (; sm != NULL; sm = sm->next) {
+            if (sm->type != DETECT_CONTENT)
+                continue;
 
-        const DetectContentData *cd = (DetectContentData *)sm->ctx;
+            const DetectContentData *cd = (DetectContentData *)sm->ctx;
 
-        if (cd->flags & DETECT_CONTENT_NOCASE) {
-            *sigerror = "ja3.hash should not be used together with "
-                        "nocase, since the rule is automatically "
-                        "lowercased anyway which makes nocase redundant.";
-            SCLogWarning(SC_WARN_POOR_RULE, "rule %u: %s", s->id, *sigerror);
+            if (cd->flags & DETECT_CONTENT_NOCASE) {
+                *sigerror = "ja3.hash should not be used together with "
+                            "nocase, since the rule is automatically "
+                            "lowercased anyway which makes nocase redundant.";
+                SCLogWarning("rule %u: %s", s->id, *sigerror);
+            }
+
+            if (cd->content_len == SC_MD5_HEX_LEN)
+                return true;
+
+            *sigerror = "Invalid length of the specified JA3 hash (should "
+                        "be 32 characters long). This rule will therefore "
+                        "never match.";
+            SCLogWarning("rule %u: %s", s->id, *sigerror);
+            return false;
         }
-
-        if (cd->content_len == 32)
-            return TRUE;
-
-        *sigerror = "Invalid length of the specified JA3 hash (should "
-                    "be 32 characters long). This rule will therefore "
-                    "never match.";
-        SCLogWarning(SC_WARN_POOR_RULE,  "rule %u: %s", s->id, *sigerror);
-        return FALSE;
     }
-
-    return TRUE;
+    return true;
 }
 
 static void DetectTlsJa3HashSetupCallback(const DetectEngineCtx *de_ctx,
                                           Signature *s)
 {
-    SigMatch *sm = s->init_data->smlists[g_tls_ja3_hash_buffer_id];
-    for ( ; sm != NULL; sm = sm->next)
-    {
-        if (sm->type != DETECT_CONTENT)
+    for (uint32_t x = 0; x < s->init_data->buffer_index; x++) {
+        if (s->init_data->buffers[x].id != (uint32_t)g_tls_ja3_hash_buffer_id)
             continue;
+        SigMatch *sm = s->init_data->buffers[x].head;
+        for (; sm != NULL; sm = sm->next) {
+            if (sm->type != DETECT_CONTENT)
+                continue;
 
-        DetectContentData *cd = (DetectContentData *)sm->ctx;
+            DetectContentData *cd = (DetectContentData *)sm->ctx;
 
-        bool changed = FALSE;
-        uint32_t u;
-        for (u = 0; u < cd->content_len; u++)
-        {
-            if (isupper(cd->content[u])) {
-                cd->content[u] = tolower(cd->content[u]);
-                changed = TRUE;
+            bool changed = false;
+            uint32_t u;
+            for (u = 0; u < cd->content_len; u++) {
+                if (isupper(cd->content[u])) {
+                    cd->content[u] = u8_tolower(cd->content[u]);
+                    changed = true;
+                }
             }
-        }
 
-        /* recreate the context if changes were made */
-        if (changed) {
-            SpmDestroyCtx(cd->spm_ctx);
-            cd->spm_ctx = SpmInitCtx(cd->content, cd->content_len, 1,
-                                     de_ctx->spm_global_thread_ctx);
+            /* recreate the context if changes were made */
+            if (changed) {
+                SpmDestroyCtx(cd->spm_ctx);
+                cd->spm_ctx =
+                        SpmInitCtx(cd->content, cd->content_len, 1, de_ctx->spm_global_thread_ctx);
+            }
         }
     }
 }
