@@ -16,29 +16,25 @@
  */
 
 #include "suricata-common.h"
+#include "output-filestore.h"
 
 #include "stream-tcp.h"
-#include "app-layer-parser.h"
-#include "app-layer-htp.h"
-#include "app-layer-htp-xff.h"
-#include "app-layer-smtp.h"
 
 #include "feature.h"
 
 #include "output.h"
-#include "output-filestore.h"
 #include "output-json-file.h"
 
-#include "util-print.h"
+#include "util-conf.h"
 #include "util-misc.h"
-
-#ifdef HAVE_NSS
+#include "util-path.h"
+#include "util-print.h"
 
 #define MODULE_NAME "OutputFilestore"
 
 /* Create a filestore specific PATH_MAX that is less than the system
  * PATH_MAX to prevent newer gcc truncation warnings with snprint. */
-#define SHA256_STRING_LEN (SHA256_LENGTH * 2)
+#define SHA256_STRING_LEN    (SC_SHA256_LEN * 2)
 #define LEAF_DIR_MAX_LEN 4
 #define FILESTORE_PREFIX_MAX (PATH_MAX - SHA256_STRING_LEN - LEAF_DIR_MAX_LEN)
 
@@ -62,15 +58,26 @@ typedef struct OutputFilestoreLogThread_ {
     uint16_t fs_error_counter;
 } OutputFilestoreLogThread;
 
+enum WarnOnceTypes {
+    WOT_OPEN,
+    WOT_WRITE,
+    WOT_UNLINK,
+    WOT_RENAME,
+    WOT_SNPRINTF,
+
+    WOT_MAX,
+};
+
 /* For WARN_ONCE, a record of warnings that have already been
  * issued. */
-static thread_local bool once_errs[SC_ERR_MAX];
+static thread_local bool once_errs[WOT_MAX];
 
-#define WARN_ONCE(err_code, ...)  do {                   \
-        if (!once_errs[err_code]) {                      \
-            once_errs[err_code] = true;                  \
-            SCLogWarning(err_code, __VA_ARGS__);         \
-        }                                                \
+#define WARN_ONCE(wot_type, ...)                                                                   \
+    do {                                                                                           \
+        if (!once_errs[wot_type]) {                                                                \
+            once_errs[wot_type] = true;                                                            \
+            SCLogWarning(__VA_ARGS__);                                                             \
+        }                                                                                          \
     } while (0)
 
 static uint64_t OutputFilestoreOpenFilesCounter(void)
@@ -90,15 +97,6 @@ static uint32_t FileGetMaxOpenFiles(void)
     return g_file_store_max_open_files;
 }
 
-static void PrintHexString(char *str, size_t size, uint8_t *buf, size_t buf_len)
-{
-    int i = 0;
-    size_t x = 0;
-    for (i = 0, x = 0; x < buf_len; x++) {
-        i += snprintf(&str[i], size - i, "%02x", buf[x]);
-    }
-}
-
 /**
  * \brief Update the timestamps on a file to match those of another
  *     file.
@@ -124,12 +122,13 @@ static void OutputFilestoreUpdateFileTime(const char *src_filename,
     }
 }
 
-static void OutputFilestoreFinalizeFiles(ThreadVars *tv,
-        const OutputFilestoreLogThread *oft, const OutputFilestoreCtx *ctx,
-        const Packet *p, File *ff, uint8_t dir) {
+static void OutputFilestoreFinalizeFiles(ThreadVars *tv, const OutputFilestoreLogThread *oft,
+        const OutputFilestoreCtx *ctx, const Packet *p, File *ff, void *tx, const uint64_t tx_id,
+        uint8_t dir)
+{
     /* Stringify the SHA256 which will be used in the final
      * filename. */
-    char sha256string[(SHA256_LENGTH * 2) + 1];
+    char sha256string[(SC_SHA256_LEN * 2) + 1];
     PrintHexString(sha256string, sizeof(sha256string), ff->sha256,
             sizeof(ff->sha256));
 
@@ -145,14 +144,13 @@ static void OutputFilestoreFinalizeFiles(ThreadVars *tv,
         OutputFilestoreUpdateFileTime(tmp_filename, final_filename);
         if (unlink(tmp_filename) != 0) {
             StatsIncr(tv, oft->fs_error_counter);
-            WARN_ONCE(SC_WARN_REMOVE_FILE,
-                    "Failed to remove temporary file %s: %s", tmp_filename,
+            WARN_ONCE(WOT_UNLINK, "Failed to remove temporary file %s: %s", tmp_filename,
                     strerror(errno));
         }
     } else if (rename(tmp_filename, final_filename) != 0) {
         StatsIncr(tv, oft->fs_error_counter);
-        WARN_ONCE(SC_WARN_RENAMING_FILE, "Failed to rename %s to %s: %s",
-                tmp_filename, final_filename, strerror(errno));
+        WARN_ONCE(WOT_RENAME, "Failed to rename %s to %s: %s", tmp_filename, final_filename,
+                strerror(errno));
         if (unlink(tmp_filename) != 0) {
             /* Just increment, don't log as has_fs_errors would
              * already be set above. */
@@ -163,15 +161,13 @@ static void OutputFilestoreFinalizeFiles(ThreadVars *tv,
 
     if (ctx->fileinfo) {
         char js_metadata_filename[PATH_MAX];
-        if (snprintf(js_metadata_filename, sizeof(js_metadata_filename),
-                        "%s.%"PRIuMAX".%u.json", final_filename,
-                        (uintmax_t)p->ts.tv_sec, ff->file_store_id)
-                == (int)sizeof(js_metadata_filename)) {
-            WARN_ONCE(SC_ERR_SPRINTF,
-                "Failed to write file info record. Output filename truncated.");
+        if (snprintf(js_metadata_filename, sizeof(js_metadata_filename), "%s.%" PRIuMAX ".%u.json",
+                    final_filename, (uintmax_t)SCTIME_SECS(p->ts),
+                    ff->file_store_id) == (int)sizeof(js_metadata_filename)) {
+            WARN_ONCE(WOT_SNPRINTF, "Failed to write file info record. Output filename truncated.");
         } else {
             JsonBuilder *js_fileinfo =
-                    JsonBuildFileInfoRecord(p, ff, true, dir, ctx->xff_cfg, NULL);
+                    JsonBuildFileInfoRecord(p, ff, tx, tx_id, true, dir, ctx->xff_cfg, NULL);
             if (likely(js_fileinfo != NULL)) {
                 jb_close(js_fileinfo);
                 FILE *out = fopen(js_metadata_filename, "w");
@@ -186,9 +182,9 @@ static void OutputFilestoreFinalizeFiles(ThreadVars *tv,
     }
 }
 
-static int OutputFilestoreLogger(ThreadVars *tv, void *thread_data,
-        const Packet *p, File *ff, const uint8_t *data, uint32_t data_len,
-        uint8_t flags, uint8_t dir)
+static int OutputFilestoreLogger(ThreadVars *tv, void *thread_data, const Packet *p, File *ff,
+        void *tx, const uint64_t tx_id, const uint8_t *data, uint32_t data_len, uint8_t flags,
+        uint8_t dir)
 {
     SCEnter();
     OutputFilestoreLogThread *aft = (OutputFilestoreLogThread *)thread_data;
@@ -196,15 +192,6 @@ static int OutputFilestoreLogger(ThreadVars *tv, void *thread_data,
     char filename[PATH_MAX] = "";
     int file_fd = -1;
 
-    /* no flow, no files */
-    if (p->flow == NULL) {
-        SCReturnInt(TM_ECODE_OK);
-    }
-
-    if (!(PKT_IS_IPV4(p) || PKT_IS_IPV6(p))) {
-        return 0;
-    }
-
     SCLogDebug("ff %p, data %p, data_len %u", ff, data, data_len);
 
     char base_filename[PATH_MAX] = "";
@@ -217,9 +204,7 @@ static int OutputFilestoreLogger(ThreadVars *tv, void *thread_data,
                 0644);
         if (file_fd == -1) {
             StatsIncr(tv, aft->fs_error_counter);
-            SCLogWarning(SC_ERR_OPENING_FILE,
-                    "Filestore (v2) failed to create %s: %s", filename,
-                    strerror(errno));
+            SCLogWarning("Filestore (v2) failed to create %s: %s", filename, strerror(errno));
             return -1;
         }
 
@@ -238,9 +223,8 @@ static int OutputFilestoreLogger(ThreadVars *tv, void *thread_data,
             file_fd = open(filename, O_APPEND | O_NOFOLLOW | O_WRONLY);
             if (file_fd == -1) {
                 StatsIncr(tv, aft->fs_error_counter);
-                WARN_ONCE(SC_ERR_OPENING_FILE,
-                        "Filestore (v2) failed to open file %s: %s",
-                        filename, strerror(errno));
+                WARN_ONCE(WOT_OPEN, "Filestore (v2) failed to open file %s: %s", filename,
+                        strerror(errno));
                 return -1;
             }
         } else {
@@ -252,9 +236,8 @@ static int OutputFilestoreLogger(ThreadVars *tv, void *thread_data,
         ssize_t r = write(file_fd, (const void *)data, (size_t)data_len);
         if (r == -1) {
             StatsIncr(tv, aft->fs_error_counter);
-            WARN_ONCE(SC_ERR_FWRITE,
-                    "Filestore (v2) failed to write to %s: %s",
-                    filename, strerror(errno));
+            WARN_ONCE(WOT_WRITE, "Filestore (v2) failed to write to %s: %s", filename,
+                    strerror(errno));
             if (ff->fd != -1) {
                 SC_ATOMIC_SUB(filestore_open_file_cnt, 1);
             }
@@ -271,7 +254,7 @@ static int OutputFilestoreLogger(ThreadVars *tv, void *thread_data,
             ff->fd = -1;
             SC_ATOMIC_SUB(filestore_open_file_cnt, 1);
         }
-        OutputFilestoreFinalizeFiles(tv, aft, ctx, p, ff, dir);
+        OutputFilestoreFinalizeFiles(tv, aft, ctx, p, ff, tx, tx_id, dir);
     }
 
     return 0;
@@ -299,7 +282,7 @@ static TmEcode OutputFilestoreLogThreadInit(ThreadVars *t, const void *initdata,
 
     /* File system type errors (open, write, rename) will only be
      * logged once. But this stat will be incremented for every
-     * occurence. */
+     * occurrence. */
     aft->fs_error_counter = StatsRegisterCounter("file_store.fs_errors", t);
 
     *data = (void *)aft;
@@ -352,9 +335,7 @@ static bool InitFilestoreDirectory(const char *dir)
     if (!SCPathExists(dir)) {
         SCLogInfo("Filestore (v2) creating directory %s", dir);
         if (SCCreateDirectoryTree(dir, true) != 0) {
-            SCLogError(SC_ERR_CREATE_DIRECTORY,
-                    "Filestore (v2) failed to create directory %s: %s", dir,
-                    strerror(errno));
+            SCLogError("Filestore (v2) failed to create directory %s: %s", dir, strerror(errno));
             return false;
         }
     }
@@ -363,17 +344,15 @@ static bool InitFilestoreDirectory(const char *dir)
         char leaf[PATH_MAX];
         int n = snprintf(leaf, sizeof(leaf), "%s/%02x", dir, i);
         if (n < 0 || n >= PATH_MAX) {
-            SCLogError(SC_ERR_CREATE_DIRECTORY,
-                    "Filestore (v2) failed to create leaf directory: "
-                    "path too long");
+            SCLogError("Filestore (v2) failed to create leaf directory: "
+                       "path too long");
             return false;
         }
         if (!SCPathExists(leaf)) {
             SCLogInfo("Filestore (v2) creating directory %s", leaf);
             if (SCDefaultMkDir(leaf) != 0) {
-                SCLogError(SC_ERR_CREATE_DIRECTORY,
-                        "Filestore (v2) failed to create directory %s: %s",
-                        leaf, strerror(errno));
+                SCLogError(
+                        "Filestore (v2) failed to create directory %s: %s", leaf, strerror(errno));
                 return false;
             }
         }
@@ -383,16 +362,13 @@ static bool InitFilestoreDirectory(const char *dir)
     char tmpdir[PATH_MAX];
     int n = snprintf(tmpdir, sizeof(tmpdir), "%s/tmp", dir);
     if (n < 0 || n >= PATH_MAX) {
-        SCLogError(SC_ERR_CREATE_DIRECTORY,
-                "Filestore (v2) failed to create tmp directory: path too long");
+        SCLogError("Filestore (v2) failed to create tmp directory: path too long");
         return false;
     }
     if (!SCPathExists(tmpdir)) {
         SCLogInfo("Filestore (v2) creating directory %s", tmpdir);
         if (SCDefaultMkDir(tmpdir) != 0) {
-            SCLogError(SC_ERR_CREATE_DIRECTORY,
-                    "Filestore (v2) failed to create directory %s: %s", tmpdir,
-                    strerror(errno));
+            SCLogError("Filestore (v2) failed to create directory %s: %s", tmpdir, strerror(errno));
             return false;
         }
     }
@@ -410,15 +386,13 @@ static OutputInitResult OutputFilestoreLogInitCtx(ConfNode *conf)
 
     intmax_t version = 0;
     if (!ConfGetChildValueInt(conf, "version", &version) || version < 2) {
-        SCLogWarning(SC_WARN_DEPRECATED,
-            "File-store v1 been removed. Please update to file-store v2.");
+        SCLogWarning("File-store v1 has been removed. Please update to file-store v2.");
         return result;
     }
 
     if (RunModeOutputFiledataEnabled()) {
-        SCLogWarning(SC_ERR_NOT_SUPPORTED,
-                "A file data logger is already enabled. Filestore (v2) "
-                "will not be enabled.");
+        SCLogWarning("A file data logger is already enabled. Filestore (v2) "
+                     "will not be enabled.");
         return result;
     }
 
@@ -437,7 +411,7 @@ static OutputInitResult OutputFilestoreLogInitCtx(ConfNode *conf)
     int written = snprintf(ctx->tmpdir, sizeof(ctx->tmpdir) - 1, "%s/tmp",
             log_directory);
     if (written == sizeof(ctx->tmpdir)) {
-        SCLogError(SC_ERR_SPRINTF, "File-store output directory overflow.");
+        SCLogError("File-store output directory overflow.");
         SCFree(ctx);
         return result;
     }
@@ -489,18 +463,18 @@ static OutputInitResult OutputFilestoreLogInitCtx(ConfNode *conf)
         uint32_t stream_depth = 0;
         if (ParseSizeStringU32(stream_depth_str,
                                &stream_depth) < 0) {
-            SCLogError(SC_ERR_SIZE_PARSE, "Error parsing "
+            SCLogError("Error parsing "
                        "file-store.stream-depth "
                        "from conf file - %s.  Killing engine",
-                       stream_depth_str);
+                    stream_depth_str);
             exit(EXIT_FAILURE);
         }
         if (stream_depth) {
             if (stream_depth <= stream_config.reassembly_depth) {
-                SCLogWarning(SC_WARN_FILESTORE_CONFIG,
-                           "file-store.stream-depth value %" PRIu32 " has "
-                           "no effect since it's less than stream.reassembly.depth "
-                           "value.", stream_depth);
+                SCLogWarning("file-store.stream-depth value %" PRIu32 " has "
+                             "no effect since it's less than stream.reassembly.depth "
+                             "value.",
+                        stream_depth);
             } else {
                 FileReassemblyDepthEnable(stream_depth);
             }
@@ -513,10 +487,10 @@ static OutputInitResult OutputFilestoreLogInitCtx(ConfNode *conf)
         uint32_t file_count = 0;
         if (ParseSizeStringU32(file_count_str,
                                &file_count) < 0) {
-            SCLogError(SC_ERR_SIZE_PARSE, "Error parsing "
+            SCLogError("Error parsing "
                        "file-store.max-open-files "
                        "from conf file - %s.  Killing engine",
-                       file_count_str);
+                    file_count_str);
             exit(EXIT_FAILURE);
         } else {
             if (file_count != 0) {
@@ -532,11 +506,8 @@ static OutputInitResult OutputFilestoreLogInitCtx(ConfNode *conf)
     SCReturnCT(result, "OutputInitResult");
 }
 
-#endif /* HAVE_NSS */
-
 void OutputFilestoreRegister(void)
 {
-#ifdef HAVE_NSS
     OutputRegisterFiledataModule(LOGGER_FILE_STORE, MODULE_NAME, "file-store",
             OutputFilestoreLogInitCtx, OutputFilestoreLogger,
             OutputFilestoreLogThreadInit, OutputFilestoreLogThreadDeinit,
@@ -544,12 +515,9 @@ void OutputFilestoreRegister(void)
 
     SC_ATOMIC_INIT(filestore_open_file_cnt);
     SC_ATOMIC_SET(filestore_open_file_cnt, 0);
-#endif
 }
 
 void OutputFilestoreRegisterGlobalCounters(void)
 {
-#ifdef HAVE_NSS
     StatsRegisterGlobalCounter("file_store.open_files", OutputFilestoreOpenFilesCounter);
-#endif
 }
