@@ -18,7 +18,9 @@
 #ifndef __OUTPUT_JSON_DNP3_OBJECTS_H__
 #define __OUTPUT_JSON_DNP3_OBJECTS_H__
 
-void OutputJsonDNP3SetItem(json_t *js, DNP3Object *object,
+#include "rust-bindings.h"
+
+void OutputJsonDNP3SetItem(JsonBuilder *js, DNP3Object *object,
     DNP3Point *item);
 
 #endif /* __OUTPUT_JSON_DNP3_OBJECTS_H__ */
