@@ -1,4 +1,4 @@
-/* Copyright (C) 2015-2020 Open Information Security Foundation
+/* Copyright (C) 2015-2021 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -16,7 +16,6 @@
  */
 
 #include "suricata-common.h"
-#include "debug.h"
 #include "detect.h"
 #include "pkt-var.h"
 #include "conf.h"
@@ -28,7 +27,6 @@
 #include "util-print.h"
 #include "util-unittest.h"
 #include "util-buffer.h"
-#include "util-crypt.h"
 #include "util-debug.h"
 
 #include "app-layer.h"
@@ -44,16 +42,14 @@
 #include "output-json-dnp3-objects.h"
 
 typedef struct LogDNP3FileCtx_ {
-    LogFileCtx *file_ctx;
     uint32_t    flags;
     uint8_t     include_object_data;
-    OutputJsonCommonSettings cfg;
+    OutputJsonCtx *eve_ctx;
 } LogDNP3FileCtx;
 
 typedef struct LogDNP3LogThread_ {
-    LogFileCtx *file_ctx;
     LogDNP3FileCtx *dnp3log_ctx;
-    MemBuffer      *buffer;
+    OutputJsonThreadCtx *ctx;
 } LogDNP3LogThread;
 
 static void JsonDNP3LogLinkControl(JsonBuilder *js, uint8_t lc)
@@ -67,9 +63,9 @@ static void JsonDNP3LogLinkControl(JsonBuilder *js, uint8_t lc)
 
 static void JsonDNP3LogIin(JsonBuilder *js, uint16_t iin)
 {
-    jb_open_array(js, "indicators");
-
     if (iin) {
+        jb_open_array(js, "indicators");
+
         int mapping = 0;
         do {
             if (iin & DNP3IndicatorsMap[mapping].value) {
@@ -77,8 +73,8 @@ static void JsonDNP3LogIin(JsonBuilder *js, uint16_t iin)
             }
             mapping++;
         } while (DNP3IndicatorsMap[mapping].name != NULL);
+        jb_close(js);
     }
-    jb_close(js);
 }
 
 static void JsonDNP3LogApplicationControl(JsonBuilder *js, uint8_t ac)
@@ -149,25 +145,27 @@ void JsonDNP3LogRequest(JsonBuilder *js, DNP3Transaction *dnp3tx)
     JB_SET_STRING(js, "type", "request");
 
     jb_open_object(js, "control");
-    JsonDNP3LogLinkControl(js, dnp3tx->request_lh.control);
+    JsonDNP3LogLinkControl(js, dnp3tx->lh.control);
     jb_close(js);
 
-    jb_set_uint(js, "src", DNP3_SWAP16(dnp3tx->request_lh.src));
-    jb_set_uint(js, "dst", DNP3_SWAP16(dnp3tx->request_lh.dst));
+    jb_set_uint(js, "src", DNP3_SWAP16(dnp3tx->lh.src));
+    jb_set_uint(js, "dst", DNP3_SWAP16(dnp3tx->lh.dst));
 
     jb_open_object(js, "application");
 
     jb_open_object(js, "control");
-    JsonDNP3LogApplicationControl(js, dnp3tx->request_ah.control);
+    JsonDNP3LogApplicationControl(js, dnp3tx->ah.control);
     jb_close(js);
 
-    jb_set_uint(js, "function_code", dnp3tx->request_ah.function_code);
+    jb_set_uint(js, "function_code", dnp3tx->ah.function_code);
 
-    jb_open_array(js, "objects");
-    JsonDNP3LogObjects(js, &dnp3tx->request_objects);
-    jb_close(js);
+    if (!TAILQ_EMPTY(&dnp3tx->objects)) {
+        jb_open_array(js, "objects");
+        JsonDNP3LogObjects(js, &dnp3tx->objects);
+        jb_close(js);
+    }
 
-    jb_set_bool(js, "complete", dnp3tx->request_complete);
+    jb_set_bool(js, "complete", dnp3tx->complete);
 
     /* Close application. */
     jb_close(js);
@@ -175,39 +173,40 @@ void JsonDNP3LogRequest(JsonBuilder *js, DNP3Transaction *dnp3tx)
 
 void JsonDNP3LogResponse(JsonBuilder *js, DNP3Transaction *dnp3tx)
 {
-    if (dnp3tx->response_ah.function_code == DNP3_APP_FC_UNSOLICITED_RESP) {
+    if (dnp3tx->ah.function_code == DNP3_APP_FC_UNSOLICITED_RESP) {
         JB_SET_STRING(js, "type", "unsolicited_response");
-    }
-    else {
+    } else {
         JB_SET_STRING(js, "type", "response");
     }
 
     jb_open_object(js, "control");
-    JsonDNP3LogLinkControl(js, dnp3tx->response_lh.control);
+    JsonDNP3LogLinkControl(js, dnp3tx->lh.control);
     jb_close(js);
 
-    jb_set_uint(js, "src", DNP3_SWAP16(dnp3tx->response_lh.src));
-    jb_set_uint(js, "dst", DNP3_SWAP16(dnp3tx->response_lh.dst));
+    jb_set_uint(js, "src", DNP3_SWAP16(dnp3tx->lh.src));
+    jb_set_uint(js, "dst", DNP3_SWAP16(dnp3tx->lh.dst));
 
     jb_open_object(js, "application");
 
     jb_open_object(js, "control");
-    JsonDNP3LogApplicationControl(js, dnp3tx->response_ah.control);
+    JsonDNP3LogApplicationControl(js, dnp3tx->ah.control);
     jb_close(js);
 
-    jb_set_uint(js, "function_code", dnp3tx->response_ah.function_code);
+    jb_set_uint(js, "function_code", dnp3tx->ah.function_code);
 
-    jb_open_array(js, "objects");
-    JsonDNP3LogObjects(js, &dnp3tx->response_objects);
-    jb_close(js);
+    if (!TAILQ_EMPTY(&dnp3tx->objects)) {
+        jb_open_array(js, "objects");
+        JsonDNP3LogObjects(js, &dnp3tx->objects);
+        jb_close(js);
+    }
 
-    jb_set_bool(js, "complete", dnp3tx->response_complete);
+    jb_set_bool(js, "complete", dnp3tx->complete);
 
     /* Close application. */
     jb_close(js);
 
     jb_open_object(js, "iin");
-    JsonDNP3LogIin(js, (uint16_t)(dnp3tx->response_iin.iin1 << 8 | dnp3tx->response_iin.iin2));
+    JsonDNP3LogIin(js, (uint16_t)(dnp3tx->iin.iin1 << 8 | dnp3tx->iin.iin2));
     jb_close(js);
 }
 
@@ -218,22 +217,17 @@ static int JsonDNP3LoggerToServer(ThreadVars *tv, void *thread_data,
     LogDNP3LogThread *thread = (LogDNP3LogThread *)thread_data;
     DNP3Transaction *tx = vtx;
 
-    MemBufferReset(thread->buffer);
-    if (tx->has_request && tx->request_done) {
-        JsonBuilder *js = CreateEveHeader(p, LOG_DIR_FLOW, "dnp3", NULL);
-        if (unlikely(js == NULL)) {
-            return TM_ECODE_OK;
-        }
-
-        EveAddCommonOptions(&thread->dnp3log_ctx->cfg, p, f, js);
-
-        jb_open_object(js, "dnp3");
-        JsonDNP3LogRequest(js, tx);
-        jb_close(js);
-        OutputJsonBuilderBuffer(js, thread->file_ctx, &thread->buffer);
-        jb_free(js);
+    JsonBuilder *js = CreateEveHeader(p, LOG_DIR_FLOW, "dnp3", NULL, thread->dnp3log_ctx->eve_ctx);
+    if (unlikely(js == NULL)) {
+        return TM_ECODE_OK;
     }
 
+    jb_open_object(js, "dnp3");
+    JsonDNP3LogRequest(js, tx);
+    jb_close(js);
+    OutputJsonBuilderBuffer(js, thread->ctx);
+    jb_free(js);
+
     SCReturnInt(TM_ECODE_OK);
 }
 
@@ -244,21 +238,30 @@ static int JsonDNP3LoggerToClient(ThreadVars *tv, void *thread_data,
     LogDNP3LogThread *thread = (LogDNP3LogThread *)thread_data;
     DNP3Transaction *tx = vtx;
 
-    MemBufferReset(thread->buffer);
-    if (tx->has_response && tx->response_done) {
-        JsonBuilder *js = CreateEveHeader(p, LOG_DIR_FLOW, "dnp3", NULL);
-        if (unlikely(js == NULL)) {
-            return TM_ECODE_OK;
-        }
-
-        EveAddCommonOptions(&thread->dnp3log_ctx->cfg, p, f, js);
-        jb_open_object(js, "dnp3");
-        JsonDNP3LogResponse(js, tx);
-        jb_close(js);
-        OutputJsonBuilderBuffer(js, thread->file_ctx, &thread->buffer);
-        jb_free(js);
+    JsonBuilder *js = CreateEveHeader(p, LOG_DIR_FLOW, "dnp3", NULL, thread->dnp3log_ctx->eve_ctx);
+    if (unlikely(js == NULL)) {
+        return TM_ECODE_OK;
     }
 
+    jb_open_object(js, "dnp3");
+    JsonDNP3LogResponse(js, tx);
+    jb_close(js);
+    OutputJsonBuilderBuffer(js, thread->ctx);
+    jb_free(js);
+
+    SCReturnInt(TM_ECODE_OK);
+}
+
+static int JsonDNP3Logger(ThreadVars *tv, void *thread_data, const Packet *p, Flow *f, void *state,
+        void *vtx, uint64_t tx_id)
+{
+    SCEnter();
+    DNP3Transaction *tx = vtx;
+    if (tx->is_request && tx->done) {
+        JsonDNP3LoggerToServer(tv, thread_data, p, f, state, vtx, tx_id);
+    } else if (!tx->is_request && tx->done) {
+        JsonDNP3LoggerToClient(tv, thread_data, p, f, state, vtx, tx_id);
+    }
     SCReturnInt(TM_ECODE_OK);
 }
 
@@ -281,8 +284,7 @@ static OutputInitResult OutputDNP3LogInitSub(ConfNode *conf, OutputCtx *parent_c
     if (unlikely(dnp3log_ctx == NULL)) {
         return result;
     }
-    dnp3log_ctx->file_ctx = json_ctx->file_ctx;
-    dnp3log_ctx->cfg = json_ctx->cfg;
+    dnp3log_ctx->eve_ctx = json_ctx;
 
     OutputCtx *output_ctx = SCCalloc(1, sizeof(*output_ctx));
     if (unlikely(output_ctx == NULL)) {
@@ -314,14 +316,9 @@ static TmEcode JsonDNP3LogThreadInit(ThreadVars *t, const void *initdata, void *
         goto error_exit;
     }
 
-    thread->buffer = MemBufferCreateNew(JSON_OUTPUT_BUFFER_SIZE);
-    if (unlikely(thread->buffer == NULL)) {
-        goto error_exit;
-    }
-
     thread->dnp3log_ctx = ((OutputCtx *)initdata)->data;
-    thread->file_ctx = LogFileEnsureExists(thread->dnp3log_ctx->file_ctx, t->id);
-    if (!thread->file_ctx) {
+    thread->ctx = CreateEveThreadCtx(t, thread->dnp3log_ctx->eve_ctx);
+    if (thread->ctx == NULL) {
         goto error_exit;
     }
 
@@ -330,9 +327,6 @@ static TmEcode JsonDNP3LogThreadInit(ThreadVars *t, const void *initdata, void *
     return TM_ECODE_OK;
 
 error_exit:
-    if (thread->buffer != NULL) {
-        MemBufferFree(thread->buffer);
-    }
     SCFree(thread);
     return TM_ECODE_FAILED;
 }
@@ -343,22 +337,14 @@ static TmEcode JsonDNP3LogThreadDeinit(ThreadVars *t, void *data)
     if (thread == NULL) {
         return TM_ECODE_OK;
     }
-    if (thread->buffer != NULL) {
-        MemBufferFree(thread->buffer);
-    }
+    FreeEveThreadCtx(thread->ctx);
     SCFree(thread);
     return TM_ECODE_OK;
 }
 
 void JsonDNP3LogRegister(void)
 {
-    /* Register direction aware eve sub-modules. */
-    OutputRegisterTxSubModuleWithProgress(LOGGER_JSON_DNP3_TS, "eve-log",
-        "JsonDNP3Log", "eve-log.dnp3", OutputDNP3LogInitSub, ALPROTO_DNP3,
-        JsonDNP3LoggerToServer, 0, 1, JsonDNP3LogThreadInit,
-        JsonDNP3LogThreadDeinit, NULL);
-    OutputRegisterTxSubModuleWithProgress(LOGGER_JSON_DNP3_TC, "eve-log",
-        "JsonDNP3Log", "eve-log.dnp3", OutputDNP3LogInitSub, ALPROTO_DNP3,
-        JsonDNP3LoggerToClient, 1, 1, JsonDNP3LogThreadInit,
-        JsonDNP3LogThreadDeinit, NULL);
+    OutputRegisterTxSubModule(LOGGER_JSON_TX, "eve-log", "JsonDNP3Log", "eve-log.dnp3",
+            OutputDNP3LogInitSub, ALPROTO_DNP3, JsonDNP3Logger, JsonDNP3LogThreadInit,
+            JsonDNP3LogThreadDeinit, NULL);
 }
