@@ -1,4 +1,4 @@
-/* Copyright (C) 2007-2020 Open Information Security Foundation
+/* Copyright (C) 2007-2021 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -24,7 +24,6 @@
  */
 
 #include "suricata-common.h"
-#include "debug.h"
 #include "detect.h"
 #include "pkt-var.h"
 #include "conf.h"
@@ -34,6 +33,7 @@
 #include "tm-threads.h"
 
 #include "util-print.h"
+#include "util-time.h"
 #include "util-unittest.h"
 
 #include "util-debug.h"
@@ -45,7 +45,6 @@
 #include "util-buffer.h"
 
 #include "util-logopenfile.h"
-#include "util-crypt.h"
 #include "util-ja3.h"
 
 #include "output-json.h"
@@ -74,41 +73,35 @@ SC_ATOMIC_EXTERN(unsigned int, cert_id);
 #define LOG_TLS_FIELD_SESSION_RESUMED   (1 << 10)
 #define LOG_TLS_FIELD_JA3               (1 << 11)
 #define LOG_TLS_FIELD_JA3S              (1 << 12)
+#define LOG_TLS_FIELD_CLIENT            (1 << 13) /**< client fields (issuer, subject, etc) */
+#define LOG_TLS_FIELD_CLIENT_CERT       (1 << 14)
+#define LOG_TLS_FIELD_CLIENT_CHAIN      (1 << 15)
 
 typedef struct {
     const char *name;
     uint64_t flag;
 } TlsFields;
 
-TlsFields tls_fields[] = {
-    { "version",         LOG_TLS_FIELD_VERSION },
-    { "subject",         LOG_TLS_FIELD_SUBJECT },
-    { "issuer",          LOG_TLS_FIELD_ISSUER },
-    { "serial",          LOG_TLS_FIELD_SERIAL },
-    { "fingerprint",     LOG_TLS_FIELD_FINGERPRINT },
-    { "not_before",      LOG_TLS_FIELD_NOTBEFORE },
-    { "not_after",       LOG_TLS_FIELD_NOTAFTER },
-    { "sni",             LOG_TLS_FIELD_SNI },
-    { "certificate",     LOG_TLS_FIELD_CERTIFICATE },
-    { "chain",           LOG_TLS_FIELD_CHAIN },
-    { "session_resumed", LOG_TLS_FIELD_SESSION_RESUMED },
-    { "ja3",             LOG_TLS_FIELD_JA3 },
-    { "ja3s",            LOG_TLS_FIELD_JA3S },
-    { NULL,              -1 }
-};
+TlsFields tls_fields[] = { { "version", LOG_TLS_FIELD_VERSION },
+    { "subject", LOG_TLS_FIELD_SUBJECT }, { "issuer", LOG_TLS_FIELD_ISSUER },
+    { "serial", LOG_TLS_FIELD_SERIAL }, { "fingerprint", LOG_TLS_FIELD_FINGERPRINT },
+    { "not_before", LOG_TLS_FIELD_NOTBEFORE }, { "not_after", LOG_TLS_FIELD_NOTAFTER },
+    { "sni", LOG_TLS_FIELD_SNI }, { "certificate", LOG_TLS_FIELD_CERTIFICATE },
+    { "chain", LOG_TLS_FIELD_CHAIN }, { "session_resumed", LOG_TLS_FIELD_SESSION_RESUMED },
+    { "ja3", LOG_TLS_FIELD_JA3 }, { "ja3s", LOG_TLS_FIELD_JA3S },
+    { "client", LOG_TLS_FIELD_CLIENT }, { "client_certificate", LOG_TLS_FIELD_CLIENT_CERT },
+    { "client_chain", LOG_TLS_FIELD_CLIENT_CHAIN }, { NULL, -1 } };
 
 typedef struct OutputTlsCtx_ {
-    LogFileCtx *file_ctx;
     uint32_t flags;  /** Store mode */
     uint64_t fields; /** Store fields */
-    OutputJsonCommonSettings cfg;
+    OutputJsonCtx *eve_ctx;
 } OutputTlsCtx;
 
 
 typedef struct JsonTlsLogThread_ {
-    LogFileCtx *file_ctx;
     OutputTlsCtx *tlslog_ctx;
-    MemBuffer *buffer;
+    OutputJsonThreadCtx *ctx;
 } JsonTlsLogThread;
 
 static void JsonTlsLogSubject(JsonBuilder *js, SSLState *ssl_state)
@@ -175,24 +168,14 @@ static void JsonTlsLogVersion(JsonBuilder *js, SSLState *ssl_state)
 static void JsonTlsLogNotBefore(JsonBuilder *js, SSLState *ssl_state)
 {
     if (ssl_state->server_connp.cert0_not_before != 0) {
-        char timebuf[64];
-        struct timeval tv;
-        tv.tv_sec = ssl_state->server_connp.cert0_not_before;
-        tv.tv_usec = 0;
-        CreateUtcIsoTimeString(&tv, timebuf, sizeof(timebuf));
-        jb_set_string(js, "notbefore", timebuf);
+        sc_x509_log_timestamp(js, "notbefore", ssl_state->server_connp.cert0_not_before);
     }
 }
 
 static void JsonTlsLogNotAfter(JsonBuilder *js, SSLState *ssl_state)
 {
     if (ssl_state->server_connp.cert0_not_after != 0) {
-        char timebuf[64];
-        struct timeval tv;
-        tv.tv_sec = ssl_state->server_connp.cert0_not_after;
-        tv.tv_usec = 0;
-        CreateUtcIsoTimeString(&tv, timebuf, sizeof(timebuf));
-        jb_set_string(js, "notafter", timebuf);
+        sc_x509_log_timestamp(js, "notafter", ssl_state->server_connp.cert0_not_after);
     }
 }
 
@@ -215,12 +198,16 @@ static void JsonTlsLogJa3String(JsonBuilder *js, SSLState *ssl_state)
 
 static void JsonTlsLogJa3(JsonBuilder *js, SSLState *ssl_state)
 {
-    jb_open_object(js, "ja3");
+    if ((ssl_state->client_connp.ja3_hash != NULL) ||
+            ((ssl_state->client_connp.ja3_str != NULL) &&
+                    ssl_state->client_connp.ja3_str->data != NULL)) {
+        jb_open_object(js, "ja3");
 
-    JsonTlsLogJa3Hash(js, ssl_state);
-    JsonTlsLogJa3String(js, ssl_state);
+        JsonTlsLogJa3Hash(js, ssl_state);
+        JsonTlsLogJa3String(js, ssl_state);
 
-    jb_close(js);
+        jb_close(js);
+    }
 }
 
 static void JsonTlsLogJa3SHash(JsonBuilder *js, SSLState *ssl_state)
@@ -242,54 +229,91 @@ static void JsonTlsLogJa3SString(JsonBuilder *js, SSLState *ssl_state)
 
 static void JsonTlsLogJa3S(JsonBuilder *js, SSLState *ssl_state)
 {
-    jb_open_object(js, "ja3s");
+    if ((ssl_state->server_connp.ja3_hash != NULL) ||
+            ((ssl_state->server_connp.ja3_str != NULL) &&
+                    ssl_state->server_connp.ja3_str->data != NULL)) {
+        jb_open_object(js, "ja3s");
 
-    JsonTlsLogJa3SHash(js, ssl_state);
-    JsonTlsLogJa3SString(js, ssl_state);
+        JsonTlsLogJa3SHash(js, ssl_state);
+        JsonTlsLogJa3SString(js, ssl_state);
 
-    jb_close(js);
+        jb_close(js);
+    }
 }
 
-static void JsonTlsLogCertificate(JsonBuilder *js, SSLState *ssl_state)
+static void JsonTlsLogCertificate(JsonBuilder *js, SSLStateConnp *connp)
 {
-    if (TAILQ_EMPTY(&ssl_state->server_connp.certs)) {
+    if (TAILQ_EMPTY(&connp->certs)) {
         return;
     }
 
-    SSLCertsChain *cert = TAILQ_FIRST(&ssl_state->server_connp.certs);
+    SSLCertsChain *cert = TAILQ_FIRST(&connp->certs);
     if (cert == NULL) {
         return;
     }
 
-    unsigned long len = BASE64_BUFFER_SIZE(cert->cert_len);
-    uint8_t encoded[len];
-    if (Base64Encode(cert->cert_data, cert->cert_len, encoded, &len) ==
-                     SC_BASE64_OK) {
-        jb_set_string(js, "certificate", (char *)encoded);
-    }
+    jb_set_base64(js, "certificate", cert->cert_data, cert->cert_len);
 }
 
-static void JsonTlsLogChain(JsonBuilder *js, SSLState *ssl_state)
+static void JsonTlsLogChain(JsonBuilder *js, SSLStateConnp *connp)
 {
-    if (TAILQ_EMPTY(&ssl_state->server_connp.certs)) {
+    if (TAILQ_EMPTY(&connp->certs)) {
         return;
     }
 
     jb_open_array(js, "chain");
 
     SSLCertsChain *cert;
-    TAILQ_FOREACH(cert, &ssl_state->server_connp.certs, next) {
-        unsigned long len = BASE64_BUFFER_SIZE(cert->cert_len);
-        uint8_t encoded[len];
-        if (Base64Encode(cert->cert_data, cert->cert_len, encoded, &len) ==
-                         SC_BASE64_OK) {
-            jb_append_string(js, (char *)encoded);
-        }
+    TAILQ_FOREACH (cert, &connp->certs, next) {
+        jb_append_base64(js, cert->cert_data, cert->cert_len);
     }
 
     jb_close(js);
 }
 
+static bool HasClientCert(SSLStateConnp *connp)
+{
+    if (connp->cert0_subject || connp->cert0_issuerdn)
+        return true;
+    return false;
+}
+
+static void JsonTlsLogClientCert(
+        JsonBuilder *js, SSLStateConnp *connp, const bool log_cert, const bool log_chain)
+{
+    if (connp->cert0_subject != NULL) {
+        jb_set_string(js, "subject", connp->cert0_subject);
+    }
+    if (connp->cert0_issuerdn != NULL) {
+        jb_set_string(js, "issuerdn", connp->cert0_issuerdn);
+    }
+    if (connp->cert0_fingerprint) {
+        jb_set_string(js, "fingerprint", connp->cert0_fingerprint);
+    }
+    if (connp->cert0_serial) {
+        jb_set_string(js, "serial", connp->cert0_serial);
+    }
+    if (connp->cert0_not_before != 0) {
+        char timebuf[64];
+        SCTime_t ts = SCTIME_FROM_SECS(connp->cert0_not_before);
+        CreateUtcIsoTimeString(ts, timebuf, sizeof(timebuf));
+        jb_set_string(js, "notbefore", timebuf);
+    }
+    if (connp->cert0_not_after != 0) {
+        char timebuf[64];
+        SCTime_t ts = SCTIME_FROM_SECS(connp->cert0_not_after);
+        CreateUtcIsoTimeString(ts, timebuf, sizeof(timebuf));
+        jb_set_string(js, "notafter", timebuf);
+    }
+
+    if (log_cert) {
+        JsonTlsLogCertificate(js, connp);
+    }
+    if (log_chain) {
+        JsonTlsLogChain(js, connp);
+    }
+}
+
 void JsonTlsLogJSONBasic(JsonBuilder *js, SSLState *ssl_state)
 {
     /* tls subject */
@@ -343,11 +367,11 @@ static void JsonTlsLogJSONCustom(OutputTlsCtx *tls_ctx, JsonBuilder *js,
 
     /* tls certificate */
     if (tls_ctx->fields & LOG_TLS_FIELD_CERTIFICATE)
-        JsonTlsLogCertificate(js, ssl_state);
+        JsonTlsLogCertificate(js, &ssl_state->server_connp);
 
     /* tls chain */
     if (tls_ctx->fields & LOG_TLS_FIELD_CHAIN)
-        JsonTlsLogChain(js, ssl_state);
+        JsonTlsLogChain(js, &ssl_state->server_connp);
 
     /* tls ja3_hash */
     if (tls_ctx->fields & LOG_TLS_FIELD_JA3)
@@ -356,6 +380,16 @@ static void JsonTlsLogJSONCustom(OutputTlsCtx *tls_ctx, JsonBuilder *js,
     /* tls ja3s */
     if (tls_ctx->fields & LOG_TLS_FIELD_JA3S)
         JsonTlsLogJa3S(js, ssl_state);
+
+    if (tls_ctx->fields & LOG_TLS_FIELD_CLIENT) {
+        const bool log_cert = (tls_ctx->fields & LOG_TLS_FIELD_CLIENT_CERT) != 0;
+        const bool log_chain = (tls_ctx->fields & LOG_TLS_FIELD_CLIENT_CHAIN) != 0;
+        if (HasClientCert(&ssl_state->client_connp)) {
+            jb_open_object(js, "client");
+            JsonTlsLogClientCert(js, &ssl_state->client_connp, log_cert, log_chain);
+            jb_close(js);
+        }
+    }
 }
 
 void JsonTlsLogJSONExtended(JsonBuilder *tjs, SSLState * state)
@@ -385,6 +419,12 @@ void JsonTlsLogJSONExtended(JsonBuilder *tjs, SSLState * state)
 
     /* tls ja3s */
     JsonTlsLogJa3S(tjs, state);
+
+    if (HasClientCert(&state->client_connp)) {
+        jb_open_object(tjs, "client");
+        JsonTlsLogClientCert(tjs, &state->client_connp, false, false);
+        jb_close(tjs);
+    }
 }
 
 static int JsonTlsLogger(ThreadVars *tv, void *thread_data, const Packet *p,
@@ -406,18 +446,13 @@ static int JsonTlsLogger(ThreadVars *tv, void *thread_data, const Packet *p,
         return 0;
     }
 
-    JsonBuilder *js = CreateEveHeader(p, LOG_DIR_FLOW, "tls", NULL);
+    JsonBuilder *js = CreateEveHeader(p, LOG_DIR_FLOW, "tls", NULL, aft->tlslog_ctx->eve_ctx);
     if (unlikely(js == NULL)) {
         return 0;
     }
 
-    EveAddCommonOptions(&tls_ctx->cfg, p, f, js);
-
     jb_open_object(js, "tls");
 
-    /* reset */
-    MemBufferReset(aft->buffer);
-
     /* log custom fields */
     if (tls_ctx->flags & LOG_TLS_CUSTOM) {
         JsonTlsLogJSONCustom(tls_ctx, js, ssl_state);
@@ -441,7 +476,7 @@ static int JsonTlsLogger(ThreadVars *tv, void *thread_data, const Packet *p,
     /* Close the tls object. */
     jb_close(js);
 
-    OutputJsonBuilderBuffer(js, aft->file_ctx, &aft->buffer);
+    OutputJsonBuilderBuffer(js, aft->ctx);
     jb_free(js);
 
     return 0;
@@ -459,25 +494,17 @@ static TmEcode JsonTlsLogThreadInit(ThreadVars *t, const void *initdata, void **
         goto error_exit;
     }
 
-    aft->buffer = MemBufferCreateNew(JSON_OUTPUT_BUFFER_SIZE);
-    if (aft->buffer == NULL) {
-        goto error_exit;
-    }
-
     /* use the Output Context (file pointer and mutex) */
     aft->tlslog_ctx = ((OutputCtx *)initdata)->data;
 
-    aft->file_ctx = LogFileEnsureExists(aft->tlslog_ctx->file_ctx, t->id);
-    if (!aft->file_ctx) {
+    aft->ctx = CreateEveThreadCtx(t, aft->tlslog_ctx->eve_ctx);
+    if (!aft->ctx) {
         goto error_exit;
     }
     *data = (void *)aft;
     return TM_ECODE_OK;
 
 error_exit:
-    if (aft->buffer != NULL) {
-        MemBufferFree(aft->buffer);
-    }
     SCFree(aft);
     return TM_ECODE_FAILED;
 }
@@ -489,7 +516,7 @@ static TmEcode JsonTlsLogThreadDeinit(ThreadVars *t, void *data)
         return TM_ECODE_OK;
     }
 
-    MemBufferFree(aft->buffer);
+    FreeEveThreadCtx(aft->ctx);
 
     /* clear memory */
     memset(aft, 0, sizeof(JsonTlsLogThread));
@@ -523,13 +550,19 @@ static OutputTlsCtx *OutputTlsInitCtx(ConfNode *conf)
         ConfNode *field;
         TAILQ_FOREACH(field, &custom->head, next)
         {
+            bool valid = false;
             TlsFields *valid_fields = tls_fields;
             for ( ; valid_fields->name != NULL; valid_fields++) {
                 if (strcasecmp(field->val, valid_fields->name) == 0) {
                     tls_ctx->fields |= valid_fields->flag;
+                    SCLogDebug("enabled %s", field->val);
+                    valid = true;
                     break;
                 }
             }
+            if (!valid) {
+                SCLogWarning("eve.tls: unknown 'custom' field '%s'", field->val);
+            }
         }
     }
 
@@ -547,11 +580,27 @@ static OutputTlsCtx *OutputTlsInitCtx(ConfNode *conf)
 
     if ((tls_ctx->fields & LOG_TLS_FIELD_CERTIFICATE) &&
             (tls_ctx->fields & LOG_TLS_FIELD_CHAIN)) {
-        SCLogWarning(SC_WARN_DUPLICATE_OUTPUT,
-                     "Both 'certificate' and 'chain' contains the top "
+        SCLogWarning("Both 'certificate' and 'chain' contains the top "
                      "certificate, so only one of them should be enabled "
                      "at a time");
     }
+    if ((tls_ctx->fields & LOG_TLS_FIELD_CLIENT_CERT) &&
+            (tls_ctx->fields & LOG_TLS_FIELD_CLIENT_CHAIN)) {
+        SCLogWarning("Both 'client_certificate' and 'client_chain' contains the top "
+                     "certificate, so only one of them should be enabled "
+                     "at a time");
+    }
+
+    if ((tls_ctx->fields & LOG_TLS_FIELD_CLIENT) == 0) {
+        if (tls_ctx->fields & LOG_TLS_FIELD_CLIENT_CERT) {
+            SCLogConfig("enabling \"client\" as a dependency of \"client_certificate\"");
+            tls_ctx->fields |= LOG_TLS_FIELD_CLIENT;
+        }
+        if (tls_ctx->fields & LOG_TLS_FIELD_CLIENT_CHAIN) {
+            SCLogConfig("enabling \"client\" as a dependency of \"client_chain\"");
+            tls_ctx->fields |= LOG_TLS_FIELD_CLIENT;
+        }
+    }
 
     return tls_ctx;
 }
@@ -578,13 +627,11 @@ static OutputInitResult OutputTlsLogInitSub(ConfNode *conf, OutputCtx *parent_ct
         return result;
     }
 
-    tls_ctx->file_ctx = ojc->file_ctx;
-    tls_ctx->cfg = ojc->cfg;
+    tls_ctx->eve_ctx = ojc;
 
     if ((tls_ctx->fields & LOG_TLS_FIELD_CERTIFICATE) &&
             (tls_ctx->fields & LOG_TLS_FIELD_CHAIN)) {
-        SCLogWarning(SC_WARN_DUPLICATE_OUTPUT,
-                     "Both 'certificate' and 'chain' contains the top "
+        SCLogWarning("Both 'certificate' and 'chain' contains the top "
                      "certificate, so only one of them should be enabled "
                      "at a time");
     }
@@ -602,8 +649,7 @@ static OutputInitResult OutputTlsLogInitSub(ConfNode *conf, OutputCtx *parent_ct
 void JsonTlsLogRegister (void)
 {
     /* register as child of eve-log */
-    OutputRegisterTxSubModuleWithProgress(LOGGER_JSON_TLS, "eve-log",
-        "JsonTlsLog", "eve-log.tls", OutputTlsLogInitSub, ALPROTO_TLS,
-        JsonTlsLogger, TLS_HANDSHAKE_DONE, TLS_HANDSHAKE_DONE,
-        JsonTlsLogThreadInit, JsonTlsLogThreadDeinit, NULL);
+    OutputRegisterTxSubModuleWithProgress(LOGGER_JSON_TX, "eve-log", "JsonTlsLog", "eve-log.tls",
+            OutputTlsLogInitSub, ALPROTO_TLS, JsonTlsLogger, TLS_HANDSHAKE_DONE, TLS_HANDSHAKE_DONE,
+            JsonTlsLogThreadInit, JsonTlsLogThreadDeinit, NULL);
 }
