@@ -1,4 +1,4 @@
-/* Copyright (C) 2007-2014 Open Information Security Foundation
+/* Copyright (C) 2007-2022 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -24,7 +24,6 @@
  */
 
 #include "suricata-common.h"
-#include "tm-modules.h"
 #include "output.h"
 #include "output-streaming.h"
 #include "app-layer.h"
@@ -38,17 +37,12 @@
 #include "stream-tcp-reassemble.h"
 #include "util-validate.h"
 
-typedef struct OutputLoggerThreadStore_ {
-    void *thread_data;
-    struct OutputLoggerThreadStore_ *next;
-} OutputLoggerThreadStore;
-
 /** per thread data for this module, contains a list of per thread
  *  data for the packet loggers. */
-typedef struct OutputLoggerThreadData_ {
+typedef struct OutputStreamingLoggerThreadData_ {
     OutputLoggerThreadStore *store;
     uint32_t loggers;
-} OutputLoggerThreadData;
+} OutputStreamingLoggerThreadData;
 
 /* logger instance, a module + a output ctx,
  * it's perfectly valid that have multiple instances of the same
@@ -164,11 +158,9 @@ static int HttpBodyIterator(Flow *f, int close, void *cbdata, uint8_t iflags)
     }
 
     const int tx_progress_done_value_ts =
-        AppLayerParserGetStateProgressCompletionStatus(ALPROTO_HTTP,
-                STREAM_TOSERVER);
+            AppLayerParserGetStateProgressCompletionStatus(ALPROTO_HTTP1, STREAM_TOSERVER);
     const int tx_progress_done_value_tc =
-        AppLayerParserGetStateProgressCompletionStatus(ALPROTO_HTTP,
-                STREAM_TOCLIENT);
+            AppLayerParserGetStateProgressCompletionStatus(ALPROTO_HTTP1, STREAM_TOCLIENT);
     const uint64_t total_txs = AppLayerParserGetTxCnt(f, f->alstate);
 
     uint64_t tx_id = 0;
@@ -181,10 +173,10 @@ static int HttpBodyIterator(Flow *f, int close, void *cbdata, uint8_t iflags)
         int tx_done = 0;
         int tx_logged = 0;
         int tx_progress_ts = AppLayerParserGetStateProgress(
-                IPPROTO_TCP, ALPROTO_HTTP, tx, FlowGetDisruptionFlags(f, STREAM_TOSERVER));
+                IPPROTO_TCP, ALPROTO_HTTP1, tx, FlowGetDisruptionFlags(f, STREAM_TOSERVER));
         if (tx_progress_ts >= tx_progress_done_value_ts) {
             int tx_progress_tc = AppLayerParserGetStateProgress(
-                    IPPROTO_TCP, ALPROTO_HTTP, tx, FlowGetDisruptionFlags(f, STREAM_TOCLIENT));
+                    IPPROTO_TCP, ALPROTO_HTTP1, tx, FlowGetDisruptionFlags(f, STREAM_TOCLIENT));
             if (tx_progress_tc >= tx_progress_done_value_tc) {
                 tx_done = 1;
             }
@@ -260,7 +252,8 @@ struct StreamLogData {
     Flow *f;
 };
 
-static int StreamLogFunc(void *cb_data, const uint8_t *data, const uint32_t data_len)
+static int StreamLogFunc(
+        void *cb_data, const uint8_t *data, const uint32_t data_len, const uint64_t _offset)
 {
     struct StreamLogData *log = cb_data;
 
@@ -306,7 +299,8 @@ static TmEcode OutputStreamingLog(ThreadVars *tv, Packet *p, void *thread_data)
         return TM_ECODE_OK;
     }
 
-    OutputLoggerThreadData *op_thread_data = (OutputLoggerThreadData *)thread_data;
+    OutputStreamingLoggerThreadData *op_thread_data =
+            (OutputStreamingLoggerThreadData *)thread_data;
     OutputStreamingLogger *logger = list;
     OutputLoggerThreadStore *store = op_thread_data->store;
 
@@ -351,7 +345,7 @@ static TmEcode OutputStreamingLog(ThreadVars *tv, Packet *p, void *thread_data)
         }
     }
     if (op_thread_data->loggers & (1<<STREAMING_HTTP_BODIES)) {
-        if (f->alproto == ALPROTO_HTTP && f->alstate != NULL) {
+        if (f->alproto == ALPROTO_HTTP1 && f->alstate != NULL) {
             int close = 0;
             TcpSession *ssn = f->protoctx;
             if (ssn) {
@@ -371,7 +365,7 @@ static TmEcode OutputStreamingLog(ThreadVars *tv, Packet *p, void *thread_data)
  *  This will run the thread init functions for the individual registered
  *  loggers */
 static TmEcode OutputStreamingLogThreadInit(ThreadVars *tv, const void *initdata, void **data) {
-    OutputLoggerThreadData *td = SCMalloc(sizeof(*td));
+    OutputStreamingLoggerThreadData *td = SCMalloc(sizeof(*td));
     if (td == NULL)
         return TM_ECODE_FAILED;
     memset(td, 0x00, sizeof(*td));
@@ -414,7 +408,8 @@ static TmEcode OutputStreamingLogThreadInit(ThreadVars *tv, const void *initdata
 }
 
 static TmEcode OutputStreamingLogThreadDeinit(ThreadVars *tv, void *thread_data) {
-    OutputLoggerThreadData *op_thread_data = (OutputLoggerThreadData *)thread_data;
+    OutputStreamingLoggerThreadData *op_thread_data =
+            (OutputStreamingLoggerThreadData *)thread_data;
     OutputLoggerThreadStore *store = op_thread_data->store;
     OutputStreamingLogger *logger = list;
 
@@ -434,7 +429,8 @@ static TmEcode OutputStreamingLogThreadDeinit(ThreadVars *tv, void *thread_data)
 }
 
 static void OutputStreamingLogExitPrintStats(ThreadVars *tv, void *thread_data) {
-    OutputLoggerThreadData *op_thread_data = (OutputLoggerThreadData *)thread_data;
+    OutputStreamingLoggerThreadData *op_thread_data =
+            (OutputStreamingLoggerThreadData *)thread_data;
     OutputLoggerThreadStore *store = op_thread_data->store;
     OutputStreamingLogger *logger = list;
 
