@@ -287,8 +287,17 @@ static void *ParseAFPConfig(const char *iface)
     }
 
     if (ConfGetChildValueWithDefault(if_root, if_default, "copy-iface", &out_iface) == 1) {
-        if (strlen(out_iface) > 0) {
-            aconf->out_iface = out_iface;
+        if (out_iface != NULL) {
+            if (strlen(out_iface) > 0) {
+                aconf->out_iface = out_iface;
+                if (strcmp(iface, out_iface) == 0) {
+                    FatalError(
+                            "Invalid config: interface (%s) and copy-iface (%s) can't be the same",
+                            iface, out_iface);
+                }
+            }
+        } else {
+            SCLogWarning("copy-iface corresponding to %s interface cannot be NULL", iface);
         }
     }
 
@@ -756,8 +765,6 @@ int RunModeIdsAFPAutoFp(void)
     int ret;
     const char *live_dev = NULL;
 
-    RunModeInitialize();
-
     TimeModeSetLive();
 
     (void)ConfGet("af-packet.live-interface", &live_dev);
@@ -795,7 +802,6 @@ int RunModeIdsAFPSingle(void)
     int ret;
     const char *live_dev = NULL;
 
-    RunModeInitialize();
     TimeModeSetLive();
 
     (void)ConfGet("af-packet.live-interface", &live_dev);
@@ -837,7 +843,6 @@ int RunModeIdsAFPWorkers(void)
     int ret;
     const char *live_dev = NULL;
 
-    RunModeInitialize();
     TimeModeSetLive();
 
     (void)ConfGet("af-packet.live-interface", &live_dev);
