@@ -41,11 +41,8 @@ void RunModeIdsPcapRegister(void)
     RunModeRegisterNewRunMode(RUNMODE_PCAP_DEV, "single", "Single threaded pcap live mode",
             RunModeIdsPcapSingle, NULL);
     RunModeRegisterNewRunMode(RUNMODE_PCAP_DEV, "autofp",
-            "Multi threaded pcap live mode.  Packets from "
-            "each flow are assigned to a single detect thread, "
-            "unlike \"pcap_live_auto\" where packets from "
-            "the same flow can be processed by any detect "
-            "thread",
+            "Multi-threaded pcap live mode. Packets from each flow are assigned to a consistent "
+            "detection thread",
             RunModeIdsPcapAutoFp, NULL);
     RunModeRegisterNewRunMode(RUNMODE_PCAP_DEV, "workers",
             "Workers pcap live mode, each thread does all"
