@@ -980,7 +980,7 @@ float threading_detect_ratio = 1;
 /**
  * Initialize multithreading settings.
  */
-void RunModeInitialize(void)
+void RunModeInitializeThreadSettings(void)
 {
     threading_set_cpu_affinity = FALSE;
     if ((ConfGetBool("threading.set-cpu-affinity", &threading_set_cpu_affinity)) == 0) {
@@ -1009,6 +1009,15 @@ void RunModeInitialize(void)
                 FatalError("Failed to initialize thread_stack_size output, invalid limit: %s", ss);
             }
         }
+    } else {
+        pthread_attr_t attr;
+        pthread_attr_init(&attr);
+        size_t size;
+        if (pthread_attr_getstacksize(&attr, &size) == 0 && size < 512 * 1024) {
+            threading_set_stack_size = 512 * 1024;
+            SCLogNotice("thread stack size of %" PRIuMAX " to too small: setting to 512k",
+                    (uintmax_t)size);
+        }
     }
 
     SCLogDebug("threading.stack-size %" PRIu64, threading_set_stack_size);
