@@ -1,4 +1,4 @@
-/* Copyright (C) 2011-2021 Open Information Security Foundation
+/* Copyright (C) 2011-2022 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -36,41 +36,24 @@
 
 #include "suricata.h"
 #include "suricata-common.h"
-#include "threads.h"
-#include "threadvars.h"
 #include "tm-threads.h"
-#include "conf.h"
+#include "packet.h"
 #include "util-bpf.h"
-#include "util-debug.h"
-#include "util-device.h"
-#include "util-error.h"
 #include "util-privs.h"
-#include "util-optimize.h"
-#include "util-checksum.h"
 #include "util-validate.h"
+#include "util-datalink.h"
 
-#include "tmqh-packetpool.h"
 #include "source-netmap.h"
-#include "runmodes.h"
 
 #ifdef HAVE_NETMAP
 
-#if HAVE_SYS_IOCTL_H
-#include <sys/ioctl.h>
-#endif
-
-#if HAVE_SYS_MMAN_H
-#include <sys/mman.h>
-#endif
-
 #define NETMAP_WITH_LIBS
 #ifdef DEBUG
 #define DEBUG_NETMAP_USER
 #endif
+
 #include <net/netmap_user.h>
-#if USE_NEW_NETMAP_API
 #include <libnetmap.h>
-#endif
 
 #endif /* HAVE_NETMAP */
 
@@ -83,9 +66,8 @@
 */
 static TmEcode NoNetmapSupportExit(ThreadVars *tv, const void *initdata, void **data)
 {
-    FatalError(SC_ERR_NO_NETMAP,
-            "Error creating thread %s: Netmap is not enabled. "
-            "Make sure to pass --enable-netmap to configure when building.",
+    FatalError("Error creating thread %s: Netmap is not enabled. "
+               "Make sure to pass --enable-netmap to configure when building.",
             tv->name);
 }
 
@@ -108,6 +90,8 @@ void TmModuleDecodeNetmapRegister (void)
 
 #else /* We have NETMAP support */
 
+#include "action-globals.h"
+
 #define POLL_TIMEOUT 100
 
 #if defined(__linux__)
@@ -129,11 +113,7 @@ enum { NETMAP_FLAG_ZERO_COPY = 1, NETMAP_FLAG_EXCL_RING_ACCESS = 2 };
  */
 typedef struct NetmapDevice_
 {
-#if USE_NEW_NETMAP_API
     struct nmport_d *nmd;
-#else
-    struct nm_desc *nmd;
-#endif
     unsigned int ref;
     SC_ATOMIC_DECLARE(unsigned int, threads_run);
     TAILQ_ENTRY(NetmapDevice_) next;
@@ -187,15 +167,10 @@ static SCMutex netmap_devlist_lock = SCMUTEX_INITIALIZER;
  */
 int NetmapGetRSSCount(const char *ifname)
 {
-#if USE_NEW_NETMAP_API
     struct nmreq_port_info_get req;
     struct nmreq_header hdr;
-#else
-    struct nmreq nm_req;
-#endif
     int rx_rings = 0;
 
-#if USE_NEW_NETMAP_API
     /* we need the base interface name to query queues */
     char base_name[IFNAMSIZ];
     strlcpy(base_name, ifname, sizeof(base_name));
@@ -203,49 +178,34 @@ int NetmapGetRSSCount(const char *ifname)
             (base_name[strlen(base_name) - 1] == '^' || base_name[strlen(base_name) - 1] == '*')) {
         base_name[strlen(base_name) - 1] = '\0';
     }
-#endif
+
     SCMutexLock(&netmap_devlist_lock);
 
     /* open netmap device */
     int fd = open("/dev/netmap", O_RDWR);
     if (fd == -1) {
-        SCLogError(SC_ERR_NETMAP_CREATE,
-                "Couldn't open netmap device, error %s",
-                strerror(errno));
+        SCLogError("%s: open netmap device failed: %s", ifname, strerror(errno));
         goto error_open;
     }
 
-    /* query netmap interface info */
-#if USE_NEW_NETMAP_API
+    /* query netmap interface info for ring count */
     memset(&req, 0, sizeof(req));
     memset(&hdr, 0, sizeof(hdr));
     hdr.nr_version = NETMAP_API;
     hdr.nr_reqtype = NETMAP_REQ_PORT_INFO_GET;
     hdr.nr_body = (uintptr_t)&req;
     strlcpy(hdr.nr_name, base_name, sizeof(hdr.nr_name));
-#else
-    memset(&nm_req, 0, sizeof(nm_req));
-    strlcpy(nm_req.nr_name, ifname, sizeof(nm_req.nr_name));
-    nm_req.nr_version = NETMAP_API;
-#endif
 
-#if USE_NEW_NETMAP_API
     if (ioctl(fd, NIOCCTRL, &hdr) != 0) {
-#else
-    if (ioctl(fd, NIOCGINFO, &nm_req) != 0) {
-#endif
-        SCLogError(SC_ERR_NETMAP_CREATE, "Couldn't query netmap for info about %s, error %s",
-                ifname, strerror(errno));
+        SCLogError(
+                "Query of netmap HW rings count on %s failed; error: %s", ifname, strerror(errno));
         goto error_fd;
     };
 
-#if USE_NEW_NETMAP_API
     /* return RX rings count if it equals TX rings count */
-    if (req.nr_rx_rings == req.nr_tx_rings)
+    if (req.nr_rx_rings == req.nr_tx_rings) {
         rx_rings = req.nr_rx_rings;
-#else
-    rx_rings = nm_req.nr_rx_rings;
-#endif
+    }
 
 error_fd:
     close(fd);
@@ -256,11 +216,7 @@ error_open:
 
 static void NetmapDestroyDevice(NetmapDevice *pdev)
 {
-#if USE_NEW_NETMAP_API
     nmport_close(pdev->nmd);
-#else
-    nm_close(pdev->nmd);
-#endif
     SCMutexDestroy(&pdev->netmap_dev_lock);
     SCFree(pdev);
 }
@@ -298,9 +254,13 @@ static void NetmapCloseAll(void)
 {
     NetmapDevice *pdev, *tmp;
 
+    SCMutexLock(&netmap_devlist_lock);
+
     TAILQ_FOREACH_SAFE (pdev, &netmap_devlist, next, tmp) {
         NetmapDestroyDevice(pdev);
     }
+
+    SCMutexUnlock(&netmap_devlist_lock);
 }
 
 /**
@@ -334,15 +294,14 @@ static int NetmapOpen(NetmapIfaceSettings *ns, NetmapDevice **pdevice, int verbo
         int if_flags = GetIfaceFlags(base_name);
         if (if_flags == -1) {
             if (verbose) {
-                SCLogError(SC_ERR_NETMAP_CREATE, "Cannot access network interface '%s' (%s)",
-                        base_name, ns->iface);
+                SCLogError("%s: cannot access network interface: %s", base_name, ns->iface);
             }
             goto error;
         }
 
         /* bring iface up if it is down */
         if ((if_flags & IFF_UP) == 0) {
-            SCLogError(SC_ERR_NETMAP_CREATE, "interface '%s' (%s) is down", base_name, ns->iface);
+            SCLogError("%s: interface is down", base_name);
             goto error;
         }
         /* if needed, try to set iface in promisc mode */
@@ -353,12 +312,11 @@ static int NetmapOpen(NetmapIfaceSettings *ns, NetmapDevice **pdevice, int verbo
         }
     }
     NetmapDevice *pdev = NULL, *spdev = NULL;
-    pdev = SCMalloc(sizeof(*pdev));
+    pdev = SCCalloc(1, sizeof(*pdev));
     if (unlikely(pdev == NULL)) {
-        SCLogError(SC_ERR_MEM_ALLOC, "Memory allocation failed");
+        SCLogError("%s: memory allocation failed", base_name);
         goto error;
     }
-    memset(pdev, 0, sizeof(*pdev));
     SC_ATOMIC_INIT(pdev->threads_run);
 
     SCMutexLock(&netmap_devlist_lock);
@@ -381,11 +339,6 @@ static int NetmapOpen(NetmapIfaceSettings *ns, NetmapDevice **pdevice, int verbo
     const char *opt_x = "x"; // not for IPS
     const char *opt_z = "z"; // zero copy, not for IPS
 
-#if NETMAP_API <= 11
-    // FreeBSD 11 doesn't have R and T.
-    opt_R = "";
-    opt_T = "";
-#endif
     /* assemble options string */
     char optstr[16];
     if (ns->ips)
@@ -430,23 +383,12 @@ retry:
                 ns->iface, ring, strlen(optstr) ? "/" : "", optstr);
     } else if (strlen(ns->iface) > 5 && strncmp(ns->iface, "vale", 4) == 0 && isdigit(ns->iface[4])) {
         snprintf(devname, sizeof(devname), "%s", ns->iface);
-#if NETMAP_API < 14 || !USE_NEW_NETMAP_API
-    } else if (ns->iface[strlen(ns->iface)-1] == '*' ||
-            ns->iface[strlen(ns->iface)-1] == '^') {
-        SCLogDebug("device with SW-ring enabled (ns->iface): %s",ns->iface);
-        snprintf(devname, sizeof(devname), "netmap:%s", ns->iface);
-        SCLogDebug("device with SW-ring enabled (devname): %s",devname);
-#endif
     } else if (ring == 0 && ns->threads == 1) {
         /* just a single thread and ring, so don't use ring param */
         snprintf(devname, sizeof(devname), "netmap:%s%s%s",
                 ns->iface, strlen(optstr) ? "/" : "", optstr);
         SCLogDebug("device with %s-ring enabled (devname): %s", soft ? "SW" : "HW", devname);
     } else {
-#if !USE_NEW_NETMAP_API
-        snprintf(devname, sizeof(devname), "netmap:%s-%d%s%s", ns->iface, ring,
-                strlen(optstr) ? "/" : "", optstr);
-#else
         /* Going to be using multiple threads and rings */
         if (ns->sw_ring) {
             /* Opening a host stack interface */
@@ -474,12 +416,10 @@ retry:
                     strlen(optstr) ? "/" : "", optstr);
             SCLogDebug("device with HW-ring enabled (devname): %s", devname);
         }
-#endif
     }
 
     strlcpy(pdev->ifname, ns->iface, sizeof(pdev->ifname));
 
-#if USE_NEW_NETMAP_API
     /* have the netmap API parse device name and prepare the port descriptor for us */
     pdev->nmd = nmport_prepare(devname);
 
@@ -497,36 +437,33 @@ retry:
             pdev->nmd = NULL;
         }
     }
-#else
-    pdev->nmd = nm_open(devname, NULL, 0, NULL);
-#endif
 
     if (pdev->nmd == NULL) {
         if (errno == EINVAL) {
             if (opt_z[0] == 'z') {
-                SCLogNotice("got '%s' EINVAL: going to retry without 'z'", devname);
+                SCLogNotice(
+                        "%s: dev '%s' got EINVAL: going to retry without 'z'", base_name, devname);
                 opt_z = "";
                 goto retry;
             } else if (opt_x[0] == 'x') {
-                SCLogNotice("dev '%s' got EINVAL: going to retry without 'x'", devname);
+                SCLogNotice(
+                        "%s: dev '%s' got EINVAL: going to retry without 'x'", base_name, devname);
                 opt_x = "";
                 goto retry;
             }
         }
 
         NetmapCloseAll();
-        FatalError(SC_ERR_FATAL, "opening devname %s failed: %s", devname, strerror(errno));
+        FatalError("opening devname %s failed: %s", devname, strerror(errno));
     }
 
-#if USE_NEW_NETMAP_API
     /* Work around bug in libnetmap library where "cur_{r,t}x_ring" values not initialized */
     SCLogDebug("%s -- cur rings: [%d, %d] first rings: [%d, %d]", devname, pdev->nmd->cur_rx_ring,
             pdev->nmd->cur_tx_ring, pdev->nmd->first_rx_ring, pdev->nmd->first_tx_ring);
     pdev->nmd->cur_rx_ring = pdev->nmd->first_rx_ring;
     pdev->nmd->cur_tx_ring = pdev->nmd->first_tx_ring;
-#endif
 
-    SCLogInfo("devname [fd: %d] %s %s opened", pdev->nmd->fd, devname, ns->iface);
+    SCLogInfo("%s: %s opened [fd: %d]", devname, ns->iface, pdev->nmd->fd);
 
     pdev->direction = direction;
     pdev->ring = ring;
@@ -564,30 +501,29 @@ static inline void NetmapDumpCounters(NetmapThreadVars *ntv)
 static TmEcode ReceiveNetmapThreadInit(ThreadVars *tv, const void *initdata, void **data)
 {
     SCEnter();
-    NetmapIfaceConfig *aconf = (NetmapIfaceConfig *)initdata;
 
+    NetmapIfaceConfig *aconf = (NetmapIfaceConfig *)initdata;
     if (initdata == NULL) {
-        SCLogError(SC_ERR_INVALID_ARGUMENT, "initdata == NULL");
+        SCLogError("initdata == NULL");
         SCReturnInt(TM_ECODE_FAILED);
     }
 
-    NetmapThreadVars *ntv = SCMalloc(sizeof(*ntv));
+    NetmapThreadVars *ntv = SCCalloc(1, sizeof(*ntv));
     if (unlikely(ntv == NULL)) {
-        SCLogError(SC_ERR_MEM_ALLOC, "Memory allocation failed");
+        SCLogError("Memory allocation failed");
         goto error;
     }
-    memset(ntv, 0, sizeof(*ntv));
+
+    ntv->livedev = LiveGetDevice(aconf->iface_name);
+    if (ntv->livedev == NULL) {
+        SCLogError("Unable to find Live device");
+        goto error_ntv;
+    }
 
     ntv->tv = tv;
     ntv->checksum_mode = aconf->in.checksum_mode;
     ntv->copy_mode = aconf->in.copy_mode;
 
-    ntv->livedev = LiveGetDevice(aconf->iface_name);
-    if (ntv->livedev == NULL) {
-        SCLogError(SC_ERR_INVALID_VALUE, "Unable to find Live device");
-        goto error_ntv;
-    }
-
     /* enable zero-copy mode for workers runmode */
     char const *active_runmode = RunmodeGetActive();
     if (strcmp("workers", active_runmode) == 0) {
@@ -604,20 +540,10 @@ static TmEcode ReceiveNetmapThreadInit(ThreadVars *tv, const void *initdata, voi
         goto error_ntv;
     }
 
-#if !USE_NEW_NETMAP_API
-    if (unlikely(aconf->in.sw_ring && aconf->in.threads > 1)) {
-        SCLogError(SC_ERR_INVALID_VALUE,
-                "Interface '%s^'. "
-                "Thread count can't be greater than 1 for SW ring.",
-                aconf->iface_name);
-        goto error_src;
-    }
-#endif
-
     if (aconf->in.copy_mode != NETMAP_COPY_MODE_NONE) {
         if (NetmapOpen(&aconf->out, &ntv->ifdst, 1, 0, (ntv->flags & NETMAP_FLAG_ZERO_COPY) != 0,
                     soft) != 0) {
-            goto error_dst;
+            goto error_src;
         }
     }
 
@@ -628,8 +554,7 @@ static TmEcode ReceiveNetmapThreadInit(ThreadVars *tv, const void *initdata, voi
             ntv->tv);
 
     if (aconf->in.bpf_filter) {
-        SCLogConfig("Using BPF '%s' on iface '%s'",
-                  aconf->in.bpf_filter, ntv->ifsrc->ifname);
+        SCLogConfig("%s: using BPF '%s'", ntv->ifsrc->ifname, aconf->in.bpf_filter);
         char errbuf[PCAP_ERRBUF_SIZE];
         if (SCBPFCompile(default_packet_size,  /* snaplen_arg */
                     LINKTYPE_ETHERNET,    /* linktype_arg */
@@ -640,28 +565,31 @@ static TmEcode ReceiveNetmapThreadInit(ThreadVars *tv, const void *initdata, voi
                     errbuf,
                     sizeof(errbuf)) == -1)
         {
-            SCLogError(SC_ERR_NETMAP_CREATE, "Failed to compile BPF \"%s\": %s",
-                   aconf->in.bpf_filter,
-                   errbuf);
+            SCLogError("%s: failed to compile BPF \"%s\": %s", ntv->ifsrc->ifname,
+                    aconf->in.bpf_filter, errbuf);
             goto error_dst;
         }
     }
 
     SCLogDebug("thread: %s polling on fd: %d", tv->name, ntv->ifsrc->nmd->fd);
 
+    DatalinkSetGlobalType(LINKTYPE_ETHERNET);
+
     *data = (void *)ntv;
     aconf->DerefFunc(aconf);
     SCReturnInt(TM_ECODE_OK);
+
 error_dst:
     if (aconf->in.copy_mode != NETMAP_COPY_MODE_NONE) {
         NetmapClose(ntv->ifdst);
     }
-#if !USE_NEW_NETMAP_API
+
 error_src:
-#endif
     NetmapClose(ntv->ifsrc);
+
 error_ntv:
     SCFree(ntv);
+
 error:
     aconf->DerefFunc(aconf);
     SCReturnInt(TM_ECODE_FAILED);
@@ -675,7 +603,7 @@ error:
 static TmEcode NetmapWritePacket(NetmapThreadVars *ntv, Packet *p)
 {
     if (ntv->copy_mode == NETMAP_COPY_MODE_IPS) {
-        if (PACKET_TEST_ACTION(p, ACTION_DROP)) {
+        if (PacketCheckAction(p, ACTION_DROP)) {
             return TM_ECODE_OK;
         }
     }
@@ -686,12 +614,21 @@ static TmEcode NetmapWritePacket(NetmapThreadVars *ntv, Packet *p)
         SCMutexLock(&ntv->ifdst->netmap_dev_lock);
     }
 
+    int write_tries = 0;
+try_write:
     /* attempt to write the packet into the netmap ring buffer(s) */
-#if USE_NEW_NETMAP_API
     if (nmport_inject(ntv->ifdst->nmd, GET_PKT_DATA(p), GET_PKT_LEN(p)) == 0) {
-#else
-    if (nm_inject(ntv->ifdst->nmd, GET_PKT_DATA(p), GET_PKT_LEN(p)) == 0) {
-#endif
+
+        /* writing the packet failed, but ask kernel to sync TX rings
+         * for us as the ring buffers may simply be full */
+        (void)ioctl(ntv->ifdst->nmd->fd, NIOCTXSYNC, 0);
+
+        /* Try write up to 2 more times before giving up */
+        if (write_tries < 3) {
+            write_tries++;
+            goto try_write;
+        }
+
         if (ntv->flags & NETMAP_FLAG_EXCL_RING_ACCESS) {
             SCMutexUnlock(&ntv->ifdst->netmap_dev_lock);
         }
@@ -703,7 +640,8 @@ static TmEcode NetmapWritePacket(NetmapThreadVars *ntv, Packet *p)
     SCLogDebug("sent successfully: %s(%d)->%s(%d) (%u)", ntv->ifsrc->ifname, ntv->ifsrc->ring,
             ntv->ifdst->ifname, ntv->ifdst->ring, GET_PKT_LEN(p));
 
-    ioctl(ntv->ifdst->nmd->fd, NIOCTXSYNC, 0);
+    /* Instruct netmap to push the data on the TX ring on the destination port */
+    (void)ioctl(ntv->ifdst->nmd->fd, NIOCTXSYNC, 0);
     if (ntv->flags & NETMAP_FLAG_EXCL_RING_ACCESS) {
         SCMutexUnlock(&ntv->ifdst->netmap_dev_lock);
     }
@@ -725,21 +663,11 @@ static void NetmapReleasePacket(Packet *p)
     PacketFreeOrRelease(p);
 }
 
-#if USE_NEW_NETMAP_API
 static void NetmapProcessPacket(NetmapThreadVars *ntv, const struct nm_pkthdr *ph)
-#else
-static void NetmapCallback(u_char *user, const struct nm_pkthdr *ph, const u_char *d)
-#endif
 {
-
-#if USE_NEW_NETMAP_API
-    const u_char *d = ph->buf;
-#else
-    NetmapThreadVars *ntv = (NetmapThreadVars *)user;
-#endif
     if (ntv->bpf_prog.bf_len) {
         struct pcap_pkthdr pkthdr = { {0, 0}, ph->len, ph->len };
-        if (pcap_offline_filter(&ntv->bpf_prog, &pkthdr, d) == 0) {
+        if (pcap_offline_filter(&ntv->bpf_prog, &pkthdr, ph->buf) == 0) {
             return;
         }
     }
@@ -752,17 +680,17 @@ static void NetmapCallback(u_char *user, const struct nm_pkthdr *ph, const u_cha
     PKT_SET_SRC(p, PKT_SRC_WIRE);
     p->livedev = ntv->livedev;
     p->datalink = LINKTYPE_ETHERNET;
-    p->ts = ph->ts;
+    p->ts = SCTIME_FROM_TIMEVAL(&ph->ts);
     ntv->pkts++;
     ntv->bytes += ph->len;
 
     if (ntv->flags & NETMAP_FLAG_ZERO_COPY) {
-        if (PacketSetData(p, (uint8_t *)d, ph->len) == -1) {
+        if (PacketSetData(p, (uint8_t *)ph->buf, ph->len) == -1) {
             TmqhOutputPacketpool(ntv->tv, p);
             return;
         }
     } else {
-        if (PacketCopyData(p, (uint8_t *)d, ph->len) == -1) {
+        if (PacketCopyData(p, (uint8_t *)ph->buf, ph->len) == -1) {
             TmqhOutputPacketpool(ntv->tv, p);
             return;
         }
@@ -783,7 +711,6 @@ static void NetmapCallback(u_char *user, const struct nm_pkthdr *ph, const u_cha
  * \param cnt int count of packets to read (-1 = all).
  * \param *ntv NetmapThreadVars.
  */
-#if USE_NEW_NETMAP_API
 static TmEcode NetmapReadPackets(struct nmport_d *d, int cnt, NetmapThreadVars *ntv)
 {
     struct nm_pkthdr hdr;
@@ -854,7 +781,6 @@ static TmEcode NetmapReadPackets(struct nmport_d *d, int cnt, NetmapThreadVars *
     }
     return got;
 }
-#endif
 
 /**
  *  \brief Main netmap reading loop function
@@ -872,6 +798,11 @@ static TmEcode ReceiveNetmapLoop(ThreadVars *tv, void *data, void *slot)
     fds.events = POLLIN;
 
     SCLogDebug("thread %s polling on %d", tv->name, fds.fd);
+
+    // Indicate that the thread is actually running its application level code (i.e., it can poll
+    // packets)
+    TmThreadsSetFlag(tv, THV_RUNNING);
+
     for(;;) {
         if (unlikely(suricata_ctl_flags != 0)) {
             break;
@@ -885,16 +816,11 @@ static TmEcode ReceiveNetmapLoop(ThreadVars *tv, void *data, void *slot)
         if (r < 0) {
             /* error */
             if (errno != EINTR)
-                SCLogError(SC_ERR_NETMAP_READ,
-                        "Error polling netmap from iface '%s': (%d" PRIu32 ") %s",
-                        ntv->ifsrc->ifname, errno, strerror(errno));
+                SCLogError("%s: error polling netmap: %s", ntv->ifsrc->ifname, strerror(errno));
             continue;
 
         } else if (r == 0) {
             /* no events, timeout */
-            // SCLogDebug("(%s:%d-%d) Poll timeout", ntv->ifsrc->ifname,
-            //           ntv->src_ring_from, ntv->src_ring_to);
-
             /* sync counters */
             NetmapDumpCounters(ntv);
             StatsSyncCountersIfSignalled(tv);
@@ -906,25 +832,17 @@ static TmEcode ReceiveNetmapLoop(ThreadVars *tv, void *data, void *slot)
 
         if (unlikely(fds.revents & POLL_EVENTS)) {
             if (fds.revents & POLLERR) {
-                SCLogError(SC_ERR_NETMAP_READ,
-                        "Error reading netmap data via polling from iface '%s': (%d" PRIu32 ") %s",
-                        ntv->ifsrc->ifname, errno, strerror(errno));
-                // SCLogError(SC_ERR_NETMAP_READ,
-                //        "Error reading data from iface '%s': (%d" PRIu32 ") %s",
-                //        ntv->ifsrc->ifname, errno, strerror(errno));
+                SCLogError("%s: error reading netmap data via polling: %s", ntv->ifsrc->ifname,
+                        strerror(errno));
             } else if (fds.revents & POLLNVAL) {
-                SCLogError(SC_ERR_NETMAP_READ, "Invalid polling request");
+                SCLogError("%s: invalid polling request", ntv->ifsrc->ifname);
             }
             continue;
         }
 
         if (likely(fds.revents & POLLIN)) {
-#if USE_NEW_NETMAP_API
             /* have data on RX ring, so copy to Packet for processing */
             NetmapReadPackets(ntv->ifsrc->nmd, -1, ntv);
-#else
-            nm_dispatch(ntv->ifsrc->nmd, -1, NetmapCallback, (void *)ntv);
-#endif
         }
 
         NetmapDumpCounters(ntv);
@@ -947,11 +865,10 @@ static void ReceiveNetmapThreadExitStats(ThreadVars *tv, void *data)
     NetmapThreadVars *ntv = (NetmapThreadVars *)data;
 
     NetmapDumpCounters(ntv);
-    SCLogPerf("(%s) Kernel: Packets %" PRIu64 ", dropped %" PRIu64 ", bytes %" PRIu64 "",
-              tv->name,
-              StatsGetLocalCounterValue(tv, ntv->capture_kernel_packets),
-              StatsGetLocalCounterValue(tv, ntv->capture_kernel_drops),
-              ntv->bytes);
+    SCLogPerf("%s: (%s) packets %" PRIu64 ", dropped %" PRIu64 ", bytes %" PRIu64 "",
+            ntv->ifsrc->ifname, tv->name,
+            StatsGetLocalCounterValue(tv, ntv->capture_kernel_packets),
+            StatsGetLocalCounterValue(tv, ntv->capture_kernel_drops), ntv->bytes);
 }
 
 /**
