@@ -95,7 +95,6 @@ typedef struct WinDivertThreadVars_ {
     WinDivertHandle filter_handle;
 
     int thread_num;
-    CaptureStats stats;
     int64_t qpc_start_time;
     int64_t qpc_start_count;
     int64_t qpc_freq_usec;
@@ -233,8 +232,10 @@ static SCTime_t WinDivertTimestampToTimeStamp(WinDivertThreadVars *wd_tv, INT64
     struct timeval tv;
 
     int64_t qpc_delta = (int64_t)timestamp_count - wd_tv->qpc_start_count;
-    int64_t unix_usec =
-            wd_tv->qpc_start_time + (qpc_delta / wd_tv->qpc_freq_usec);
+    int64_t unix_usec = wd_tv->qpc_start_time;
+    if (wd_tv->qpc_freq_usec) {
+        unix_usec += qpc_delta / wd_tv->qpc_freq_usec;
+    }
 
     tv.tv_sec = (long)(unix_usec / (1000 * 1000));
     tv.tv_usec = (long)(unix_usec - (int64_t)tv.tv_sec * (1000 * 1000));
@@ -749,9 +750,6 @@ static TmEcode WinDivertVerdictHelper(ThreadVars *tv, Packet *p)
     SCEnter();
     WinDivertThreadVars *wd_tv = WinDivertGetThread(p->windivert_v.thread_num);
 
-    /* update counters */
-    CaptureStatsUpdate(tv, &wd_tv->stats, p);
-
 #ifdef COUNTERS
     WinDivertQueueVars *wd_qv = WinDivertGetQueue(wd_tv->thread_num);
 #endif /* COUNTERS */
@@ -822,9 +820,6 @@ TmEcode VerdictWinDivertThreadInit(ThreadVars *tv, const void *initdata,
     SCEnter();
 
     WinDivertThreadVars *wd_tv = (WinDivertThreadVars *)initdata;
-
-    CaptureStatsSetup(tv, &wd_tv->stats);
-
     *data = wd_tv;
 
     SCReturnInt(TM_ECODE_OK);
