@@ -52,7 +52,11 @@ typedef enum {
     TMM_RECEIVEERFDAG,
     TMM_DECODEERFDAG,
     TMM_RECEIVEAFP,
+    TMM_RECEIVEAFXDP,
     TMM_DECODEAFP,
+    TMM_DECODEAFXDP,
+    TMM_RECEIVEDPDK,
+    TMM_DECODEDPDK,
     TMM_RECEIVENETMAP,
     TMM_DECODENETMAP,
     TMM_ALERTPCAPINFO,
