@@ -78,12 +78,12 @@ static void *SCHInfoAllocUserDataOSPolicy(const char *host_os)
     int *user_data = NULL;
 
     if ( (user_data = SCMalloc(sizeof(int))) == NULL) {
-        FatalError(SC_ERR_FATAL, "Error allocating memory. Exiting");
+        FatalError("Error allocating memory. Exiting");
     }
 
     /* the host os flavour that has to be sent as user data */
     if ( (*user_data = SCMapEnumNameToValue(host_os, sc_hinfo_os_policy_map)) == -1) {
-        SCLogError(SC_ERR_INVALID_ENUM_MAP, "Invalid enum map inside "
+        SCLogError("Invalid enum map inside "
                    "SCHInfoAddHostOSInfo()");
         SCFree(user_data);
         return NULL;
@@ -126,13 +126,13 @@ int SCHInfoAddHostOSInfo(const char *host_os, const char *host_os_ip_range, int
     struct in_addr *ipv4_addr = NULL;
     struct in6_addr *ipv6_addr = NULL;
     char *netmask_str = NULL;
-    int netmask_value = 0;
+    uint8_t netmask_value = 0;
     int *user_data = NULL;
-    char recursive = FALSE;
+    bool recursive = false;
 
     if (host_os == NULL || host_os_ip_range == NULL ||
             strlen(host_os_ip_range) == 0) {
-        SCLogError(SC_ERR_INVALID_ARGUMENT, "Invalid arguments");
+        SCLogError("Invalid arguments");
         return -1;
     }
 
@@ -142,7 +142,7 @@ int SCHInfoAddHostOSInfo(const char *host_os, const char *host_os_ip_range, int
 
     /* the host os flavour that has to be sent as user data */
     if ( (user_data = SCHInfoAllocUserDataOSPolicy(host_os)) == NULL) {
-        SCLogError(SC_ERR_INVALID_ENUM_MAP, "Invalid enum map inside");
+        SCLogError("Invalid enum map inside");
         return -1;
     }
 
@@ -156,14 +156,14 @@ int SCHInfoAddHostOSInfo(const char *host_os, const char *host_os_ip_range, int
     }
 
     if ( (ip_str = SCStrdup(host_os_ip_range)) == NULL) {
-        FatalError(SC_ERR_FATAL, "Error allocating memory");
+        FatalError("Error allocating memory");
     }
 
     /* check if we have more addresses in the host_os_ip_range */
     if ((ip_str_rem = strchr(ip_str, ',')) != NULL) {
         ip_str_rem[0] = '\0';
         ip_str_rem++;
-        recursive = TRUE;
+        recursive = true;
     }
 
     /* check if we have received a netblock */
@@ -175,7 +175,7 @@ int SCHInfoAddHostOSInfo(const char *host_os, const char *host_os_ip_range, int
     if (strchr(ip_str, ':') == NULL) {
         /* if we are here, we have an IPV4 address */
         if ( (ipv4_addr = ValidateIPV4Address(ip_str)) == NULL) {
-            SCLogError(SC_ERR_INVALID_IPV4_ADDR, "Invalid IPV4 address");
+            SCLogError("Invalid IPV4 address");
             SCHInfoFreeUserDataOSPolicy(user_data);
             SCFree(ip_str);
             return -1;
@@ -185,8 +185,9 @@ int SCHInfoAddHostOSInfo(const char *host_os, const char *host_os_ip_range, int
             SCRadixAddKeyIPV4((uint8_t *)ipv4_addr, sc_hinfo_tree,
                               (void *)user_data);
         } else {
-            if (StringParseI32RangeCheck(&netmask_value, 10, 0, (const char *)netmask_str, 0, 32) < 0) {
-                SCLogError(SC_ERR_INVALID_IP_NETBLOCK, "Invalid IPV4 Netblock");
+            if (StringParseU8RangeCheck(&netmask_value, 10, 0, (const char *)netmask_str, 0, 32) <
+                    0) {
+                SCLogError("Invalid IPV4 Netblock");
                 SCHInfoFreeUserDataOSPolicy(user_data);
                 SCFree(ipv4_addr);
                 SCFree(ip_str);
@@ -200,7 +201,7 @@ int SCHInfoAddHostOSInfo(const char *host_os, const char *host_os_ip_range, int
     } else {
         /* if we are here, we have an IPV6 address */
         if ( (ipv6_addr = ValidateIPV6Address(ip_str)) == NULL) {
-            SCLogError(SC_ERR_INVALID_IPV6_ADDR, "Invalid IPV6 address inside");
+            SCLogError("Invalid IPV6 address inside");
             SCHInfoFreeUserDataOSPolicy(user_data);
             SCFree(ip_str);
             return -1;
@@ -210,8 +211,9 @@ int SCHInfoAddHostOSInfo(const char *host_os, const char *host_os_ip_range, int
             SCRadixAddKeyIPV6((uint8_t *)ipv6_addr, sc_hinfo_tree,
                               (void *)user_data);
         } else {
-            if (StringParseI32RangeCheck(&netmask_value, 10, 0, (const char *)netmask_str, 0, 128) < 0) {
-                SCLogError(SC_ERR_INVALID_IP_NETBLOCK, "Invalid IPV6 Netblock");
+            if (StringParseU8RangeCheck(&netmask_value, 10, 0, (const char *)netmask_str, 0, 128) <
+                    0) {
+                SCLogError("Invalid IPV6 Netblock");
                 SCHInfoFreeUserDataOSPolicy(user_data);
                 SCFree(ipv6_addr);
                 SCFree(ip_str);
@@ -224,7 +226,7 @@ int SCHInfoAddHostOSInfo(const char *host_os, const char *host_os_ip_range, int
         }
     }
 
-    if (recursive == TRUE) {
+    if (recursive) {
         SCHInfoAddHostOSInfo(host_os, ip_str_rem, is_ipv4);
     }
 
@@ -254,7 +256,7 @@ int SCHInfoGetHostOSFlavour(const char *ip_addr_str)
 
     if (strchr(ip_addr_str, ':') != NULL) {
         if ( (ipv6_addr = ValidateIPV6Address(ip_addr_str)) == NULL) {
-            SCLogError(SC_ERR_INVALID_IPV4_ADDR, "Invalid IPV4 address");
+            SCLogError("Invalid IPV4 address");
             return -1;
         }
 
@@ -266,7 +268,7 @@ int SCHInfoGetHostOSFlavour(const char *ip_addr_str)
             return *((int *)user_data);
     } else {
         if ( (ipv4_addr = ValidateIPV4Address(ip_addr_str)) == NULL) {
-            SCLogError(SC_ERR_INVALID_IPV4_ADDR, "Invalid IPV4 address");
+            SCLogError("Invalid IPV4 address");
             return -1;
         }
 
@@ -344,9 +346,9 @@ void SCHInfoLoadFromConfig(void)
             if (host->val != NULL && strchr(host->val, ':') != NULL)
                 is_ipv4 = 0;
             if (SCHInfoAddHostOSInfo(policy->name, host->val, is_ipv4) == -1) {
-                SCLogError(SC_ERR_INVALID_ARGUMENT,
-                    "Failed to add host \"%s\" with policy \"%s\" to host "
-                    "info database", host->val, policy->name);
+                SCLogError("Failed to add host \"%s\" with policy \"%s\" to host "
+                           "info database",
+                        host->val, policy->name);
                 exit(EXIT_FAILURE);
             }
         }
@@ -1137,6 +1139,9 @@ static int SCHInfoTestValidIPV6Address08(void)
                              SC_HINFO_IS_IPV6) == -1) {
         goto end;
     }
+    if (SCHInfoAddHostOSInfo("vista", "8.8.8.0/24", SC_HINFO_IS_IPV4) == -1) {
+        goto end;
+    }
     if (SCHInfoAddHostOSInfo("irix", "default", SC_HINFO_IS_IPV6) == -1) {
         goto end;
     }
@@ -1227,7 +1232,10 @@ static int SCHInfoTestValidIPV6Address08(void)
         SCMapEnumNameToValue("irix", sc_hinfo_os_policy_map)) {
         goto end;
     }
-
+    if (SCHInfoGetHostOSFlavour("8.8.8.8") !=
+            SCMapEnumNameToValue("vista", sc_hinfo_os_policy_map)) {
+        goto end;
+    }
     result = 1;
 
  end:
