@@ -1,5 +1,5 @@
 /* vi: set et ts=4: */
-/* Copyright (C) 2007-2016 Open Information Security Foundation
+/* Copyright (C) 2007-2021 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -27,6 +27,7 @@
 #include "util-log-redis.h"
 #include "util-logopenfile.h"
 #include "util-byte.h"
+#include "util-debug.h"
 
 #ifdef HAVE_LIBHIREDIS
 
@@ -53,13 +54,13 @@ void SCLogRedisInit(void)
 #endif /* HAVE_LIBEVENT_PTHREADS */
 }
 
-/** \brief SCLogRedisContextAlloc() - Allocates and initalizes redis context
+/** \brief SCLogRedisContextAlloc() - Allocates and initializes redis context
  */
 static SCLogRedisContext *SCLogRedisContextAlloc(void)
 {
     SCLogRedisContext* ctx = (SCLogRedisContext*) SCCalloc(1, sizeof(SCLogRedisContext));
     if (ctx == NULL) {
-        FatalError(SC_ERR_FATAL, "Unable to allocate redis context");
+        FatalError("Unable to allocate redis context");
     }
     ctx->sync = NULL;
 #if HAVE_LIBEVENT
@@ -78,13 +79,13 @@ static SCLogRedisContext *SCLogRedisContextAlloc(void)
 static int SCConfLogReopenAsyncRedis(LogFileCtx *log_ctx);
 #include <hiredis/adapters/libevent.h>
 
-/** \brief SCLogRedisAsyncContextAlloc() - Allocates and initalizes redis context with async
+/** \brief SCLogRedisAsyncContextAlloc() - Allocates and initializes redis context with async
  */
 static SCLogRedisContext *SCLogRedisContextAsyncAlloc(void)
 {
     SCLogRedisContext* ctx = (SCLogRedisContext*) SCCalloc(1, sizeof(SCLogRedisContext));
     if (unlikely(ctx == NULL)) {
-        FatalError(SC_ERR_FATAL, "Unable to allocate redis context");
+        FatalError("Unable to allocate redis context");
     }
 
     ctx->sync = NULL;
@@ -101,7 +102,7 @@ static SCLogRedisContext *SCLogRedisContextAsyncAlloc(void)
 /** \brief SCRedisAsyncCommandCallback() Callback when reply from redis happens.
  *  \param ac redis async context
  *  \param r redis reply
- *  \param privvata opaque datq with pointer to LogFileCtx
+ *  \param privdata opaque data with pointer to LogFileCtx
  */
 static void SCRedisAsyncCommandCallback(redisAsyncContext *ac, void *r, void *privdata)
 {
@@ -123,7 +124,7 @@ static void SCRedisAsyncCommandCallback(redisAsyncContext *ac, void *r, void *pr
  *         This is used to check if redis is connected.
  *  \param ac redis async context
  *  \param r redis reply
- *  \param privvata opaque datq with pointer to LogFileCtx
+ *  \param privdata opaque data with pointer to LogFileCtx
  */
 static void SCRedisAsyncEchoCommandCallback(redisAsyncContext *ac, void *r, void *privdata)
 {
@@ -139,7 +140,7 @@ static void SCRedisAsyncEchoCommandCallback(redisAsyncContext *ac, void *r, void
     } else {
        ctx->connected = 0;
        if (ctx->tried == 0) {
-          SCLogWarning(SC_ERR_SOCKET, "Failed to connect to Redis... (will keep trying)");
+           SCLogWarning("Failed to connect to Redis... (will keep trying)");
        }
        ctx->tried = time(NULL);
     }
@@ -161,7 +162,7 @@ static void SCLogAsyncRedisSendEcho(SCLogRedisContext * ctx)
  *         This is used to terminate connection with redis.
  *  \param ac redis async context
  *  \param r redis reply
- *  \param privvata opaque datq with pointer to LogFileCtx
+ *  \param privdata opaque data with pointer to LogFileCtx
  */
 static void SCRedisAsyncQuitCommandCallback(redisAsyncContext *ac, void *r, void *privdata)
 {
@@ -210,16 +211,17 @@ static int SCConfLogReopenAsyncRedis(LogFileCtx *log_ctx)
 
     if (ctx->ev_base != NULL) {
         event_base_free(ctx->ev_base);
+        ctx->ev_base = NULL;
     }
 
     if (ctx->async == NULL) {
-        SCLogError(SC_ERR_MEM_ALLOC, "Error allocate redis async.");
+        SCLogError("Error allocate redis async.");
         ctx->tried = time(NULL);
         return -1;
     }
 
     if (ctx->async != NULL && ctx->async->err) {
-        SCLogError(SC_ERR_SOCKET, "Error setting to redis async: [%s].", ctx->async->errstr);
+        SCLogError("Error setting to redis async: [%s].", ctx->async->errstr);
         ctx->tried = time(NULL);
         return -1;
     }
@@ -307,12 +309,12 @@ static int SCConfLogReopenSyncRedis(LogFileCtx *log_ctx)
         ctx->sync = redisConnectUnix(redis_server);
     }
     if (ctx->sync == NULL) {
-        SCLogError(SC_ERR_SOCKET, "Error connecting to redis server.");
+        SCLogError("Error connecting to redis server.");
         ctx->tried = time(NULL);
         return -1;
     }
     if (ctx->sync->err) {
-        SCLogError(SC_ERR_SOCKET, "Error connecting to redis server: [%s].", ctx->sync->errstr);
+        SCLogError("Error connecting to redis server: [%s].", ctx->sync->errstr);
         redisFree(ctx->sync);
         ctx->sync = NULL;
         ctx->tried = time(NULL);
@@ -385,9 +387,7 @@ static int SCLogRedisWriteSync(LogFileCtx *file_ctx, const char *string)
                             }
                             break;
                         default:
-                            SCLogWarning(SC_ERR_INVALID_VALUE,
-                                    "Unsupported error code %d",
-                                    redis->err);
+                            SCLogWarning("Unsupported error code %d", redis->err);
                             return -1;
                     }
                 }
@@ -404,7 +404,7 @@ static int SCLogRedisWriteSync(LogFileCtx *file_ctx, const char *string)
         if (reply) {
             switch (reply->type) {
                 case REDIS_REPLY_ERROR:
-                    SCLogWarning(SC_ERR_SOCKET, "Redis error: %s", reply->str);
+                    SCLogWarning("Redis error: %s", reply->str);
                     SCConfLogReopenSyncRedis(file_ctx);
                     break;
                 case REDIS_REPLY_INTEGER:
@@ -412,8 +412,7 @@ static int SCLogRedisWriteSync(LogFileCtx *file_ctx, const char *string)
                     ret = 0;
                     break;
                 default:
-                    SCLogError(SC_ERR_INVALID_VALUE,
-                            "Redis default triggered with %d", reply->type);
+                    SCLogError("Redis default triggered with %d", reply->type);
                     SCConfLogReopenSyncRedis(file_ctx);
                     break;
             }
@@ -430,7 +429,7 @@ static int SCLogRedisWriteSync(LogFileCtx *file_ctx, const char *string)
  * \param log_ctx Log file context allocated by caller
  * \param string buffer with data to write
  * \param string_len data length
- * \retval 0 on sucess;
+ * \retval 0 on success;
  * \retval -1 on failure;
  */
 int LogFileWriteRedis(void *lf_ctx, const char *string, size_t string_len)
@@ -462,6 +461,10 @@ int SCConfLogOpenRedis(ConfNode *redis_node, void *lf_ctx)
 {
     LogFileCtx *log_ctx = lf_ctx;
 
+    if (log_ctx->threaded) {
+        FatalError("redis does not support threaded output");
+    }
+
     const char *redis_port = NULL;
     const char *redis_mode = NULL;
 
@@ -490,7 +493,7 @@ int SCConfLogOpenRedis(ConfNode *redis_node, void *lf_ctx)
 
 #ifndef HAVE_LIBEVENT
     if (is_async) {
-        SCLogWarning(SC_ERR_NO_REDIS_ASYNC, "async option not available.");
+        SCLogWarning("async option not available.");
     }
     is_async = 0;
 #endif //ifndef HAVE_LIBEVENT
@@ -524,15 +527,15 @@ int SCConfLogOpenRedis(ConfNode *redis_node, void *lf_ctx)
     } else if(!strcmp(redis_mode,"channel") || !strcmp(redis_mode,"publish")) {
         log_ctx->redis_setup.command = redis_publish_cmd;
     } else {
-        FatalError(SC_ERR_FATAL, "Invalid redis mode");
+        FatalError("Invalid redis mode");
     }
 
     /* store server params for reconnection */
     if (!log_ctx->redis_setup.server) {
-        FatalError(SC_ERR_FATAL, "Error allocating redis server string");
+        FatalError("Error allocating redis server string");
     }
     if (StringParseUint16(&log_ctx->redis_setup.port, 10, 0, (const char *)redis_port) < 0) {
-        FatalError(SC_ERR_INVALID_VALUE, "Invalid value for redis port: %s", redis_port);
+        FatalError("Invalid value for redis port: %s", redis_port);
     }
     log_ctx->Close = SCLogFileCloseRedis;
 
