@@ -33,6 +33,7 @@
 #include "tm-threads.h"
 #include "util-napatech.h"
 #include "source-napatech.h"
+#include "runmode-napatech.h"
 
 #ifdef NAPATECH_ENABLE_BYPASS
 
@@ -642,7 +643,7 @@ static void *NapatechStatsLoop(void *arg)
                      "active streams.");
     }
 
-    TmThreadsSetFlag(tv, THV_INIT_DONE);
+    TmThreadsSetFlag(tv, THV_INIT_DONE | THV_RUNNING);
     while (1) {
         if (TmThreadsCheckFlag(tv, THV_KILL)) {
             SCLogDebug("NapatechStatsLoop THV_KILL detected");
@@ -1026,7 +1027,7 @@ static void *NapatechBufMonitorLoop(void *arg)
         exit(EXIT_FAILURE);
     }
 
-    TmThreadsSetFlag(tv, THV_INIT_DONE);
+    TmThreadsSetFlag(tv, THV_INIT_DONE | THV_RUNNING);
     while (1) {
         if (TmThreadsCheckFlag(tv, THV_KILL)) {
             SCLogDebug("NapatechBufMonitorLoop THV_KILL detected");
@@ -1216,7 +1217,6 @@ void NapatechStartStats(void)
         FatalError("Failed to spawn thread for NapatechBufMonitor - Killing engine.");
     }
 
-
     return;
 }
 
@@ -1409,8 +1409,6 @@ uint32_t NapatechSetupTraffic(uint32_t first_stream, uint32_t last_stream)
 #ifdef NAPATECH_ENABLE_BYPASS
     if (NapatechUseHWBypass()) {
         SCLogInfo("Napatech Hardware Bypass enabled.");
-    } else {
-        SCLogInfo("Napatech Hardware Bypass available but disabled.");
     }
 #else
     if (NapatechUseHWBypass()) {
@@ -1454,8 +1452,16 @@ uint32_t NapatechSetupTraffic(uint32_t first_stream, uint32_t last_stream)
             if (strchr(port->val, '-')) {
                 stream_spec = CONFIG_SPECIFIER_RANGE;
 
-                ByteExtractStringUint8(&ports_spec.first[iteration], 10, 0, port->val);
-                ByteExtractStringUint8(&ports_spec.second[iteration], 10, 0, strchr(port->val, '-')+1);
+                if (ByteExtractStringUint8(&ports_spec.first[iteration], 10, 0, port->val) == -1) {
+                    FatalError("Invalid value '%s' in napatech.ports specification in conf file.",
+                            port->val);
+                }
+
+                if (ByteExtractStringUint8(&ports_spec.second[iteration], 10, 0,
+                            strchr(port->val, '-') + 1) == -1) {
+                    FatalError("Invalid value '%s' in napatech.ports specification in conf file.",
+                            port->val);
+                }
 
                 if (ports_spec.first[iteration] == ports_spec.second[iteration]) {
                     if (is_inline) {
@@ -1533,8 +1539,17 @@ uint32_t NapatechSetupTraffic(uint32_t first_stream, uint32_t last_stream)
                 }
                 stream_spec = CONFIG_SPECIFIER_RANGE;
 
-                ByteExtractStringUint8(&ports_spec.first[iteration], 10, 0, port->val);
-                ByteExtractStringUint8(&ports_spec.second[iteration], 10, 0, strchr(port->val, '-') + 1);
+                if (ByteExtractStringUint8(&ports_spec.first[iteration], 10, 0, port->val) == -1) {
+                    FatalError("Invalid value '%s' in napatech.ports specification in conf file.",
+                            port->val);
+                }
+
+                if (ByteExtractStringUint8(&ports_spec.second[iteration], 10, 0,
+                            strchr(port->val, '-') + 1) == -1) {
+                    FatalError("Invalid value '%s' in napatech.ports specification in conf file.",
+                            port->val);
+                }
+
                 snprintf(ports_spec.str, sizeof (ports_spec.str), "(%d..%d)", ports_spec.first[iteration], ports_spec.second[iteration]);
             } else {
                 /* check that the sting in the config file is correctly specified */
@@ -1544,7 +1559,10 @@ uint32_t NapatechSetupTraffic(uint32_t first_stream, uint32_t last_stream)
                 }
                 stream_spec = CONFIG_SPECIFIER_INDIVIDUAL;
 
-                ByteExtractStringUint8(&ports_spec.first[iteration], 10, 0, port->val);
+                if (ByteExtractStringUint8(&ports_spec.first[iteration], 10, 0, port->val) == -1) {
+                    FatalError("Invalid value '%s' in napatech.ports specification in conf file.",
+                            port->val);
+                }
 
                 /* Determine the ports to use on the NTPL assign statement*/
                 if (iteration == 0) {
