@@ -33,11 +33,12 @@
 #include "util-byte.h"
 
 #include "util-hash-lookup3.h"
+#include "util-validate.h"
 
 static THashData *THashGetUsed(THashTableContext *ctx);
 static void THashDataEnqueue (THashDataQueue *q, THashData *h);
 
-static void THashDataMoveToSpare(THashTableContext *ctx, THashData *h)
+void THashDataMoveToSpare(THashTableContext *ctx, THashData *h)
 {
     THashDataEnqueue(&ctx->spare_q, h);
     (void) SC_ATOMIC_SUB(ctx->counter, 1);
@@ -56,7 +57,7 @@ THashDataQueue *THashDataQueueNew(void)
 {
     THashDataQueue *q = (THashDataQueue *)SCMalloc(sizeof(THashDataQueue));
     if (q == NULL) {
-        SCLogError(SC_ERR_FATAL, "Fatal error encountered in THashDataQueueNew. Exiting...");
+        SCLogError("Fatal error encountered in THashDataQueueNew. Exiting...");
         exit(EXIT_SUCCESS);
     }
     q = THashDataQueueInit(q);
@@ -186,6 +187,8 @@ error:
 static void THashDataFree(THashTableContext *ctx, THashData *h)
 {
     if (h != NULL) {
+        DEBUG_VALIDATE_BUG_ON(SC_ATOMIC_GET(h->use_cnt) != 0);
+
         if (h->data != NULL) {
             ctx->config.DataFree(h->data);
         }
@@ -219,17 +222,16 @@ static int THashInitConfig(THashTableContext *ctx, const char *cnf_prefix)
     if ((ConfGet(varname, &conf_val)) == 1)
     {
         if (ParseSizeStringU64(conf_val, &ctx->config.memcap) < 0) {
-            SCLogError(SC_ERR_SIZE_PARSE, "Error parsing %s "
+            SCLogError("Error parsing %s "
                        "from conf file - %s.  Killing engine",
-                       varname, conf_val);
+                    varname, conf_val);
             return -1;
         }
     }
     GET_VAR(cnf_prefix, "hash-size");
     if ((ConfGet(varname, &conf_val)) == 1)
     {
-        if (StringParseUint32(&configval, 10, strlen(conf_val),
-                                    conf_val) > 0) {
+        if (StringParseUint32(&configval, 10, (uint16_t)strlen(conf_val), conf_val) > 0) {
             ctx->config.hash_size = configval;
         }
     }
@@ -237,8 +239,7 @@ static int THashInitConfig(THashTableContext *ctx, const char *cnf_prefix)
     GET_VAR(cnf_prefix, "prealloc");
     if ((ConfGet(varname, &conf_val)) == 1)
     {
-        if (StringParseUint32(&configval, 10, strlen(conf_val),
-                                    conf_val) > 0) {
+        if (StringParseUint32(&configval, 10, (uint16_t)strlen(conf_val), conf_val) > 0) {
             ctx->config.prealloc = configval;
         } else {
             WarnInvalidConfEntry(varname, "%"PRIu32, ctx->config.prealloc);
@@ -248,17 +249,17 @@ static int THashInitConfig(THashTableContext *ctx, const char *cnf_prefix)
     /* alloc hash memory */
     uint64_t hash_size = ctx->config.hash_size * sizeof(THashHashRow);
     if (!(THASH_CHECK_MEMCAP(ctx, hash_size))) {
-        SCLogError(SC_ERR_THASH_INIT, "allocating hash failed: "
-                "max hash memcap is smaller than projected hash size. "
-                "Memcap: %"PRIu64", Hash table size %"PRIu64". Calculate "
-                "total hash size by multiplying \"hash-size\" with %"PRIuMAX", "
-                "which is the hash bucket size.", ctx->config.memcap, hash_size,
-                (uintmax_t)sizeof(THashHashRow));
+        SCLogError("allocating hash failed: "
+                   "max hash memcap is smaller than projected hash size. "
+                   "Memcap: %" PRIu64 ", Hash table size %" PRIu64 ". Calculate "
+                   "total hash size by multiplying \"hash-size\" with %" PRIuMAX ", "
+                   "which is the hash bucket size.",
+                ctx->config.memcap, hash_size, (uintmax_t)sizeof(THashHashRow));
         return -1;
     }
     ctx->array = SCMallocAligned(ctx->config.hash_size * sizeof(THashHashRow), CLS);
     if (unlikely(ctx->array == NULL)) {
-        SCLogError(SC_ERR_THASH_INIT, "Fatal error encountered in THashInitConfig. Exiting...");
+        SCLogError("Fatal error encountered in THashInitConfig. Exiting...");
         return -1;
     }
     memset(ctx->array, 0, ctx->config.hash_size * sizeof(THashHashRow));
@@ -272,16 +273,17 @@ static int THashInitConfig(THashTableContext *ctx, const char *cnf_prefix)
     /* pre allocate prealloc */
     for (i = 0; i < ctx->config.prealloc; i++) {
         if (!(THASH_CHECK_MEMCAP(ctx, THASH_DATA_SIZE(ctx)))) {
-            SCLogError(SC_ERR_THASH_INIT, "preallocating data failed: "
-                    "max thash memcap reached. Memcap %"PRIu64", "
-                    "Memuse %"PRIu64".", ctx->config.memcap,
+            SCLogError("preallocating data failed: "
+                       "max thash memcap reached. Memcap %" PRIu64 ", "
+                       "Memuse %" PRIu64 ".",
+                    ctx->config.memcap,
                     ((uint64_t)SC_ATOMIC_GET(ctx->memuse) + THASH_DATA_SIZE(ctx)));
             return -1;
         }
 
         THashData *h = THashDataAlloc(ctx);
         if (h == NULL) {
-            SCLogError(SC_ERR_THASH_INIT, "preallocating data failed: %s", strerror(errno));
+            SCLogError("preallocating data failed: %s", strerror(errno));
             return -1;
         }
         THashDataEnqueue(&ctx->spare_q,h);
@@ -336,7 +338,7 @@ THashTableContext *THashInit(const char *cnf_prefix, size_t data_size,
  * */
 void THashConsolidateMemcap(THashTableContext *ctx)
 {
-    ctx->config.memcap = MAX(SC_ATOMIC_GET(ctx->memuse), THASH_DEFAULT_MEMCAP);
+    ctx->config.memcap = MAX(SC_ATOMIC_GET(ctx->memuse), ctx->config.memcap);
     SCLogDebug("memcap after load set to: %" PRIu64, ctx->config.memcap);
 }
 
@@ -345,7 +347,6 @@ void THashConsolidateMemcap(THashTableContext *ctx)
 void THashShutdown(THashTableContext *ctx)
 {
     THashData *h;
-    uint32_t u;
 
     /* free spare queue */
     while ((h = THashDataDequeue(&ctx->spare_q))) {
@@ -355,7 +356,7 @@ void THashShutdown(THashTableContext *ctx)
 
     /* clear and free the hash */
     if (ctx->array != NULL) {
-        for (u = 0; u < ctx->config.hash_size; u++) {
+        for (uint32_t u = 0; u < ctx->config.hash_size; u++) {
             h = ctx->array[u].head;
             while (h) {
                 THashData *n = h->next;
@@ -466,7 +467,7 @@ static uint32_t THashGetKey(const THashConfig *cnf, void *data)
 
 static inline int THashCompare(const THashConfig *cnf, void *a, void *b)
 {
-    if (cnf->DataCompare(a, b) == TRUE)
+    if (cnf->DataCompare(a, b))
         return 1;
     return 0;
 }
@@ -510,7 +511,7 @@ static THashData *THashDataGetNew(THashTableContext *ctx, void *data)
     } else {
         /* data has been recycled before it went into the spare queue */
 
-        /* data is initialized (recylced) but *unlocked* */
+        /* data is initialized (recycled) but *unlocked* */
     }
 
     // setup the data
